/* Generated by Cython 0.29.32 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "linux/ftd2xx.h",
            "linux/libft4222.h"
        ],
        "include_dirs": [
            "linux"
        ],
        "libraries": [
            "ft4222"
        ],
        "library_dirs": [
            "linux/build-x86_64"
        ],
        "name": "ft4222.ft4222",
        "runtime_library_dirs": [
            "$ORIGIN/."
        ],
        "sources": [
            "ft4222/ft4222.pyx"
        ]
    },
    "module_name": "ft4222.ft4222"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_32"
#define CYTHON_HEX_VERSION 0x001D20F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_HEX >= 0x07030900)
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PY_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ft4222__ft4222
#define __PYX_HAVE_API__ft4222__ft4222
/* Early includes */
#include "ftd2xx.h"
#include <stdint.h>
#include "libft4222.h"
#include <string.h>
#include <stdio.h>
#include <alloca.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "ft4222/ft4222.pyx",
  "stringsource",
  "array.pxd",
  "type.pxd",
};
/* BufferFormatStructs.proto */
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;


/*--- Type declarations ---*/
#ifndef _ARRAYARRAY_H
struct arrayobject;
typedef struct arrayobject arrayobject;
#endif
struct __pyx_obj_6ft4222_6ft4222__closer;
struct __pyx_obj_6ft4222_6ft4222_FT4222;

/* "ft4222/ft4222.pyx":169
 * 
 * 
 * cdef class _closer:             # <<<<<<<<<<<<<<
 *     """Wrap a hid_device *ptr and a provide a way to call hid_close() on it.
 *     Used internally for weakref.finalize, which only accepts Python objects.
 */
struct __pyx_obj_6ft4222_6ft4222__closer {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ft4222_6ft4222__closer *__pyx_vtab;
  FT_HANDLE _handle;
};


/* "ft4222/ft4222.pyx":187
 * 
 * 
 * cdef class FT4222:             # <<<<<<<<<<<<<<
 *     cdef FT_HANDLE _handle
 *     cdef DWORD _chip_version
 */
struct __pyx_obj_6ft4222_6ft4222_FT4222 {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 *__pyx_vtab;
  FT_HANDLE _handle;
  DWORD _chip_version;
  DWORD _dll_version;
  PyObject *__weakref__;
  PyObject *_close;
};



/* "ft4222/ft4222.pyx":169
 * 
 * 
 * cdef class _closer:             # <<<<<<<<<<<<<<
 *     """Wrap a hid_device *ptr and a provide a way to call hid_close() on it.
 *     Used internally for weakref.finalize, which only accepts Python objects.
 */

struct __pyx_vtabstruct_6ft4222_6ft4222__closer {
  PyObject *(*wrap)(FT_HANDLE);
};
static struct __pyx_vtabstruct_6ft4222_6ft4222__closer *__pyx_vtabptr_6ft4222_6ft4222__closer;


/* "ft4222/ft4222.pyx":187
 * 
 * 
 * cdef class FT4222:             # <<<<<<<<<<<<<<
 *     cdef FT_HANDLE _handle
 *     cdef DWORD _chip_version
 */

struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 {
  PyObject *(*_get_version)(struct __pyx_obj_6ft4222_6ft4222_FT4222 *);
};
static struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 *__pyx_vtabptr_6ft4222_6ft4222_FT4222;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* IsLittleEndian.proto */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void);

/* BufferFormatCheck.proto */
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts);
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type);

/* BufferGetAndValidate.proto */
#define __Pyx_GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack)\
    ((obj == Py_None || obj == NULL) ?\
    (__Pyx_ZeroBuffer(buf), 0) :\
    __Pyx__GetBufferAndValidate(buf, obj, dtype, flags, nd, cast, stack))
static int  __Pyx__GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static void __Pyx_ZeroBuffer(Py_buffer* buf);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);
static Py_ssize_t __Pyx_minusones[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
static Py_ssize_t __Pyx_zeros[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_TrueDivideCObj(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* ArrayAPI.proto */
#ifndef _ARRAYARRAY_H
#define _ARRAYARRAY_H
typedef struct arraydescr {
    int typecode;
    int itemsize;
    PyObject * (*getitem)(struct arrayobject *, Py_ssize_t);
    int (*setitem)(struct arrayobject *, Py_ssize_t, PyObject *);
#if PY_MAJOR_VERSION >= 3
    char *formats;
#endif
} arraydescr;
struct arrayobject {
    PyObject_HEAD
    Py_ssize_t ob_size;
    union {
        char *ob_item;
        float *as_floats;
        double *as_doubles;
        int *as_ints;
        unsigned int *as_uints;
        unsigned char *as_uchars;
        signed char *as_schars;
        char *as_chars;
        unsigned long *as_ulongs;
        long *as_longs;
#if PY_MAJOR_VERSION >= 3
        unsigned long long *as_ulonglongs;
        long long *as_longlongs;
#endif
        short *as_shorts;
        unsigned short *as_ushorts;
        Py_UNICODE *as_pyunicodes;
        void *as_voidptr;
    } data;
    Py_ssize_t allocated;
    struct arraydescr *ob_descr;
    PyObject *weakreflist;
#if PY_MAJOR_VERSION >= 3
        int ob_exports;
#endif
};
#ifndef NO_NEWARRAY_INLINE
static CYTHON_INLINE PyObject * newarrayobject(PyTypeObject *type, Py_ssize_t size,
    struct arraydescr *descr) {
    arrayobject *op;
    size_t nbytes;
    if (size < 0) {
        PyErr_BadInternalCall();
        return NULL;
    }
    nbytes = size * descr->itemsize;
    if (nbytes / descr->itemsize != (size_t)size) {
        return PyErr_NoMemory();
    }
    op = (arrayobject *) type->tp_alloc(type, 0);
    if (op == NULL) {
        return NULL;
    }
    op->ob_descr = descr;
    op->allocated = size;
    op->weakreflist = NULL;
    __Pyx_SET_SIZE(op, size);
    if (size <= 0) {
        op->data.ob_item = NULL;
    }
    else {
        op->data.ob_item = PyMem_NEW(char, nbytes);
        if (op->data.ob_item == NULL) {
            Py_DECREF(op);
            return PyErr_NoMemory();
        }
    }
    return (PyObject *) op;
}
#else
PyObject* newarrayobject(PyTypeObject *type, Py_ssize_t size,
    struct arraydescr *descr);
#endif
static CYTHON_INLINE int resize(arrayobject *self, Py_ssize_t n) {
    void *items = (void*) self->data.ob_item;
    PyMem_Resize(items, char, (size_t)(n * self->ob_descr->itemsize));
    if (items == NULL) {
        PyErr_NoMemory();
        return -1;
    }
    self->data.ob_item = (char*) items;
    __Pyx_SET_SIZE(self, n);
    self->allocated = n;
    return 0;
}
static CYTHON_INLINE int resize_smart(arrayobject *self, Py_ssize_t n) {
    void *items = (void*) self->data.ob_item;
    Py_ssize_t newsize;
    if (n < self->allocated && n*4 > self->allocated) {
        __Pyx_SET_SIZE(self, n);
        return 0;
    }
    newsize = n + (n / 2) + 1;
    if (newsize <= n) {
        PyErr_NoMemory();
        return -1;
    }
    PyMem_Resize(items, char, (size_t)(newsize * self->ob_descr->itemsize));
    if (items == NULL) {
        PyErr_NoMemory();
        return -1;
    }
    self->data.ob_item = (char*) items;
    __Pyx_SET_SIZE(self, n);
    self->allocated = newsize;
    return 0;
}
#endif

/* BufferStructDeclare.proto */
typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT4222_STATUS(FT4222_STATUS value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_DWORD(DWORD value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ULONG(ULONG value);

/* CIntFromPy.proto */
static CYTHON_INLINE DWORD __Pyx_PyInt_As_DWORD(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE ULONG __Pyx_PyInt_As_ULONG(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_ClockRate __Pyx_PyInt_As_FT4222_ClockRate(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT4222_ClockRate(FT4222_ClockRate value);

/* CIntFromPy.proto */
static CYTHON_INLINE BOOL __Pyx_PyInt_As_BOOL(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE UCHAR __Pyx_PyInt_As_UCHAR(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE USHORT __Pyx_PyInt_As_USHORT(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GPIO_Dir __Pyx_PyInt_As_GPIO_Dir(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GPIO_Port __Pyx_PyInt_As_GPIO_Port(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_BOOL(BOOL value);

/* CIntFromPy.proto */
static CYTHON_INLINE GPIO_Trigger __Pyx_PyInt_As_GPIO_Trigger(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint16_t __Pyx_PyInt_As_uint16_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GPIO_Trigger(GPIO_Trigger value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE SPI_DrivingStrength __Pyx_PyInt_As_SPI_DrivingStrength(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_SPIMode __Pyx_PyInt_As_FT4222_SPIMode(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_SPIClock __Pyx_PyInt_As_FT4222_SPIClock(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_SPICPOL __Pyx_PyInt_As_FT4222_SPICPOL(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT4222_SPICPHA __Pyx_PyInt_As_FT4222_SPICPHA(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE SPI_SlaveProtocol __Pyx_PyInt_As_SPI_SlaveProtocol(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_6ft4222_6ft4222_7_closer_wrap(FT_HANDLE __pyx_v_h); /* proto*/
static PyObject *__pyx_f_6ft4222_6ft4222_6FT4222__get_version(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto*/

/* Module declarations from 'ft4222.cftd2xx' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'ft4222.clibft4222' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'array' */

/* Module declarations from 'cpython.array' */
static PyTypeObject *__pyx_ptype_7cpython_5array_array = 0;
static CYTHON_INLINE int __pyx_f_7cpython_5array_extend_buffer(arrayobject *, char *, Py_ssize_t); /*proto*/

/* Module declarations from 'ft4222.ft4222' */
static PyTypeObject *__pyx_ptype_6ft4222_6ft4222__closer = 0;
static PyTypeObject *__pyx_ptype_6ft4222_6ft4222_FT4222 = 0;
static __Pyx_TypeInfo __Pyx_TypeInfo_nn_uint8 = { "uint8", NULL, sizeof(uint8), { 0 }, 0, IS_UNSIGNED(uint8) ? 'U' : 'I', IS_UNSIGNED(uint8), 0 };
#define __Pyx_MODULE_NAME "ft4222.ft4222"
extern int __pyx_module_is_main_ft4222__ft4222;
int __pyx_module_is_main_ft4222__ft4222 = 0;

/* Implementation of 'ft4222.ft4222' */
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_xrange;
static PyObject *__pyx_builtin_round;
static PyObject *__pyx_builtin_MemoryError;
static const char __pyx_k_B[] = "B";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_h[] = "h";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_l[] = "l";
static const char __pyx_k_n[] = "n";
static const char __pyx_k_t[] = "t";
static const char __pyx_k_OK[] = "OK";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_nb[] = "nb";
static const char __pyx_k_Dir[] = "Dir";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_req[] = "req";
static const char __pyx_k_str[] = "__str__";
static const char __pyx_k_GPIO[] = "GPIO";
static const char __pyx_k_addr[] = "addr";
static const char __pyx_k_cpha[] = "cpha";
static const char __pyx_k_cpol[] = "cpol";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_desc[] = "desc";
static const char __pyx_k_enum[] = "enum";
static const char __pyx_k_flag[] = "flag";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_kbps[] = "kbps";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mode[] = "mode";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_INPUT[] = "INPUT";
static const char __pyx_k_Rev_A[] = "Rev. A";
static const char __pyx_k_Rev_B[] = "Rev. B";
static const char __pyx_k_Rev_C[] = "Rev. C";
static const char __pyx_k_cdesc[] = "cdesc";
static const char __pyx_k_clock[] = "clock";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_gpio0[] = "gpio0";
static const char __pyx_k_gpio1[] = "gpio1";
static const char __pyx_k_gpio2[] = "gpio2";
static const char __pyx_k_gpio3[] = "gpio3";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_locId[] = "locId";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_round[] = "round";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_utf_8[] = "utf-8";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_CLK_24[] = "CLK_24";
static const char __pyx_k_CLK_48[] = "CLK_48";
static const char __pyx_k_CLK_60[] = "CLK_60";
static const char __pyx_k_CLK_80[] = "CLK_80";
static const char __pyx_k_FT4222[] = "FT4222";
static const char __pyx_k_closer[] = "_closer";
static const char __pyx_k_devnum[] = "devnum";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_handle[] = "handle";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_msgnum[] = "msgnum";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_serial[] = "serial";
static const char __pyx_k_ssoMap[] = "ssoMap";
static const char __pyx_k_status[] = "status";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_xrange[] = "xrange";
static const char __pyx_k_IntEnum[] = "IntEnum";
static const char __pyx_k_Trigger[] = "Trigger";
static const char __pyx_k_cserial[] = "cserial";
static const char __pyx_k_message[] = "message";
static const char __pyx_k_portNum[] = "portNum";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_trigger[] = "trigger";
static const char __pyx_k_weakref[] = "weakref";
static const char __pyx_k_IO_ERROR[] = "IO_ERROR";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_SysClock[] = "SysClock";
static const char __pyx_k_finalize[] = "finalize";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_location[] = "location";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_readSize[] = "readSize";
static const char __pyx_k_setClock[] = "setClock";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_I2CMaster[] = "I2CMaster";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_ioStrength[] = "ioStrength";
static const char __pyx_k_multiWrite[] = "multiWrite";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_OTHER_ERROR[] = "OTHER_ERROR";
static const char __pyx_k_Rev_unknown[] = "Rev. unknown";
static const char __pyx_k_bytesToRead[] = "bytesToRead";
static const char __pyx_k_clkStrength[] = "clkStrength";
static const char __pyx_k_description[] = "description";
static const char __pyx_k_ftd2xx_msgs[] = "__ftd2xx_msgs";
static const char __pyx_k_singleWrite[] = "singleWrite";
static const char __pyx_k_ssoStrength[] = "ssoStrength";
static const char __pyx_k_INVALID_ARGS[] = "INVALID_ARGS";
static const char __pyx_k_chipRevision[] = "chipRevision";
static const char __pyx_k_chip_rev_map[] = "__chip_rev_map";
static const char __pyx_k_ftd4222_msgs[] = "__ftd4222_msgs";
static const char __pyx_k_openBySerial[] = "openBySerial";
static const char __pyx_k_read_timeout[] = "read_timeout";
static const char __pyx_k_vendorCmdSet[] = "vendorCmdSet";
static const char __pyx_k_NOT_SUPPORTED[] = "NOT_SUPPORTED";
static const char __pyx_k_ft4222_ft4222[] = "ft4222.ft4222";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_write_timeout[] = "write_timeout";
static const char __pyx_k_INVALID_HANDLE[] = "INVALID_HANDLE";
static const char __pyx_k_WRONG_I2C_ADDR[] = "WRONG_I2C_ADDR";
static const char __pyx_k_openByLocation[] = "openByLocation";
static const char __pyx_k_FUN_NOT_SUPPORT[] = "FUN_NOT_SUPPORT";
static const char __pyx_k_INVALID_POINTER[] = "INVALID_POINTER";
static const char __pyx_k_IS_NOT_I2C_MODE[] = "IS_NOT_I2C_MODE";
static const char __pyx_k_IS_NOT_SPI_MODE[] = "IS_NOT_SPI_MODE";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_ControllerStatus[] = "ControllerStatus";
static const char __pyx_k_DEVICE_NOT_FOUND[] = "DEVICE_NOT_FOUND";
static const char __pyx_k_FT2XXDeviceError[] = "FT2XXDeviceError";
static const char __pyx_k_INVAILD_FUNCTION[] = "INVAILD_FUNCTION";
static const char __pyx_k_chip_rev_min_lib[] = "__chip_rev_min_lib";
static const char __pyx_k_isEndTransaction[] = "isEndTransaction";
static const char __pyx_k_weakref_finalize[] = "weakref_finalize";
static const char __pyx_k_CLK_NOT_SUPPORTED[] = "CLK_NOT_SUPPORTED";
static const char __pyx_k_DEVICE_NOT_OPENED[] = "DEVICE_NOT_OPENED";
static const char __pyx_k_FT4222DeviceError[] = "FT4222DeviceError";
static const char __pyx_k_INVALID_BAUD_RATE[] = "INVALID_BAUD_RATE";
static const char __pyx_k_INVALID_PARAMETER[] = "INVALID_PARAMETER";
static const char __pyx_k_ft4222_ft4222_pyx[] = "ft4222/ft4222.pyx";
static const char __pyx_k_openByDescription[] = "openByDescription";
static const char __pyx_k_EEPROM_NOT_PRESENT[] = "EEPROM_NOT_PRESENT";
static const char __pyx_k_EEPROM_READ_FAILED[] = "EEPROM_READ_FAILED";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_EEPROM_ERASE_FAILED[] = "EEPROM_ERASE_FAILED";
static const char __pyx_k_EEPROM_WRITE_FAILED[] = "EEPROM_WRITE_FAILED";
static const char __pyx_k_EVENT_NOT_SUPPORTED[] = "EVENT_NOT_SUPPORTED";
static const char __pyx_k_getDeviceInfoDetail[] = "getDeviceInfoDetail";
static const char __pyx_k_DEVICE_NOT_SUPPORTED[] = "DEVICE_NOT_SUPPORTED";
static const char __pyx_k_createDeviceInfoList[] = "createDeviceInfoList";
static const char __pyx_k_DEVICE_LIST_NOT_READY[] = "DEVICE_LIST_NOT_READY";
static const char __pyx_k_EEPROM_NOT_PROGRAMMED[] = "EEPROM_NOT_PROGRAMMED";
static const char __pyx_k_FAILED_TO_READ_DEVICE[] = "FAILED_TO_READ_DEVICE";
static const char __pyx_k_IS_NOT_SPI_MULTI_MODE[] = "IS_NOT_SPI_MULTI_MODE";
static const char __pyx_k_gpio_GetTriggerStatus[] = "gpio_GetTriggerStatus";
static const char __pyx_k_FT2XXDeviceError___str[] = "FT2XXDeviceError.__str__";
static const char __pyx_k_INSUFFICIENT_RESOURCES[] = "INSUFFICIENT_RESOURCES";
static const char __pyx_k_IS_NOT_SPI_SINGLE_MODE[] = "IS_NOT_SPI_SINGLE_MODE";
static const char __pyx_k_FAILED_TO_WRITE_DEVICE0[] = "FAILED_TO_WRITE_DEVICE0";
static const char __pyx_k_FT2XXDeviceError___init[] = "FT2XXDeviceError.__init__";
static const char __pyx_k_FT4222DeviceError___str[] = "FT4222DeviceError.__str__";
static const char __pyx_k_INTERRUPT_NOT_SUPPORTED[] = "INTERRUPT_NOT_SUPPORTED";
static const char __pyx_k_FT4222DeviceError___init[] = "FT4222DeviceError.__init__";
static const char __pyx_k_GPIO_INPUT_NOT_SUPPORTED[] = "GPIO_INPUT_NOT_SUPPORTED";
static const char __pyx_k_GPIO_WRITE_NOT_SUPPORTED[] = "GPIO_WRITE_NOT_SUPPORTED";
static const char __pyx_k_VENDER_CMD_NOT_SUPPORTED[] = "VENDER_CMD_NOT_SUPPORTED";
static const char __pyx_k_GPIO_EXCEEDED_MAX_PORTNUM[] = "GPIO_EXCEEDED_MAX_PORTNUM";
static const char __pyx_k_EXCEEDED_MAX_TRANSFER_SIZE[] = "EXCEEDED_MAX_TRANSFER_SIZE";
static const char __pyx_k_DEVICE_NOT_OPENED_FOR_ERASE[] = "DEVICE_NOT_OPENED_FOR_ERASE";
static const char __pyx_k_DEVICE_NOT_OPENED_FOR_WRITE[] = "DEVICE_NOT_OPENED_FOR_WRITE";
static const char __pyx_k_I2C_NOT_SUPPORTED_IN_THIS_MODE[] = "I2C_NOT_SUPPORTED_IN_THIS_MODE";
static const char __pyx_k_GPIO_NOT_SUPPORTED_IN_THIS_MODE[] = "GPIO_NOT_SUPPORTED_IN_THIS_MODE";
static const char __pyx_k_the_multiWrite_argument_must_be[] = "the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'";
static const char __pyx_k_Chip_system_clock_Attributes_CLK[] = "Chip system clock\n\n    Attributes:\n        CLK_60: 60 MHz\n        CLK_24: 24 MHz\n        CLK_48: 48 MHz\n        CLK_80: 80 MHz\n\n    ";
static const char __pyx_k_Exception_class_for_status_messa[] = "Exception class for status messages";
static const char __pyx_k_FT4222_chipVersion_0x_x_s_libVer[] = "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}";
static const char __pyx_k_GPIO_OPENDRAIN_INVALID_IN_OUTPUT[] = "GPIO_OPENDRAIN_INVALID_IN_OUTPUTMODE";
static const char __pyx_k_GPIO_PULLDOWN_INVALID_IN_INPUTMO[] = "GPIO_PULLDOWN_INVALID_IN_INPUTMODE";
static const char __pyx_k_GPIO_PULLUP_INVALID_IN_INPUTMODE[] = "GPIO_PULLUP_INVALID_IN_INPUTMODE";
static const char __pyx_k_self__handle_cannot_be_converted[] = "self._handle cannot be converted to a Python object for pickling";
static const char __pyx_k_the_data_argument_must_be_of_typ[] = "the data argument must be of type 'int', 'bytes' or 'bytearray'";
static const char __pyx_k_the_singleWrite_argument_must_be[] = "the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'";
static PyObject *__pyx_n_u_B;
static PyObject *__pyx_n_s_CLK_24;
static PyObject *__pyx_n_s_CLK_48;
static PyObject *__pyx_n_s_CLK_60;
static PyObject *__pyx_n_s_CLK_80;
static PyObject *__pyx_n_u_CLK_NOT_SUPPORTED;
static PyObject *__pyx_kp_s_Chip_system_clock_Attributes_CLK;
static PyObject *__pyx_n_s_ControllerStatus;
static PyObject *__pyx_n_u_DEVICE_LIST_NOT_READY;
static PyObject *__pyx_n_u_DEVICE_NOT_FOUND;
static PyObject *__pyx_n_u_DEVICE_NOT_OPENED;
static PyObject *__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE;
static PyObject *__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE;
static PyObject *__pyx_n_u_DEVICE_NOT_SUPPORTED;
static PyObject *__pyx_n_s_Dir;
static PyObject *__pyx_n_u_EEPROM_ERASE_FAILED;
static PyObject *__pyx_n_u_EEPROM_NOT_PRESENT;
static PyObject *__pyx_n_u_EEPROM_NOT_PROGRAMMED;
static PyObject *__pyx_n_u_EEPROM_READ_FAILED;
static PyObject *__pyx_n_u_EEPROM_WRITE_FAILED;
static PyObject *__pyx_n_u_EVENT_NOT_SUPPORTED;
static PyObject *__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE;
static PyObject *__pyx_kp_s_Exception_class_for_status_messa;
static PyObject *__pyx_n_u_FAILED_TO_READ_DEVICE;
static PyObject *__pyx_n_u_FAILED_TO_WRITE_DEVICE0;
static PyObject *__pyx_n_s_FT2XXDeviceError;
static PyObject *__pyx_n_s_FT2XXDeviceError___init;
static PyObject *__pyx_n_s_FT2XXDeviceError___str;
static PyObject *__pyx_n_s_FT4222;
static PyObject *__pyx_n_s_FT4222DeviceError;
static PyObject *__pyx_n_s_FT4222DeviceError___init;
static PyObject *__pyx_n_s_FT4222DeviceError___str;
static PyObject *__pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer;
static PyObject *__pyx_n_u_FUN_NOT_SUPPORT;
static PyObject *__pyx_n_s_GPIO;
static PyObject *__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM;
static PyObject *__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED;
static PyObject *__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE;
static PyObject *__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT;
static PyObject *__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO;
static PyObject *__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE;
static PyObject *__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED;
static PyObject *__pyx_n_s_I2CMaster;
static PyObject *__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE;
static PyObject *__pyx_n_s_INPUT;
static PyObject *__pyx_n_u_INSUFFICIENT_RESOURCES;
static PyObject *__pyx_n_u_INTERRUPT_NOT_SUPPORTED;
static PyObject *__pyx_n_u_INVAILD_FUNCTION;
static PyObject *__pyx_n_u_INVALID_ARGS;
static PyObject *__pyx_n_u_INVALID_BAUD_RATE;
static PyObject *__pyx_n_u_INVALID_HANDLE;
static PyObject *__pyx_n_u_INVALID_PARAMETER;
static PyObject *__pyx_n_u_INVALID_POINTER;
static PyObject *__pyx_n_u_IO_ERROR;
static PyObject *__pyx_n_u_IS_NOT_I2C_MODE;
static PyObject *__pyx_n_u_IS_NOT_SPI_MODE;
static PyObject *__pyx_n_u_IS_NOT_SPI_MULTI_MODE;
static PyObject *__pyx_n_u_IS_NOT_SPI_SINGLE_MODE;
static PyObject *__pyx_n_s_IntEnum;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_n_u_NOT_SUPPORTED;
static PyObject *__pyx_n_u_OK;
static PyObject *__pyx_n_u_OTHER_ERROR;
static PyObject *__pyx_kp_u_Rev_A;
static PyObject *__pyx_kp_u_Rev_B;
static PyObject *__pyx_kp_u_Rev_C;
static PyObject *__pyx_kp_u_Rev_unknown;
static PyObject *__pyx_n_s_SysClock;
static PyObject *__pyx_n_s_Trigger;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_u_VENDER_CMD_NOT_SUPPORTED;
static PyObject *__pyx_n_u_WRONG_I2C_ADDR;
static PyObject *__pyx_n_s_addr;
static PyObject *__pyx_n_s_bytesToRead;
static PyObject *__pyx_n_s_cdesc;
static PyObject *__pyx_n_s_chipRevision;
static PyObject *__pyx_n_s_chip_rev_map;
static PyObject *__pyx_n_s_chip_rev_min_lib;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_clkStrength;
static PyObject *__pyx_n_s_clock;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_closer;
static PyObject *__pyx_n_s_cpha;
static PyObject *__pyx_n_s_cpol;
static PyObject *__pyx_n_s_createDeviceInfoList;
static PyObject *__pyx_n_s_cserial;
static PyObject *__pyx_n_s_d;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_desc;
static PyObject *__pyx_n_u_description;
static PyObject *__pyx_n_s_devnum;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_enum;
static PyObject *__pyx_n_s_f;
static PyObject *__pyx_n_s_finalize;
static PyObject *__pyx_n_s_flag;
static PyObject *__pyx_n_u_flags;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_ft4222_ft4222;
static PyObject *__pyx_kp_s_ft4222_ft4222_pyx;
static PyObject *__pyx_n_s_ftd2xx_msgs;
static PyObject *__pyx_n_s_ftd4222_msgs;
static PyObject *__pyx_n_s_getDeviceInfoDetail;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_gpio0;
static PyObject *__pyx_n_s_gpio1;
static PyObject *__pyx_n_s_gpio2;
static PyObject *__pyx_n_s_gpio3;
static PyObject *__pyx_n_s_gpio_GetTriggerStatus;
static PyObject *__pyx_n_s_h;
static PyObject *__pyx_n_s_handle;
static PyObject *__pyx_n_u_handle;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_u_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_u_index;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_ioStrength;
static PyObject *__pyx_n_s_isEndTransaction;
static PyObject *__pyx_n_s_kbps;
static PyObject *__pyx_n_s_l;
static PyObject *__pyx_n_s_locId;
static PyObject *__pyx_n_u_location;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_message;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_msgnum;
static PyObject *__pyx_n_s_multiWrite;
static PyObject *__pyx_n_s_n;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_nb;
static PyObject *__pyx_n_s_openByDescription;
static PyObject *__pyx_n_s_openByLocation;
static PyObject *__pyx_n_s_openBySerial;
static PyObject *__pyx_n_s_portNum;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_readSize;
static PyObject *__pyx_n_s_read_timeout;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_req;
static PyObject *__pyx_n_s_round;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_kp_s_self__handle_cannot_be_converted;
static PyObject *__pyx_n_s_serial;
static PyObject *__pyx_n_u_serial;
static PyObject *__pyx_n_s_setClock;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_singleWrite;
static PyObject *__pyx_n_s_ssoMap;
static PyObject *__pyx_n_s_ssoStrength;
static PyObject *__pyx_n_s_status;
static PyObject *__pyx_n_s_str;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_t;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_kp_u_the_data_argument_must_be_of_typ;
static PyObject *__pyx_kp_u_the_multiWrite_argument_must_be;
static PyObject *__pyx_kp_u_the_singleWrite_argument_must_be;
static PyObject *__pyx_n_s_trigger;
static PyObject *__pyx_n_u_type;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_kp_u_utf_8;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_vendorCmdSet;
static PyObject *__pyx_n_s_weakref;
static PyObject *__pyx_n_s_weakref_finalize;
static PyObject *__pyx_n_s_write_timeout;
static PyObject *__pyx_n_s_xrange;
static PyObject *__pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_msgnum); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_17FT4222DeviceError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_msgnum); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_17FT4222DeviceError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_createDeviceInfoList(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_2getDeviceInfoDetail(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_devnum, PyObject *__pyx_v_update); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_4openBySerial(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_serial); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6openByDescription(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_desc); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_8openByLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_locId); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_close(struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ft4222_6ft4222_6FT4222___init__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_handle, CYTHON_UNUSED PyObject *__pyx_v_update); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_2close(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_11chipVersion___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_10libVersion___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_12chipRevision___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_4__repr__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_6setTimeouts(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_read_timeout, PyObject *__pyx_v_write_timeout); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_8setClock(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_clk); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_10getClock(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_12setSuspendOut(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_enable); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_enable); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_16vendorCmdGet(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_req, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_18vendorCmdSet(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_req, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_20gpio_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_gpio0, PyObject *__pyx_v_gpio1, PyObject *__pyx_v_gpio2, PyObject *__pyx_v_gpio3, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_22gpio_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_24gpio_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_26gpio_SetInputTrigger(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_trigger); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_28gpio_GetTriggerStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_30gpio_ReadTriggerQueue(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_readSize); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_32i2cMaster_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_kbps); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_34i2cMaster_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_36i2cMaster_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_38i2cMaster_ReadEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_flag, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_40i2cMaster_WriteEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_flag, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_42i2cMaster_Reset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_44i2cMaster_GetStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_46spi_Reset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_48spi_ResetTransaction(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_spiIdx); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_50spi_SetDrivingStrength(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_clkStrength, PyObject *__pyx_v_ioStrength, PyObject *__pyx_v_ssoStrength); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_52spiMaster_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_mode, PyObject *__pyx_v_clock, PyObject *__pyx_v_cpol, PyObject *__pyx_v_cpha, PyObject *__pyx_v_ssoMap); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_54spiMaster_SetLines(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_mode); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_56spiMaster_SingleRead(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_bytesToRead, PyObject *__pyx_v_isEndTransaction); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_58spiMaster_SingleWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_isEndTransaction); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_60spiMaster_SingleReadWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_isEndTransaction); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_62spiMaster_MultiReadWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_singleWrite, PyObject *__pyx_v_multiWrite, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_64spiMaster_EndTransaction(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_66spiSlave_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_68spiSlave_InitEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_proto); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_70spiSlave_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_bytesToRead); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_72spiSlave_SetMode(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_cpol, PyObject *__pyx_v_cpha); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_74spiSlave_GetRxStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_76spiSlave_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_78__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_80__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7cpython_5array_5array___getbuffer__(arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info, CYTHON_UNUSED int __pyx_v_flags); /* proto */
static void __pyx_pf_7cpython_5array_5array_2__releasebuffer__(CYTHON_UNUSED arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_tp_new_6ft4222_6ft4222__closer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ft4222_6ft4222_FT4222(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_24000000_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_60;
static PyObject *__pyx_int_82;
static PyObject *__pyx_int_100;
static PyObject *__pyx_int_8000;
static PyObject *__pyx_int_16908288;
static PyObject *__pyx_int_16973824;
static PyObject *__pyx_int_1109524736;
static PyObject *__pyx_int_1109524992;
static PyObject *__pyx_int_1109525248;
static PyObject *__pyx_k__4;
static PyObject *__pyx_k__5;
static PyObject *__pyx_k__6;
static PyObject *__pyx_k__7;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_codeobj__13;
static PyObject *__pyx_codeobj__15;
static PyObject *__pyx_codeobj__17;
static PyObject *__pyx_codeobj__19;
static PyObject *__pyx_codeobj__21;
static PyObject *__pyx_codeobj__23;
static PyObject *__pyx_codeobj__25;
static PyObject *__pyx_codeobj__27;
static PyObject *__pyx_codeobj__29;
/* Late includes */

/* "ft4222/ft4222.pyx":60
 * class FT2XXDeviceError(Exception):
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         self.message = __ftd2xx_msgs[msgnum]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_16FT2XXDeviceError_1__init__ = {"__init__", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_msgnum = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_msgnum,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_msgnum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 60, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 60, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_msgnum = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 60, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT2XXDeviceError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError___init__(__pyx_self, __pyx_v_self, __pyx_v_msgnum);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_msgnum) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ft4222/ft4222.pyx":61
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):
 *         self.message = __ftd2xx_msgs[msgnum]             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_ftd2xx_msgs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_msgnum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_message, __pyx_t_2) < 0) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":60
 * class FT2XXDeviceError(Exception):
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         self.message = __ftd2xx_msgs[msgnum]
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ft4222.ft4222.FT2XXDeviceError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":63
 *         self.message = __ftd2xx_msgs[msgnum]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_3__str__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_16FT2XXDeviceError_3__str__ = {"__str__", (PyCFunction)__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_3__str__, METH_O, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_16FT2XXDeviceError_3__str__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError_2__str__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_16FT2XXDeviceError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ft4222/ft4222.pyx":64
 * 
 *     def __str__(self):
 *         return self.message             # <<<<<<<<<<<<<<
 * 
 * class FT4222DeviceError(FT2XXDeviceError):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_message); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":63
 *         self.message = __ftd2xx_msgs[msgnum]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT2XXDeviceError.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":68
 * class FT4222DeviceError(FT2XXDeviceError):
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = __ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_17FT4222DeviceError_1__init__ = {"__init__", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_msgnum = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_msgnum,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_msgnum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 68, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 68, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_msgnum = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 68, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222DeviceError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_17FT4222DeviceError___init__(__pyx_self, __pyx_v_self, __pyx_v_msgnum);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_17FT4222DeviceError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_msgnum) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ft4222/ft4222.pyx":69
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:             # <<<<<<<<<<<<<<
 *             self.message = __ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 *         else:
 */
  __pyx_t_1 = __Pyx_PyInt_From_FT4222_STATUS(FT4222_DEVICE_NOT_SUPPORTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_msgnum, __pyx_t_1, Py_GE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "ft4222/ft4222.pyx":70
 *     def __init__(self, msgnum):
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = __ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]             # <<<<<<<<<<<<<<
 *         else:
 *             super(FT4222DeviceError, self).__init__(msgnum)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_ftd4222_msgs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyInt_From_FT4222_STATUS(FT4222_DEVICE_NOT_SUPPORTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyNumber_Subtract(__pyx_v_msgnum, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_message, __pyx_t_1) < 0) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ft4222/ft4222.pyx":69
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:             # <<<<<<<<<<<<<<
 *             self.message = __ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":72
 *             self.message = __ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 *         else:
 *             super(FT4222DeviceError, self).__init__(msgnum)             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 72, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 72, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_self);
    __Pyx_GIVEREF(__pyx_v_self);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 72, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 72, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v_msgnum) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_msgnum);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 72, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":68
 * class FT4222DeviceError(FT2XXDeviceError):
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = __ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222DeviceError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":74
 *             super(FT4222DeviceError, self).__init__(msgnum)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_3__str__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_17FT4222DeviceError_3__str__ = {"__str__", (PyCFunction)__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_3__str__, METH_O, 0};
static PyObject *__pyx_pw_6ft4222_6ft4222_17FT4222DeviceError_3__str__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_17FT4222DeviceError_2__str__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_17FT4222DeviceError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ft4222/ft4222.pyx":75
 * 
 *     def __str__(self):
 *         return self.message             # <<<<<<<<<<<<<<
 * 
 * class SysClock(IntEnum):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_message); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":74
 *             super(FT4222DeviceError, self).__init__(msgnum)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222DeviceError.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":92
 *     CLK_80 = 3
 * 
 * def createDeviceInfoList():             # <<<<<<<<<<<<<<
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_1createDeviceInfoList(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_createDeviceInfoList[] = "Create the internal device info list and return number of entries";
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_1createDeviceInfoList = {"createDeviceInfoList", (PyCFunction)__pyx_pw_6ft4222_6ft4222_1createDeviceInfoList, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_createDeviceInfoList};
static PyObject *__pyx_pw_6ft4222_6ft4222_1createDeviceInfoList(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("createDeviceInfoList (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_createDeviceInfoList(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_createDeviceInfoList(CYTHON_UNUSED PyObject *__pyx_self) {
  DWORD __pyx_v_nb;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("createDeviceInfoList", 0);

  /* "ft4222/ft4222.pyx":95
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 *     status = FT_CreateDeviceInfoList(&nb)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         return nb
 */
  __pyx_v_status = FT_CreateDeviceInfoList((&__pyx_v_nb));

  /* "ft4222/ft4222.pyx":96
 *     cdef DWORD nb
 *     status = FT_CreateDeviceInfoList(&nb)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return nb
 *     raise FT2XXDeviceError, status
 */
  __pyx_t_1 = ((__pyx_v_status == FT_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":97
 *     status = FT_CreateDeviceInfoList(&nb)
 *     if status == FT_OK:
 *         return nb             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_DWORD(__pyx_v_nb); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":96
 *     cdef DWORD nb
 *     status = FT_CreateDeviceInfoList(&nb)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return nb
 *     raise FT2XXDeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":98
 *     if status == FT_OK:
 *         return nb
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 98, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":92
 *     CLK_80 = 3
 * 
 * def createDeviceInfoList():             # <<<<<<<<<<<<<<
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.createDeviceInfoList", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":100
 *     raise FT2XXDeviceError, status
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):             # <<<<<<<<<<<<<<
 *     """Get an entry from the internal device info list. Set update to
 *     False to avoid a slow call to createDeviceInfoList."""
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_3getDeviceInfoDetail(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_2getDeviceInfoDetail[] = "Get an entry from the internal device info list. Set update to\n    False to avoid a slow call to createDeviceInfoList.";
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_3getDeviceInfoDetail = {"getDeviceInfoDetail", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_3getDeviceInfoDetail, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_2getDeviceInfoDetail};
static PyObject *__pyx_pw_6ft4222_6ft4222_3getDeviceInfoDetail(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_devnum = 0;
  PyObject *__pyx_v_update = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getDeviceInfoDetail (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_devnum,&__pyx_n_s_update,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)__pyx_int_0);
    values[1] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_devnum);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_update);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "getDeviceInfoDetail") < 0)) __PYX_ERR(0, 100, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_devnum = values[0];
    __pyx_v_update = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getDeviceInfoDetail", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 100, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.getDeviceInfoDetail", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_2getDeviceInfoDetail(__pyx_self, __pyx_v_devnum, __pyx_v_update);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_2getDeviceInfoDetail(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_devnum, PyObject *__pyx_v_update) {
  DWORD __pyx_v_f;
  DWORD __pyx_v_t;
  DWORD __pyx_v_i;
  DWORD __pyx_v_l;
  FT_HANDLE __pyx_v_h;
  char __pyx_v_n[0x100];
  char __pyx_v_d[0x100];
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  DWORD __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getDeviceInfoDetail", 0);

  /* "ft4222/ft4222.pyx":109
 *         char d[MAX_DESCRIPTION_SIZE]
 *     # createDeviceInfoList is slow, only run if update is True
 *     if update: createDeviceInfoList()             # <<<<<<<<<<<<<<
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)
 *     if status == FT_OK:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_update); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 109, __pyx_L1_error)
  if (__pyx_t_1) {
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_createDeviceInfoList); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "ft4222/ft4222.pyx":110
 *     # createDeviceInfoList is slow, only run if update is True
 *     if update: createDeviceInfoList()
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         return {'index': devnum, 'flags': f, 'type': t,
 */
  __pyx_t_5 = __Pyx_PyInt_As_DWORD(__pyx_v_devnum); if (unlikely((__pyx_t_5 == ((DWORD)-1)) && PyErr_Occurred())) __PYX_ERR(0, 110, __pyx_L1_error)
  __pyx_v_status = FT_GetDeviceInfoDetail(__pyx_t_5, (&__pyx_v_f), (&__pyx_v_t), (&__pyx_v_i), (&__pyx_v_l), __pyx_v_n, __pyx_v_d, (&__pyx_v_h));

  /* "ft4222/ft4222.pyx":111
 *     if update: createDeviceInfoList()
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return {'index': devnum, 'flags': f, 'type': t,
 *                 'id': i, 'location': l, 'serial': n,
 */
  __pyx_t_1 = ((__pyx_v_status == FT_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":112
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)
 *     if status == FT_OK:
 *         return {'index': devnum, 'flags': f, 'type': t,             # <<<<<<<<<<<<<<
 *                 'id': i, 'location': l, 'serial': n,
 *                 'description': d, 'handle': <size_t>h}
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyDict_NewPresized(8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_index, __pyx_v_devnum) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_flags, __pyx_t_3) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_type, __pyx_t_3) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":113
 *     if status == FT_OK:
 *         return {'index': devnum, 'flags': f, 'type': t,
 *                 'id': i, 'location': l, 'serial': n,             # <<<<<<<<<<<<<<
 *                 'description': d, 'handle': <size_t>h}
 *     raise FT2XXDeviceError, status
 */
    __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_id, __pyx_t_3) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_l); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_location, __pyx_t_3) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_FromString(__pyx_v_n); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_serial, __pyx_t_3) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":114
 *         return {'index': devnum, 'flags': f, 'type': t,
 *                 'id': i, 'location': l, 'serial': n,
 *                 'description': d, 'handle': <size_t>h}             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_FromString(__pyx_v_d); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_description, __pyx_t_3) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_FromSize_t(((size_t)__pyx_v_h)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_handle, __pyx_t_3) < 0) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":111
 *     if update: createDeviceInfoList()
 *     status = FT_GetDeviceInfoDetail(devnum, &f, &t, &i, &l, n, d, &h)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return {'index': devnum, 'flags': f, 'type': t,
 *                 'id': i, 'location': l, 'serial': n,
 */
  }

  /* "ft4222/ft4222.pyx":115
 *                 'id': i, 'location': l, 'serial': n,
 *                 'description': d, 'handle': <size_t>h}
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * def openBySerial(serial):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 115, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":100
 *     raise FT2XXDeviceError, status
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):             # <<<<<<<<<<<<<<
 *     """Get an entry from the internal device info list. Set update to
 *     False to avoid a slow call to createDeviceInfoList."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.getDeviceInfoDetail", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":117
 *     raise FT2XXDeviceError, status
 * 
 * def openBySerial(serial):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by serial number"""
 *     cdef FT_HANDLE handle
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_5openBySerial(PyObject *__pyx_self, PyObject *__pyx_v_serial); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_4openBySerial[] = "Open a handle to a usb device by serial number";
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_5openBySerial = {"openBySerial", (PyCFunction)__pyx_pw_6ft4222_6ft4222_5openBySerial, METH_O, __pyx_doc_6ft4222_6ft4222_4openBySerial};
static PyObject *__pyx_pw_6ft4222_6ft4222_5openBySerial(PyObject *__pyx_self, PyObject *__pyx_v_serial) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("openBySerial (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_4openBySerial(__pyx_self, ((PyObject *)__pyx_v_serial));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_4openBySerial(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_serial) {
  FT_HANDLE __pyx_v_handle;
  char *__pyx_v_cserial;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("openBySerial", 0);

  /* "ft4222/ft4222.pyx":120
 *     """Open a handle to a usb device by serial number"""
 *     cdef FT_HANDLE handle
 *     cdef char* cserial = serial             # <<<<<<<<<<<<<<
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)
 *     if status == FT_OK:
 */
  __pyx_t_1 = __Pyx_PyObject_AsWritableString(__pyx_v_serial); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(0, 120, __pyx_L1_error)
  __pyx_v_cserial = __pyx_t_1;

  /* "ft4222/ft4222.pyx":121
 *     cdef FT_HANDLE handle
 *     cdef char* cserial = serial
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)
 */
  __pyx_v_status = FT_OpenEx(((PVOID)__pyx_v_cserial), FT_OPEN_BY_SERIAL_NUMBER, (&__pyx_v_handle));

  /* "ft4222/ft4222.pyx":122
 *     cdef char* cserial = serial
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status
 */
  __pyx_t_2 = ((__pyx_v_status == FT_OK) != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":123
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_FromSize_t(((uintptr_t)__pyx_v_handle)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_update, Py_False) < 0) __PYX_ERR(0, 123, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222), __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":122
 *     cdef char* cserial = serial
 *     status = FT_OpenEx(<PVOID>cserial, FT_OPEN_BY_SERIAL_NUMBER, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":124
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * def openByDescription(desc):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_5, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 124, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":117
 *     raise FT2XXDeviceError, status
 * 
 * def openBySerial(serial):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by serial number"""
 *     cdef FT_HANDLE handle
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.openBySerial", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":126
 *     raise FT2XXDeviceError, status
 * 
 * def openByDescription(desc):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by description
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_7openByDescription(PyObject *__pyx_self, PyObject *__pyx_v_desc); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6openByDescription[] = "Open a handle to a usb device by description\n\n    Args:\n        desc (bytes, str): Description of the device\n\n    Returns:\n        :obj:`FT4222`: Opened device\n\n    Raises:\n        FT2XXDeviceError: on error\n\n    ";
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_7openByDescription = {"openByDescription", (PyCFunction)__pyx_pw_6ft4222_6ft4222_7openByDescription, METH_O, __pyx_doc_6ft4222_6ft4222_6openByDescription};
static PyObject *__pyx_pw_6ft4222_6ft4222_7openByDescription(PyObject *__pyx_self, PyObject *__pyx_v_desc) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("openByDescription (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6openByDescription(__pyx_self, ((PyObject *)__pyx_v_desc));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6openByDescription(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_desc) {
  FT_HANDLE __pyx_v_handle;
  char *__pyx_v_cdesc;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("openByDescription", 0);
  __Pyx_INCREF(__pyx_v_desc);

  /* "ft4222/ft4222.pyx":139
 * 
 *     """
 *     if isinstance(desc, str):             # <<<<<<<<<<<<<<
 *         desc = desc.encode('utf-8')
 *     cdef FT_HANDLE handle
 */
  __pyx_t_1 = PyUnicode_Check(__pyx_v_desc); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":140
 *     """
 *     if isinstance(desc, str):
 *         desc = desc.encode('utf-8')             # <<<<<<<<<<<<<<
 *     cdef FT_HANDLE handle
 *     cdef char* cdesc = desc
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_encode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_kp_u_utf_8) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_kp_u_utf_8);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_desc, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":139
 * 
 *     """
 *     if isinstance(desc, str):             # <<<<<<<<<<<<<<
 *         desc = desc.encode('utf-8')
 *     cdef FT_HANDLE handle
 */
  }

  /* "ft4222/ft4222.pyx":142
 *         desc = desc.encode('utf-8')
 *     cdef FT_HANDLE handle
 *     cdef char* cdesc = desc             # <<<<<<<<<<<<<<
 *     status = FT_OpenEx(<PVOID>cdesc, FT_OPEN_BY_DESCRIPTION, &handle)
 *     if status == FT_OK:
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableString(__pyx_v_desc); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 142, __pyx_L1_error)
  __pyx_v_cdesc = __pyx_t_6;

  /* "ft4222/ft4222.pyx":143
 *     cdef FT_HANDLE handle
 *     cdef char* cdesc = desc
 *     status = FT_OpenEx(<PVOID>cdesc, FT_OPEN_BY_DESCRIPTION, &handle)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         #printf("handle: %d\n", handle)
 */
  __pyx_v_status = FT_OpenEx(((PVOID)__pyx_v_cdesc), FT_OPEN_BY_DESCRIPTION, (&__pyx_v_handle));

  /* "ft4222/ft4222.pyx":144
 *     cdef char* cdesc = desc
 *     status = FT_OpenEx(<PVOID>cdesc, FT_OPEN_BY_DESCRIPTION, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         #printf("handle: %d\n", handle)
 *         return FT4222(<uintptr_t>handle, update=False)
 */
  __pyx_t_2 = ((__pyx_v_status == FT_OK) != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":146
 *     if status == FT_OK:
 *         #printf("handle: %d\n", handle)
 *         return FT4222(<uintptr_t>handle, update=False)             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_FromSize_t(((uintptr_t)__pyx_v_handle)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_update, Py_False) < 0) __PYX_ERR(0, 146, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222), __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":144
 *     cdef char* cdesc = desc
 *     status = FT_OpenEx(<PVOID>cdesc, FT_OPEN_BY_DESCRIPTION, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         #printf("handle: %d\n", handle)
 *         return FT4222(<uintptr_t>handle, update=False)
 */
  }

  /* "ft4222/ft4222.pyx":147
 *         #printf("handle: %d\n", handle)
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * def openByLocation(locId):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_5, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 147, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":126
 *     raise FT2XXDeviceError, status
 * 
 * def openByDescription(desc):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by description
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.openByDescription", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":149
 *     raise FT2XXDeviceError, status
 * 
 * def openByLocation(locId):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by location
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_9openByLocation(PyObject *__pyx_self, PyObject *__pyx_v_locId); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_8openByLocation[] = "Open a handle to a usb device by location\n\n    Args:\n        locId (int): Location id\n\n    Returns:\n        :obj:`FT4222`: Opened device\n\n    Raises:\n        FT2XXDeviceError: on error\n\n    ";
static PyMethodDef __pyx_mdef_6ft4222_6ft4222_9openByLocation = {"openByLocation", (PyCFunction)__pyx_pw_6ft4222_6ft4222_9openByLocation, METH_O, __pyx_doc_6ft4222_6ft4222_8openByLocation};
static PyObject *__pyx_pw_6ft4222_6ft4222_9openByLocation(PyObject *__pyx_self, PyObject *__pyx_v_locId) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("openByLocation (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_8openByLocation(__pyx_self, ((PyObject *)__pyx_v_locId));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_8openByLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_locId) {
  FT_HANDLE __pyx_v_handle;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uintptr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("openByLocation", 0);

  /* "ft4222/ft4222.pyx":163
 *     """
 *     cdef FT_HANDLE handle
 *     status = FT_OpenEx(<PVOID><uintptr_t>locId, FT_OPEN_BY_LOCATION, &handle)             # <<<<<<<<<<<<<<
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_locId); if (unlikely((__pyx_t_1 == ((uintptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 163, __pyx_L1_error)
  __pyx_v_status = FT_OpenEx(((PVOID)((uintptr_t)__pyx_t_1)), FT_OPEN_BY_LOCATION, (&__pyx_v_handle));

  /* "ft4222/ft4222.pyx":164
 *     cdef FT_HANDLE handle
 *     status = FT_OpenEx(<PVOID><uintptr_t>locId, FT_OPEN_BY_LOCATION, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status
 */
  __pyx_t_2 = ((__pyx_v_status == FT_OK) != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":165
 *     status = FT_OpenEx(<PVOID><uintptr_t>locId, FT_OPEN_BY_LOCATION, &handle)
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)             # <<<<<<<<<<<<<<
 *     raise FT2XXDeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_FromSize_t(((uintptr_t)__pyx_v_handle)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_update, Py_False) < 0) __PYX_ERR(0, 165, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ft4222_6ft4222_FT4222), __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":164
 *     cdef FT_HANDLE handle
 *     status = FT_OpenEx(<PVOID><uintptr_t>locId, FT_OPEN_BY_LOCATION, &handle)
 *     if status == FT_OK:             # <<<<<<<<<<<<<<
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":166
 *     if status == FT_OK:
 *         return FT4222(<uintptr_t>handle, update=False)
 *     raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_5, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 166, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":149
 *     raise FT2XXDeviceError, status
 * 
 * def openByLocation(locId):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by location
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.openByLocation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":177
 * 
 *     @staticmethod
 *     cdef wrap(FT_HANDLE h):             # <<<<<<<<<<<<<<
 *         cdef _closer closer = _closer()
 *         closer._handle = h
 */

static PyObject *__pyx_f_6ft4222_6ft4222_7_closer_wrap(FT_HANDLE __pyx_v_h) {
  struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_closer = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("wrap", 0);

  /* "ft4222/ft4222.pyx":178
 *     @staticmethod
 *     cdef wrap(FT_HANDLE h):
 *         cdef _closer closer = _closer()             # <<<<<<<<<<<<<<
 *         closer._handle = h
 *         return closer
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_6ft4222_6ft4222__closer)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_closer = ((struct __pyx_obj_6ft4222_6ft4222__closer *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":179
 *     cdef wrap(FT_HANDLE h):
 *         cdef _closer closer = _closer()
 *         closer._handle = h             # <<<<<<<<<<<<<<
 *         return closer
 * 
 */
  __pyx_v_closer->_handle = __pyx_v_h;

  /* "ft4222/ft4222.pyx":180
 *         cdef _closer closer = _closer()
 *         closer._handle = h
 *         return closer             # <<<<<<<<<<<<<<
 * 
 *     def close(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_closer));
  __pyx_r = ((PyObject *)__pyx_v_closer);
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":177
 * 
 *     @staticmethod
 *     cdef wrap(FT_HANDLE h):             # <<<<<<<<<<<<<<
 *         cdef _closer closer = _closer()
 *         closer._handle = h
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222._closer.wrap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_closer);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":182
 *         return closer
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         FT4222_UnInitialize(self._handle)
 *         FT_Close(self._handle)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_1close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_1close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_7_closer_close(((struct __pyx_obj_6ft4222_6ft4222__closer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_close(struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close", 0);

  /* "ft4222/ft4222.pyx":183
 * 
 *     def close(self):
 *         FT4222_UnInitialize(self._handle)             # <<<<<<<<<<<<<<
 *         FT_Close(self._handle)
 * 
 */
  (void)(FT4222_UnInitialize(__pyx_v_self->_handle));

  /* "ft4222/ft4222.pyx":184
 *     def close(self):
 *         FT4222_UnInitialize(self._handle)
 *         FT_Close(self._handle)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (void)(FT_Close(__pyx_v_self->_handle));

  /* "ft4222/ft4222.pyx":182
 *         return closer
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         FT4222_UnInitialize(self._handle)
 *         FT_Close(self._handle)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_7_closer_2__reduce_cython__(((struct __pyx_obj_6ft4222_6ft4222__closer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222._closer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_7_closer_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_7_closer_4__setstate_cython__(((struct __pyx_obj_6ft4222_6ft4222__closer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_7_closer_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222__closer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222._closer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":194
 *     cdef object _close
 * 
 *     def __init__(self, handle, update=True):             # <<<<<<<<<<<<<<
 *         self._handle = <FT_HANDLE><uintptr_t>handle
 *         self._chip_version = 0
 */

/* Python wrapper */
static int __pyx_pw_6ft4222_6ft4222_6FT4222_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ft4222_6ft4222_6FT4222_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_handle = 0;
  CYTHON_UNUSED PyObject *__pyx_v_update = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_handle,&__pyx_n_s_update,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_update);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 194, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_handle = values[0];
    __pyx_v_update = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 194, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222___init__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_handle, __pyx_v_update);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ft4222_6ft4222_6FT4222___init__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_handle, CYTHON_UNUSED PyObject *__pyx_v_update) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  uintptr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ft4222/ft4222.pyx":195
 * 
 *     def __init__(self, handle, update=True):
 *         self._handle = <FT_HANDLE><uintptr_t>handle             # <<<<<<<<<<<<<<
 *         self._chip_version = 0
 *         self._dll_version = 0
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_1 == ((uintptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 195, __pyx_L1_error)
  __pyx_v_self->_handle = ((FT_HANDLE)((uintptr_t)__pyx_t_1));

  /* "ft4222/ft4222.pyx":196
 *     def __init__(self, handle, update=True):
 *         self._handle = <FT_HANDLE><uintptr_t>handle
 *         self._chip_version = 0             # <<<<<<<<<<<<<<
 *         self._dll_version = 0
 *         self._get_version()
 */
  __pyx_v_self->_chip_version = 0;

  /* "ft4222/ft4222.pyx":197
 *         self._handle = <FT_HANDLE><uintptr_t>handle
 *         self._chip_version = 0
 *         self._dll_version = 0             # <<<<<<<<<<<<<<
 *         self._get_version()
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 */
  __pyx_v_self->_dll_version = 0;

  /* "ft4222/ft4222.pyx":198
 *         self._chip_version = 0
 *         self._dll_version = 0
 *         self._get_version()             # <<<<<<<<<<<<<<
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 * 
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 *)__pyx_v_self->__pyx_vtab)->_get_version(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":199
 *         self._dll_version = 0
 *         self._get_version()
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)             # <<<<<<<<<<<<<<
 * 
 *     def close(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_weakref_finalize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_f_6ft4222_6ft4222_7_closer_wrap(__pyx_v_self->_handle); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_close); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_self), __pyx_t_5};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_self), __pyx_t_5};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, ((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_close);
  __Pyx_DECREF(__pyx_v_self->_close);
  __pyx_v_self->_close = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":194
 *     cdef object _close
 * 
 *     def __init__(self, handle, update=True):             # <<<<<<<<<<<<<<
 *         self._handle = <FT_HANDLE><uintptr_t>handle
 *         self._chip_version = 0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":201
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_3close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_2close[] = "Closes the device.";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_3close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_2close(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_2close(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);

  /* "ft4222/ft4222.pyx":203
 *     def close(self):
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)             # <<<<<<<<<<<<<<
 *         status = FT_Close(self._handle)
 *         if status != FT4222_OK:
 */
  (void)(FT4222_UnInitialize(__pyx_v_self->_handle));

  /* "ft4222/ft4222.pyx":204
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)
 *         status = FT_Close(self._handle)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT_Close(__pyx_v_self->_handle);

  /* "ft4222/ft4222.pyx":205
 *         FT4222_UnInitialize(self._handle)
 *         status = FT_Close(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 *         self._handle = NULL
 */
  __pyx_t_1 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":206
 *         status = FT_Close(self._handle)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 *         self._handle = NULL
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 206, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 206, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 206, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":205
 *         FT4222_UnInitialize(self._handle)
 *         status = FT_Close(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 *         self._handle = NULL
 */
  }

  /* "ft4222/ft4222.pyx":207
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 *         self._handle = NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_version(self):
 */
  __pyx_v_self->_handle = NULL;

  /* "ft4222/ft4222.pyx":201
 *         self._close = weakref_finalize(self, _closer.wrap(self._handle).close)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Closes the device."""
 *         FT4222_UnInitialize(self._handle)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":209
 *         self._handle = NULL
 * 
 *     cdef _get_version(self):             # <<<<<<<<<<<<<<
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)
 */

static PyObject *__pyx_f_6ft4222_6ft4222_6FT4222__get_version(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_Version __pyx_v_ver;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  DWORD __pyx_t_2;
  __Pyx_RefNannySetupContext("_get_version", 0);

  /* "ft4222/ft4222.pyx":211
 *     cdef _get_version(self):
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             self._chip_version = ver.chipVersion
 */
  __pyx_v_status = FT4222_GetVersion(__pyx_v_self->_handle, (&__pyx_v_ver));

  /* "ft4222/ft4222.pyx":212
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             self._chip_version = ver.chipVersion
 *             self._dll_version = ver.dllVersion
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":213
 *         status = FT4222_GetVersion(self._handle, &ver)
 *         if status == FT4222_OK:
 *             self._chip_version = ver.chipVersion             # <<<<<<<<<<<<<<
 *             self._dll_version = ver.dllVersion
 * 
 */
    __pyx_t_2 = __pyx_v_ver.chipVersion;
    __pyx_v_self->_chip_version = __pyx_t_2;

    /* "ft4222/ft4222.pyx":214
 *         if status == FT4222_OK:
 *             self._chip_version = ver.chipVersion
 *             self._dll_version = ver.dllVersion             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __pyx_t_2 = __pyx_v_ver.dllVersion;
    __pyx_v_self->_dll_version = __pyx_t_2;

    /* "ft4222/ft4222.pyx":212
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             self._chip_version = ver.chipVersion
 *             self._dll_version = ver.dllVersion
 */
  }

  /* "ft4222/ft4222.pyx":209
 *         self._handle = NULL
 * 
 *     cdef _get_version(self):             # <<<<<<<<<<<<<<
 *         cdef FT4222_Version ver
 *         status = FT4222_GetVersion(self._handle, &ver)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":217
 * 
 *     @property
 *     def chipVersion(self) -> int:             # <<<<<<<<<<<<<<
 *         """Chip version as number"""
 *         return self._chip_version
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_11chipVersion_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_11chipVersion_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_11chipVersion___get__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_11chipVersion___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ft4222/ft4222.pyx":219
 *     def chipVersion(self) -> int:
 *         """Chip version as number"""
 *         return self._chip_version             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_DWORD(__pyx_v_self->_chip_version); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":217
 * 
 *     @property
 *     def chipVersion(self) -> int:             # <<<<<<<<<<<<<<
 *         """Chip version as number"""
 *         return self._chip_version
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.chipVersion.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":222
 * 
 *     @property
 *     def libVersion(self) -> int:             # <<<<<<<<<<<<<<
 *         """Library version as number"""
 *         return self._dll_version
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_10libVersion_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_10libVersion_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_10libVersion___get__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_10libVersion___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ft4222/ft4222.pyx":224
 *     def libVersion(self) -> int:
 *         """Library version as number"""
 *         return self._dll_version             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_DWORD(__pyx_v_self->_dll_version); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":222
 * 
 *     @property
 *     def libVersion(self) -> int:             # <<<<<<<<<<<<<<
 *         """Library version as number"""
 *         return self._dll_version
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.libVersion.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":227
 * 
 *     @property
 *     def chipRevision(self) -> str:             # <<<<<<<<<<<<<<
 *         """The revision of the chip in human readable format"""
 *         try:
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_12chipRevision_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_12chipRevision_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_12chipRevision___get__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_12chipRevision___get__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ft4222/ft4222.pyx":229
 *     def chipRevision(self) -> str:
 *         """The revision of the chip in human readable format"""
 *         try:             # <<<<<<<<<<<<<<
 *             return __chip_rev_map[self._chip_version]
 *         except KeyError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "ft4222/ft4222.pyx":230
 *         """The revision of the chip in human readable format"""
 *         try:
 *             return __chip_rev_map[self._chip_version]             # <<<<<<<<<<<<<<
 *         except KeyError:
 *             return "Rev. unknown"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_chip_rev_map); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_4, __pyx_v_self->_chip_version, DWORD, 0, __Pyx_PyInt_From_DWORD, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 230, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_5;
      __pyx_t_5 = 0;
      goto __pyx_L7_try_return;

      /* "ft4222/ft4222.pyx":229
 *     def chipRevision(self) -> str:
 *         """The revision of the chip in human readable format"""
 *         try:             # <<<<<<<<<<<<<<
 *             return __chip_rev_map[self._chip_version]
 *         except KeyError:
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "ft4222/ft4222.pyx":231
 *         try:
 *             return __chip_rev_map[self._chip_version]
 *         except KeyError:             # <<<<<<<<<<<<<<
 *             return "Rev. unknown"
 * 
 */
    __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_6) {
      __Pyx_AddTraceback("ft4222.ft4222.FT4222.chipRevision.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_4, &__pyx_t_7) < 0) __PYX_ERR(0, 231, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_7);

      /* "ft4222/ft4222.pyx":232
 *             return __chip_rev_map[self._chip_version]
 *         except KeyError:
 *             return "Rev. unknown"             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_Rev_unknown);
      __pyx_r = __pyx_kp_u_Rev_unknown;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "ft4222/ft4222.pyx":229
 *     def chipRevision(self) -> str:
 *         """The revision of the chip in human readable format"""
 *         try:             # <<<<<<<<<<<<<<
 *             return __chip_rev_map[self._chip_version]
 *         except KeyError:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "ft4222/ft4222.pyx":227
 * 
 *     @property
 *     def chipRevision(self) -> str:             # <<<<<<<<<<<<<<
 *         """The revision of the chip in human readable format"""
 *         try:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.chipRevision.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":234
 *             return "Rev. unknown"
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_4__repr__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_4__repr__(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "ft4222/ft4222.pyx":235
 * 
 *     def __repr__(self):
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)             # <<<<<<<<<<<<<<
 * 
 *     def setTimeouts(self, read_timeout, write_timeout):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_DWORD(__pyx_v_self->_chip_version); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_chipRevision); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_DWORD(__pyx_v_self->_dll_version); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ft4222/ft4222.pyx":234
 *             return "Rev. unknown"
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":237
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 *     def setTimeouts(self, read_timeout, write_timeout):             # <<<<<<<<<<<<<<
 *         """Set the read and write timeouts
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_7setTimeouts(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_6setTimeouts[] = "Set the read and write timeouts\n\n        Args:\n            read_timeout (int): Read timeout in milliseconds.\n            write_timeout (int): Write timeout in milliseconds.\n\n        Raises:\n            FT2XXDeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_7setTimeouts(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read_timeout = 0;
  PyObject *__pyx_v_write_timeout = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setTimeouts (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read_timeout,&__pyx_n_s_write_timeout,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_read_timeout)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_write_timeout)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("setTimeouts", 1, 2, 2, 1); __PYX_ERR(0, 237, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "setTimeouts") < 0)) __PYX_ERR(0, 237, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_read_timeout = values[0];
    __pyx_v_write_timeout = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setTimeouts", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 237, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setTimeouts", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_6setTimeouts(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_read_timeout, __pyx_v_write_timeout);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_6setTimeouts(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_read_timeout, PyObject *__pyx_v_write_timeout) {
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  ULONG __pyx_t_1;
  ULONG __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setTimeouts", 0);

  /* "ft4222/ft4222.pyx":248
 * 
 *         """
 *         status = FT_SetTimeouts(self._handle, read_timeout, write_timeout)             # <<<<<<<<<<<<<<
 *         if status != FT_OK:
 *             raise FT2XXDeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_ULONG(__pyx_v_read_timeout); if (unlikely((__pyx_t_1 == ((ULONG)-1)) && PyErr_Occurred())) __PYX_ERR(0, 248, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_ULONG(__pyx_v_write_timeout); if (unlikely((__pyx_t_2 == ((ULONG)-1)) && PyErr_Occurred())) __PYX_ERR(0, 248, __pyx_L1_error)
  __pyx_v_status = FT_SetTimeouts(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2);

  /* "ft4222/ft4222.pyx":249
 *         """
 *         status = FT_SetTimeouts(self._handle, read_timeout, write_timeout)
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT2XXDeviceError, status
 * 
 */
  __pyx_t_3 = ((__pyx_v_status != FT_OK) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "ft4222/ft4222.pyx":250
 *         status = FT_SetTimeouts(self._handle, read_timeout, write_timeout)
 *         if status != FT_OK:
 *             raise FT2XXDeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def setClock(self, clk):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_4, __pyx_t_5, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 250, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":249
 *         """
 *         status = FT_SetTimeouts(self._handle, read_timeout, write_timeout)
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT2XXDeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":237
 *         return "FT4222: chipVersion: 0x{:x} ({:s}), libVersion: 0x{:x}".format(self._chip_version, self.chipRevision, self._dll_version)
 * 
 *     def setTimeouts(self, read_timeout, write_timeout):             # <<<<<<<<<<<<<<
 *         """Set the read and write timeouts
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setTimeouts", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":252
 *             raise FT2XXDeviceError, status
 * 
 *     def setClock(self, clk):             # <<<<<<<<<<<<<<
 *         """Set the system clock
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_9setClock(PyObject *__pyx_v_self, PyObject *__pyx_v_clk); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_8setClock[] = "Set the system clock\n\n        Args:\n            clk (:obj:`ft4222.SysClock`): Desired clock\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_9setClock(PyObject *__pyx_v_self, PyObject *__pyx_v_clk) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setClock (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_8setClock(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_clk));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_8setClock(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_clk) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  FT4222_ClockRate __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setClock", 0);

  /* "ft4222/ft4222.pyx":262
 * 
 *         """
 *         status = FT4222_SetClock(self._handle, clk)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((FT4222_ClockRate)__Pyx_PyInt_As_FT4222_ClockRate(__pyx_v_clk)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 262, __pyx_L1_error)
  __pyx_v_status = FT4222_SetClock(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":263
 *         """
 *         status = FT4222_SetClock(self._handle, clk)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":264
 *         status = FT4222_SetClock(self._handle, clk)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def getClock(self):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 264, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":263
 *         """
 *         status = FT4222_SetClock(self._handle, clk)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":252
 *             raise FT2XXDeviceError, status
 * 
 *     def setClock(self, clk):             # <<<<<<<<<<<<<<
 *         """Set the system clock
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setClock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":266
 *             raise FT4222DeviceError, status
 * 
 *     def getClock(self):             # <<<<<<<<<<<<<<
 *         """Get the system clock
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_11getClock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_10getClock[] = "Get the system clock\n\n        Returns:\n            :obj:`ft4222.SysClock`: Clock\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_11getClock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getClock (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_10getClock(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_10getClock(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_ClockRate __pyx_v_clk;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getClock", 0);

  /* "ft4222/ft4222.pyx":277
 *         """
 *         cdef FT4222_ClockRate clk
 *         status = FT4222_GetClock(self._handle, &clk)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return SysClock(clk)
 */
  __pyx_v_status = FT4222_GetClock(__pyx_v_self->_handle, (&__pyx_v_clk));

  /* "ft4222/ft4222.pyx":278
 *         cdef FT4222_ClockRate clk
 *         status = FT4222_GetClock(self._handle, &clk)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return SysClock(clk)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":279
 *         status = FT4222_GetClock(self._handle, &clk)
 *         if status == FT4222_OK:
 *             return SysClock(clk)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SysClock); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_ClockRate(__pyx_v_clk); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":278
 *         cdef FT4222_ClockRate clk
 *         status = FT4222_GetClock(self._handle, &clk)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return SysClock(clk)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":280
 *         if status == FT4222_OK:
 *             return SysClock(clk)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def setSuspendOut(self, enable):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 280, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":266
 *             raise FT4222DeviceError, status
 * 
 *     def getClock(self):             # <<<<<<<<<<<<<<
 *         """Get the system clock
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.getClock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":282
 *         raise FT4222DeviceError, status
 * 
 *     def setSuspendOut(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable, suspend out, which will emit a signal when FT4222H enters suspend mode.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_13setSuspendOut(PyObject *__pyx_v_self, PyObject *__pyx_v_enable); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_12setSuspendOut[] = "Enable or disable, suspend out, which will emit a signal when FT4222H enters suspend mode.\n\n        Args:\n            enable (bool): TRUE to enable suspend out and configure GPIO2 as an output pin for emitting a signal when suspended. FALSE to switch back to GPIO2\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_13setSuspendOut(PyObject *__pyx_v_self, PyObject *__pyx_v_enable) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setSuspendOut (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_12setSuspendOut(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_enable));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_12setSuspendOut(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_enable) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  BOOL __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setSuspendOut", 0);

  /* "ft4222/ft4222.pyx":292
 * 
 *         """
 *         status = FT4222_SetSuspendOut(self._handle, enable)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_BOOL(__pyx_v_enable); if (unlikely((__pyx_t_1 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 292, __pyx_L1_error)
  __pyx_v_status = FT4222_SetSuspendOut(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":293
 *         """
 *         status = FT4222_SetSuspendOut(self._handle, enable)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":294
 *         status = FT4222_SetSuspendOut(self._handle, enable)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def setWakeUpInterrupt(self, enable):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 294, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":293
 *         """
 *         status = FT4222_SetSuspendOut(self._handle, enable)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":282
 *         raise FT4222DeviceError, status
 * 
 *     def setSuspendOut(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable, suspend out, which will emit a signal when FT4222H enters suspend mode.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setSuspendOut", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":296
 *             raise FT4222DeviceError, status
 * 
 *     def setWakeUpInterrupt(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable the wakeup/interrupt
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_15setWakeUpInterrupt(PyObject *__pyx_v_self, PyObject *__pyx_v_enable); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt[] = "Enable or disable the wakeup/interrupt\n\n        Args:\n            enable (bool): True to configure GPIO3 as an input pin for wakeup/interrupt. FALSE to switch back to GPIO3.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_15setWakeUpInterrupt(PyObject *__pyx_v_self, PyObject *__pyx_v_enable) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setWakeUpInterrupt (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_enable));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_enable) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  BOOL __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setWakeUpInterrupt", 0);

  /* "ft4222/ft4222.pyx":306
 * 
 *         """
 *         status = FT4222_SetWakeUpInterrupt(self._handle, enable)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_BOOL(__pyx_v_enable); if (unlikely((__pyx_t_1 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 306, __pyx_L1_error)
  __pyx_v_status = FT4222_SetWakeUpInterrupt(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":307
 *         """
 *         status = FT4222_SetWakeUpInterrupt(self._handle, enable)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":308
 *         status = FT4222_SetWakeUpInterrupt(self._handle, enable)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 308, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":307
 *         """
 *         status = FT4222_SetWakeUpInterrupt(self._handle, enable)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":296
 *             raise FT4222DeviceError, status
 * 
 *     def setWakeUpInterrupt(self, enable):             # <<<<<<<<<<<<<<
 *         """Enable or disable the wakeup/interrupt
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.setWakeUpInterrupt", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":311
 * 
 * 
 *     def vendorCmdGet(self, req, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Vendor get command"""
 *         cdef:
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_17vendorCmdGet(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_16vendorCmdGet[] = "Vendor get command";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_17vendorCmdGet(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_req = 0;
  PyObject *__pyx_v_bytesToRead = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("vendorCmdGet (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_req,&__pyx_n_s_bytesToRead,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_req)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_bytesToRead)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("vendorCmdGet", 1, 2, 2, 1); __PYX_ERR(0, 311, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "vendorCmdGet") < 0)) __PYX_ERR(0, 311, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_req = values[0];
    __pyx_v_bytesToRead = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("vendorCmdGet", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 311, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.vendorCmdGet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_16vendorCmdGet(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_req, __pyx_v_bytesToRead);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_16vendorCmdGet(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_req, PyObject *__pyx_v_bytesToRead) {
  arrayobject *__pyx_v_buf = 0;
  FT_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  UCHAR __pyx_t_5;
  USHORT __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("vendorCmdGet", 0);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":314
 *         """Vendor get command"""
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *         resize(buf, bytesToRead)
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 314, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":315
 *         cdef:
 *             array[uint8] buf = array('B', [])
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 *         if status == FT_OK:
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 315, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 315, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":316
 *             array[uint8] buf = array('B', [])
 *         resize(buf, bytesToRead)
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)             # <<<<<<<<<<<<<<
 *         if status == FT_OK:
 *             return bytes(buf)
 */
  __pyx_t_5 = __Pyx_PyInt_As_UCHAR(__pyx_v_req); if (unlikely((__pyx_t_5 == ((UCHAR)-1)) && PyErr_Occurred())) __PYX_ERR(0, 316, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_USHORT(__pyx_v_bytesToRead); if (unlikely((__pyx_t_6 == ((USHORT)-1)) && PyErr_Occurred())) __PYX_ERR(0, 316, __pyx_L1_error)
  __pyx_v_status = FT_VendorCmdGet(__pyx_v_self->_handle, __pyx_t_5, __pyx_v_buf->data.as_uchars, __pyx_t_6);

  /* "ft4222/ft4222.pyx":317
 *         resize(buf, bytesToRead)
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_7 = ((__pyx_v_status == FT_OK) != 0);
  if (__pyx_t_7) {

    /* "ft4222/ft4222.pyx":318
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 *         if status == FT_OK:
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":317
 *         resize(buf, bytesToRead)
 *         status = FT_VendorCmdGet(self._handle, req, buf.data.as_uchars, bytesToRead)
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":319
 *         if status == FT_OK:
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def vendorCmdSet(self, req, data):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 319, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":311
 * 
 * 
 *     def vendorCmdGet(self, req, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Vendor get command"""
 *         cdef:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.vendorCmdGet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":321
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdSet(self, req, data):             # <<<<<<<<<<<<<<
 *         """Vendor set command"""
 *         if isinstance(data, int):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_19vendorCmdSet(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_18vendorCmdSet[] = "Vendor set command";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_19vendorCmdSet(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_req = 0;
  PyObject *__pyx_v_data = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("vendorCmdSet (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_req,&__pyx_n_s_data,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_req)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("vendorCmdSet", 1, 2, 2, 1); __PYX_ERR(0, 321, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "vendorCmdSet") < 0)) __PYX_ERR(0, 321, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_req = values[0];
    __pyx_v_data = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("vendorCmdSet", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 321, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.vendorCmdSet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_18vendorCmdSet(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_req, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_18vendorCmdSet(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_req, PyObject *__pyx_v_data) {
  uint8 *__pyx_v_cdata;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  uint8 *__pyx_t_6;
  UCHAR __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("vendorCmdSet", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":323
 *     def vendorCmdSet(self, req, data):
 *         """Vendor set command"""
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":324
 *         """Vendor set command"""
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_data);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":323
 *     def vendorCmdSet(self, req, data):
 *         """Vendor set command"""
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":325
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_data); 
  __pyx_t_5 = (__pyx_t_1 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":326
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 326, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":325
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":329
 *         cdef:
 *             uint16 bytesSent
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))
 *         if status != FT_OK:
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 329, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_6;

  /* "ft4222/ft4222.pyx":330
 *             uint16 bytesSent
 *             uint8* cdata = data
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))             # <<<<<<<<<<<<<<
 *         if status != FT_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_7 = __Pyx_PyInt_As_UCHAR(__pyx_v_req); if (unlikely((__pyx_t_7 == ((UCHAR)-1)) && PyErr_Occurred())) __PYX_ERR(0, 330, __pyx_L1_error)
  __pyx_t_8 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 330, __pyx_L1_error)
  __pyx_v_status = FT_VendorCmdSet(__pyx_v_self->_handle, __pyx_t_7, __pyx_v_cdata, __pyx_t_8);

  /* "ft4222/ft4222.pyx":331
 *             uint8* cdata = data
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_1 = ((__pyx_v_status != FT_OK) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":332
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))
 *         if status != FT_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_4, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 332, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":331
 *             uint8* cdata = data
 *         status = FT_VendorCmdSet(self._handle, req, cdata, len(data))
 *         if status != FT_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":321
 *         raise FT4222DeviceError, status
 * 
 *     def vendorCmdSet(self, req, data):             # <<<<<<<<<<<<<<
 *         """Vendor set command"""
 *         if isinstance(data, int):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.vendorCmdSet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":335
 * 
 * 
 *     def gpio_Init(self, *args, gpio0=Dir.INPUT, gpio1=Dir.INPUT, gpio2=Dir.INPUT, gpio3=Dir.INPUT):             # <<<<<<<<<<<<<<
 *         """Initialize the GPIO interface.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_21gpio_Init(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_20gpio_Init[] = "Initialize the GPIO interface.\n\n        Args:\n            *args (:obj:`list` of :obj:`ft4222.GPIO.Dir`, optional): List containing a direction for each port.\n            gpio0 (:obj:`ft4222.GPIO.Dir`, optional): Direction of gpio0\n            gpio1 (:obj:`ft4222.GPIO.Dir`, optional): Direction of gpio1\n            gpio2 (:obj:`ft4222.GPIO.Dir`, optional): Direction of gpio2\n            gpio3 (:obj:`ft4222.GPIO.Dir`, optional): Direction of gpio3\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_21gpio_Init(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_gpio0 = 0;
  PyObject *__pyx_v_gpio1 = 0;
  PyObject *__pyx_v_gpio2 = 0;
  PyObject *__pyx_v_gpio3 = 0;
  PyObject *__pyx_v_args = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_Init (wrapper)", 0);
  if (PyTuple_GET_SIZE(__pyx_args) > 0) {
    __pyx_v_args = PyTuple_GetSlice(__pyx_args, 0, PyTuple_GET_SIZE(__pyx_args));
    if (unlikely(!__pyx_v_args)) {
      __Pyx_RefNannyFinishContext();
      return NULL;
    }
    __Pyx_GOTREF(__pyx_v_args);
  } else {
    __pyx_v_args = __pyx_empty_tuple; __Pyx_INCREF(__pyx_empty_tuple);
  }
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_gpio0,&__pyx_n_s_gpio1,&__pyx_n_s_gpio2,&__pyx_n_s_gpio3,0};
    PyObject* values[4] = {0,0,0,0};
    values[0] = __pyx_k__4;
    values[1] = __pyx_k__5;
    values[2] = __pyx_k__6;
    values[3] = __pyx_k__7;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        default:
        case  0: break;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      if (kw_args > 0 && likely(kw_args <= 4)) {
        Py_ssize_t index;
        for (index = 0; index < 4 && kw_args > 0; index++) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, 0, "gpio_Init") < 0)) __PYX_ERR(0, 335, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) < 0) {
      goto __pyx_L5_argtuple_error;
    } else {
    }
    __pyx_v_gpio0 = values[0];
    __pyx_v_gpio1 = values[1];
    __pyx_v_gpio2 = values[2];
    __pyx_v_gpio3 = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_Init", 0, 0, 0, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 335, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_args); __pyx_v_args = 0;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_20gpio_Init(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_gpio0, __pyx_v_gpio1, __pyx_v_gpio2, __pyx_v_gpio3, __pyx_v_args);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_20gpio_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_gpio0, PyObject *__pyx_v_gpio1, PyObject *__pyx_v_gpio2, PyObject *__pyx_v_gpio3, PyObject *__pyx_v_args) {
  GPIO_Dir __pyx_v_ioDir[4];
  Py_ssize_t __pyx_v_i;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  GPIO_Dir __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_Init", 0);

  /* "ft4222/ft4222.pyx":351
 *         cdef:
 *             GPIO_Dir ioDir[4]
 *         if len(args) > 0:             # <<<<<<<<<<<<<<
 *             for i in xrange(len(args)):
 *                 ioDir[i] = args[i]
 */
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 351, __pyx_L1_error)
  __pyx_t_2 = ((__pyx_t_1 > 0) != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":352
 *             GPIO_Dir ioDir[4]
 *         if len(args) > 0:
 *             for i in xrange(len(args)):             # <<<<<<<<<<<<<<
 *                 ioDir[i] = args[i]
 *             for i in xrange(len(args), 4 - len(args)):
 */
    __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 352, __pyx_L1_error)
    __pyx_t_3 = __pyx_t_1;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "ft4222/ft4222.pyx":353
 *         if len(args) > 0:
 *             for i in xrange(len(args)):
 *                 ioDir[i] = args[i]             # <<<<<<<<<<<<<<
 *             for i in xrange(len(args), 4 - len(args)):
 *                 ioDir[i] = GPIO_INPUT
 */
      __pyx_t_5 = __Pyx_GetItemInt_Tuple(__pyx_v_args, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_t_5)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      (__pyx_v_ioDir[__pyx_v_i]) = __pyx_t_6;
    }

    /* "ft4222/ft4222.pyx":354
 *             for i in xrange(len(args)):
 *                 ioDir[i] = args[i]
 *             for i in xrange(len(args), 4 - len(args)):             # <<<<<<<<<<<<<<
 *                 ioDir[i] = GPIO_INPUT
 *         else:
 */
    __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 354, __pyx_L1_error)
    __pyx_t_3 = (4 - __pyx_t_1);
    __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 354, __pyx_L1_error)
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_7 = __pyx_t_1; __pyx_t_7 < __pyx_t_4; __pyx_t_7+=1) {
      __pyx_v_i = __pyx_t_7;

      /* "ft4222/ft4222.pyx":355
 *                 ioDir[i] = args[i]
 *             for i in xrange(len(args), 4 - len(args)):
 *                 ioDir[i] = GPIO_INPUT             # <<<<<<<<<<<<<<
 *         else:
 *             ioDir[0] = gpio0
 */
      (__pyx_v_ioDir[__pyx_v_i]) = GPIO_INPUT;
    }

    /* "ft4222/ft4222.pyx":351
 *         cdef:
 *             GPIO_Dir ioDir[4]
 *         if len(args) > 0:             # <<<<<<<<<<<<<<
 *             for i in xrange(len(args)):
 *                 ioDir[i] = args[i]
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":357
 *                 ioDir[i] = GPIO_INPUT
 *         else:
 *             ioDir[0] = gpio0             # <<<<<<<<<<<<<<
 *             ioDir[1] = gpio1
 *             ioDir[2] = gpio2
 */
  /*else*/ {
    __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_v_gpio0)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 357, __pyx_L1_error)
    (__pyx_v_ioDir[0]) = __pyx_t_6;

    /* "ft4222/ft4222.pyx":358
 *         else:
 *             ioDir[0] = gpio0
 *             ioDir[1] = gpio1             # <<<<<<<<<<<<<<
 *             ioDir[2] = gpio2
 *             ioDir[3] = gpio3
 */
    __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_v_gpio1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 358, __pyx_L1_error)
    (__pyx_v_ioDir[1]) = __pyx_t_6;

    /* "ft4222/ft4222.pyx":359
 *             ioDir[0] = gpio0
 *             ioDir[1] = gpio1
 *             ioDir[2] = gpio2             # <<<<<<<<<<<<<<
 *             ioDir[3] = gpio3
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 */
    __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_v_gpio2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 359, __pyx_L1_error)
    (__pyx_v_ioDir[2]) = __pyx_t_6;

    /* "ft4222/ft4222.pyx":360
 *             ioDir[1] = gpio1
 *             ioDir[2] = gpio2
 *             ioDir[3] = gpio3             # <<<<<<<<<<<<<<
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 *         if status != FT4222_OK:
 */
    __pyx_t_6 = ((GPIO_Dir)__Pyx_PyInt_As_GPIO_Dir(__pyx_v_gpio3)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 360, __pyx_L1_error)
    (__pyx_v_ioDir[3]) = __pyx_t_6;
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":361
 *             ioDir[2] = gpio2
 *             ioDir[3] = gpio3
 *         status = FT4222_GPIO_Init(self._handle, ioDir)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT4222_GPIO_Init(__pyx_v_self->_handle, __pyx_v_ioDir);

  /* "ft4222/ft4222.pyx":362
 *             ioDir[3] = gpio3
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":363
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_Read(self, portNum):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_5, __pyx_t_8, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 363, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":362
 *             ioDir[3] = gpio3
 *         status = FT4222_GPIO_Init(self._handle, ioDir)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":335
 * 
 * 
 *     def gpio_Init(self, *args, gpio0=Dir.INPUT, gpio1=Dir.INPUT, gpio2=Dir.INPUT, gpio3=Dir.INPUT):             # <<<<<<<<<<<<<<
 *         """Initialize the GPIO interface.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":365
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_Read(self, portNum):             # <<<<<<<<<<<<<<
 *         """Read value from selected GPIO
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_23gpio_Read(PyObject *__pyx_v_self, PyObject *__pyx_v_portNum); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_22gpio_Read[] = "Read value from selected GPIO\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n\n        Returns:\n            bool: True if high, False otherwise\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_23gpio_Read(PyObject *__pyx_v_self, PyObject *__pyx_v_portNum) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_Read (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_22gpio_Read(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_portNum));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_22gpio_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum) {
  BOOL __pyx_v_value;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GPIO_Port __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_Read", 0);

  /* "ft4222/ft4222.pyx":380
 *         cdef:
 *             BOOL value
 *         status = FT4222_GPIO_Read(self._handle, portNum, &value)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return value
 */
  __pyx_t_1 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 380, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_Read(__pyx_v_self->_handle, __pyx_t_1, (&__pyx_v_value));

  /* "ft4222/ft4222.pyx":381
 *             BOOL value
 *         status = FT4222_GPIO_Read(self._handle, portNum, &value)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return value
 *         raise FT4222DeviceError, status
 */
  __pyx_t_2 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":382
 *         status = FT4222_GPIO_Read(self._handle, portNum, &value)
 *         if status == FT4222_OK:
 *             return value             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_BOOL(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 382, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":381
 *             BOOL value
 *         status = FT4222_GPIO_Read(self._handle, portNum, &value)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return value
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":383
 *         if status == FT4222_OK:
 *             return value
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_Write(self, portNum, value):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(0, 383, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":365
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_Read(self, portNum):             # <<<<<<<<<<<<<<
 *         """Read value from selected GPIO
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":385
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_Write(self, portNum, value):             # <<<<<<<<<<<<<<
 *         """Write value to given GPIO
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_25gpio_Write(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_24gpio_Write[] = "Write value to given GPIO\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n            value (bool): True for high, False for low\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_25gpio_Write(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_portNum = 0;
  PyObject *__pyx_v_value = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_Write (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_portNum,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_portNum)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gpio_Write", 1, 2, 2, 1); __PYX_ERR(0, 385, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "gpio_Write") < 0)) __PYX_ERR(0, 385, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_portNum = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_Write", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 385, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_24gpio_Write(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_portNum, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_24gpio_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_value) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GPIO_Port __pyx_t_1;
  BOOL __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_Write", 0);

  /* "ft4222/ft4222.pyx":396
 * 
 *         """
 *         status = FT4222_GPIO_Write(self._handle, portNum, value)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 396, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_BOOL(__pyx_v_value); if (unlikely((__pyx_t_2 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 396, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_Write(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2);

  /* "ft4222/ft4222.pyx":397
 *         """
 *         status = FT4222_GPIO_Write(self._handle, portNum, value)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_3 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "ft4222/ft4222.pyx":398
 *         status = FT4222_GPIO_Write(self._handle, portNum, value)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_SetInputTrigger(self, portNum, trigger):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 398, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 398, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_4, __pyx_t_5, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 398, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":397
 *         """
 *         status = FT4222_GPIO_Write(self._handle, portNum, value)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":385
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_Write(self, portNum, value):             # <<<<<<<<<<<<<<
 *         """Write value to given GPIO
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":400
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_SetInputTrigger(self, portNum, trigger):             # <<<<<<<<<<<<<<
 *         """Set software trigger conditions on the specified GPIO pin.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_27gpio_SetInputTrigger(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_26gpio_SetInputTrigger[] = "Set software trigger conditions on the specified GPIO pin.\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n            trigger (:obj:`ft4222.GPIO.Trigger`): Combination of trigger conditions\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_27gpio_SetInputTrigger(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_portNum = 0;
  PyObject *__pyx_v_trigger = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_SetInputTrigger (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_portNum,&__pyx_n_s_trigger,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_portNum)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_trigger)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gpio_SetInputTrigger", 1, 2, 2, 1); __PYX_ERR(0, 400, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "gpio_SetInputTrigger") < 0)) __PYX_ERR(0, 400, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_portNum = values[0];
    __pyx_v_trigger = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_SetInputTrigger", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 400, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_SetInputTrigger", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_26gpio_SetInputTrigger(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_portNum, __pyx_v_trigger);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_26gpio_SetInputTrigger(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_trigger) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GPIO_Port __pyx_t_1;
  GPIO_Trigger __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_SetInputTrigger", 0);

  /* "ft4222/ft4222.pyx":411
 * 
 *         """
 *         status = FT4222_GPIO_SetInputTrigger(self._handle, portNum, trigger)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 411, __pyx_L1_error)
  __pyx_t_2 = ((GPIO_Trigger)__Pyx_PyInt_As_GPIO_Trigger(__pyx_v_trigger)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 411, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_SetInputTrigger(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2);

  /* "ft4222/ft4222.pyx":412
 *         """
 *         status = FT4222_GPIO_SetInputTrigger(self._handle, portNum, trigger)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_3 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "ft4222/ft4222.pyx":413
 *         status = FT4222_GPIO_SetInputTrigger(self._handle, portNum, trigger)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_GetTriggerStatus(self, portNum):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_4, __pyx_t_5, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 413, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":412
 *         """
 *         status = FT4222_GPIO_SetInputTrigger(self._handle, portNum, trigger)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":400
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_SetInputTrigger(self, portNum, trigger):             # <<<<<<<<<<<<<<
 *         """Set software trigger conditions on the specified GPIO pin.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_SetInputTrigger", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":415
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_GetTriggerStatus(self, portNum):             # <<<<<<<<<<<<<<
 *         """Get the size of trigger event queue.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_29gpio_GetTriggerStatus(PyObject *__pyx_v_self, PyObject *__pyx_v_portNum); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_28gpio_GetTriggerStatus[] = "Get the size of trigger event queue.\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n\n        Returns:\n            int: Size of the trigger queue\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_29gpio_GetTriggerStatus(PyObject *__pyx_v_self, PyObject *__pyx_v_portNum) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_GetTriggerStatus (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_28gpio_GetTriggerStatus(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_portNum));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_28gpio_GetTriggerStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum) {
  uint16 __pyx_v_queueSize;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GPIO_Port __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_GetTriggerStatus", 0);

  /* "ft4222/ft4222.pyx":430
 *         cdef:
 *             uint16 queueSize
 *         status = FT4222_GPIO_GetTriggerStatus(self._handle, portNum, &queueSize)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return queueSize
 */
  __pyx_t_1 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 430, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_GetTriggerStatus(__pyx_v_self->_handle, __pyx_t_1, (&__pyx_v_queueSize));

  /* "ft4222/ft4222.pyx":431
 *             uint16 queueSize
 *         status = FT4222_GPIO_GetTriggerStatus(self._handle, portNum, &queueSize)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return queueSize
 *         raise FT4222DeviceError, status
 */
  __pyx_t_2 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":432
 *         status = FT4222_GPIO_GetTriggerStatus(self._handle, portNum, &queueSize)
 *         if status == FT4222_OK:
 *             return queueSize             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint16_t(__pyx_v_queueSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 432, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":431
 *             uint16 queueSize
 *         status = FT4222_GPIO_GetTriggerStatus(self._handle, portNum, &queueSize)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return queueSize
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":433
 *         if status == FT4222_OK:
 *             return queueSize
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def gpio_ReadTriggerQueue(self, portNum, readSize=None):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(0, 433, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":415
 *             raise FT4222DeviceError, status
 * 
 *     def gpio_GetTriggerStatus(self, portNum):             # <<<<<<<<<<<<<<
 *         """Get the size of trigger event queue.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_GetTriggerStatus", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":435
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_ReadTriggerQueue(self, portNum, readSize=None):             # <<<<<<<<<<<<<<
 *         """Get events recorded in the trigger event queue.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_31gpio_ReadTriggerQueue(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_30gpio_ReadTriggerQueue[] = "Get events recorded in the trigger event queue.\n\n        Args:\n            portNum (:obj:`ft4222.GPIO.Port`): GPIO Port number\n            readSize (:obj:`int`, optional): Size of the queue, :obj:`gpio_GetTriggerStatus()` gets called if omitted\n\n        Returns:\n            :obj:`list` of :obj:`ft4222.GPIO.Trigger`: Trigger queue as list\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_31gpio_ReadTriggerQueue(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_portNum = 0;
  PyObject *__pyx_v_readSize = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gpio_ReadTriggerQueue (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_portNum,&__pyx_n_s_readSize,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_portNum)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_readSize);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "gpio_ReadTriggerQueue") < 0)) __PYX_ERR(0, 435, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_portNum = values[0];
    __pyx_v_readSize = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gpio_ReadTriggerQueue", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 435, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_ReadTriggerQueue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_30gpio_ReadTriggerQueue(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_portNum, __pyx_v_readSize);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_30gpio_ReadTriggerQueue(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_portNum, PyObject *__pyx_v_readSize) {
  GPIO_Trigger *__pyx_v_events;
  uint16 __pyx_v_sizeRead;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  GPIO_Port __pyx_t_6;
  uint16 __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gpio_ReadTriggerQueue", 0);

  /* "ft4222/ft4222.pyx":449
 * 
 *         """
 *         if readSize == None:             # <<<<<<<<<<<<<<
 *             self.gpio_GetTriggerStatus(portNum)
 *         cdef:
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_readSize, Py_None, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":450
 *         """
 *         if readSize == None:
 *             self.gpio_GetTriggerStatus(portNum)             # <<<<<<<<<<<<<<
 *         cdef:
 *             GPIO_Trigger *events = <GPIO_Trigger*>alloca(portNum * sizeof(GPIO_Trigger))
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_gpio_GetTriggerStatus); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_portNum) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_portNum);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ft4222/ft4222.pyx":449
 * 
 *         """
 *         if readSize == None:             # <<<<<<<<<<<<<<
 *             self.gpio_GetTriggerStatus(portNum)
 *         cdef:
 */
  }

  /* "ft4222/ft4222.pyx":452
 *             self.gpio_GetTriggerStatus(portNum)
 *         cdef:
 *             GPIO_Trigger *events = <GPIO_Trigger*>alloca(portNum * sizeof(GPIO_Trigger))             # <<<<<<<<<<<<<<
 *             uint16 sizeRead
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t((sizeof(GPIO_Trigger))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_portNum, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_3); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_events = ((GPIO_Trigger *)alloca(__pyx_t_5));

  /* "ft4222/ft4222.pyx":454
 *             GPIO_Trigger *events = <GPIO_Trigger*>alloca(portNum * sizeof(GPIO_Trigger))
 *             uint16 sizeRead
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             res = []
 */
  __pyx_t_6 = ((GPIO_Port)__Pyx_PyInt_As_GPIO_Port(__pyx_v_portNum)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 454, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_uint16_t(__pyx_v_readSize); if (unlikely((__pyx_t_7 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 454, __pyx_L1_error)
  __pyx_v_status = FT4222_GPIO_ReadTriggerQueue(__pyx_v_self->_handle, __pyx_t_6, __pyx_v_events, __pyx_t_7, (&__pyx_v_sizeRead));

  /* "ft4222/ft4222.pyx":455
 *             uint16 sizeRead
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             res = []
 *             for i in xrange(readSize):
 */
  __pyx_t_2 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":456
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)
 *         if status == FT4222_OK:
 *             res = []             # <<<<<<<<<<<<<<
 *             for i in xrange(readSize):
 *                 res.append(Trigger(events[i]))
 */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_res = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":457
 *         if status == FT4222_OK:
 *             res = []
 *             for i in xrange(readSize):             # <<<<<<<<<<<<<<
 *                 res.append(Trigger(events[i]))
 *             return res
 */
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_xrange, __pyx_v_readSize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 457, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 457, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 457, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_3); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 457, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 457, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        } else {
          if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_3); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 457, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 457, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_9(__pyx_t_1);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 457, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "ft4222/ft4222.pyx":458
 *             res = []
 *             for i in xrange(readSize):
 *                 res.append(Trigger(events[i]))             # <<<<<<<<<<<<<<
 *             return res
 *         raise FT4222DeviceError, status
 */
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_Trigger); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 458, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 458, __pyx_L1_error)
      __pyx_t_11 = __Pyx_PyInt_From_GPIO_Trigger((__pyx_v_events[__pyx_t_10])); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 458, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_12)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_3 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_12, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_11);
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 458, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_res, __pyx_t_3); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 458, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "ft4222/ft4222.pyx":457
 *         if status == FT4222_OK:
 *             res = []
 *             for i in xrange(readSize):             # <<<<<<<<<<<<<<
 *                 res.append(Trigger(events[i]))
 *             return res
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ft4222/ft4222.pyx":459
 *             for i in xrange(readSize):
 *                 res.append(Trigger(events[i]))
 *             return res             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":455
 *             uint16 sizeRead
 *         status = FT4222_GPIO_ReadTriggerQueue(self._handle, portNum, events, readSize, &sizeRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             res = []
 *             for i in xrange(readSize):
 */
  }

  /* "ft4222/ft4222.pyx":460
 *                 res.append(Trigger(events[i]))
 *             return res
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_1, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 460, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":435
 *         raise FT4222DeviceError, status
 * 
 *     def gpio_ReadTriggerQueue(self, portNum, readSize=None):             # <<<<<<<<<<<<<<
 *         """Get events recorded in the trigger event queue.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.gpio_ReadTriggerQueue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":463
 * 
 * 
 *     def i2cMaster_Init(self, kbps=100):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an I2C master with the requested I2C speed.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_33i2cMaster_Init(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_32i2cMaster_Init[] = "Initialize the FT4222H as an I2C master with the requested I2C speed.\n\n        Args:\n            kbps (int): Speed in kb/s\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_33i2cMaster_Init(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kbps = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_Init (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_kbps,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_int_100);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_kbps);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "i2cMaster_Init") < 0)) __PYX_ERR(0, 463, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_kbps = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_Init", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 463, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_32i2cMaster_Init(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_kbps);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_32i2cMaster_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_kbps) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint32 __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  long __pyx_t_5;
  long __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_Init", 0);

  /* "ft4222/ft4222.pyx":473
 * 
 *         """
 *         status = FT4222_I2CMaster_Init(self._handle, kbps)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint32_t(__pyx_v_kbps); if (unlikely((__pyx_t_1 == ((uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 473, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_Init(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":474
 *         """
 *         status = FT4222_I2CMaster_Init(self._handle, kbps)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 *         # current version (v1.3) of ftdi's lib can only handle clock rates down to 60kHz
 */
  __pyx_t_2 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":475
 *         status = FT4222_I2CMaster_Init(self._handle, kbps)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 *         # current version (v1.3) of ftdi's lib can only handle clock rates down to 60kHz
 *         # although the chip can handle clock rates down to ~23.6kHz
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 475, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":474
 *         """
 *         status = FT4222_I2CMaster_Init(self._handle, kbps)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 *         # current version (v1.3) of ftdi's lib can only handle clock rates down to 60kHz
 */
  }

  /* "ft4222/ft4222.pyx":479
 *         # although the chip can handle clock rates down to ~23.6kHz
 *         # there's a undocumented ways to achieve this
 *         if kbps < 60:             # <<<<<<<<<<<<<<
 *             #             Operating Clock Freq
 *             # SCL Freq = -----------------------  | M = 6 or 8; N = 1, 2, 3, , 127
 */
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_kbps, __pyx_int_60, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 479, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":488
 *             #            M * SCL Freq               8 * kbps
 *             #
 *             n = max(min(int(round(24000000.0 / (8 * 1000 * kbps) - 1)), 127), 1)             # <<<<<<<<<<<<<<
 *             self.setClock(SysClock.CLK_24)
 *             self.vendorCmdSet(0x52, n)
 */
    __pyx_t_5 = 1;
    __pyx_t_6 = 0x7F;
    __pyx_t_4 = PyNumber_Multiply(__pyx_int_8000, __pyx_v_kbps); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyFloat_TrueDivideCObj(__pyx_float_24000000_0, __pyx_t_4, 24000000.0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyInt_SubtractObjC(__pyx_t_3, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_round, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyNumber_Int(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_7 = __Pyx_PyInt_From_long(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyObject_RichCompare(__pyx_t_7, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_2) {
      __pyx_t_8 = __Pyx_PyInt_From_long(__pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 488, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = __pyx_t_8;
      __pyx_t_8 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_3 = __pyx_t_4;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = __pyx_t_3;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = __Pyx_PyInt_From_long(__pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = PyObject_RichCompare(__pyx_t_8, __pyx_t_4, Py_GT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (__pyx_t_2) {
      __pyx_t_7 = __Pyx_PyInt_From_long(__pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 488, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_3 = __pyx_t_7;
      __pyx_t_7 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_3 = __pyx_t_4;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __pyx_t_3;
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_n = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":489
 *             #
 *             n = max(min(int(round(24000000.0 / (8 * 1000 * kbps) - 1)), 127), 1)
 *             self.setClock(SysClock.CLK_24)             # <<<<<<<<<<<<<<
 *             self.vendorCmdSet(0x52, n)
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setClock); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_SysClock); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_CLK_24); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_7, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":490
 *             n = max(min(int(round(24000000.0 / (8 * 1000 * kbps) - 1)), 127), 1)
 *             self.setClock(SysClock.CLK_24)
 *             self.vendorCmdSet(0x52, n)             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_Read(self, addr, bytesToRead):
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_vendorCmdSet); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 490, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = NULL;
    __pyx_t_9 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_9 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_int_82, __pyx_v_n};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_int_82, __pyx_v_n};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_8) {
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8); __pyx_t_8 = NULL;
      }
      __Pyx_INCREF(__pyx_int_82);
      __Pyx_GIVEREF(__pyx_int_82);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_9, __pyx_int_82);
      __Pyx_INCREF(__pyx_v_n);
      __Pyx_GIVEREF(__pyx_v_n);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_9, __pyx_v_n);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":479
 *         # although the chip can handle clock rates down to ~23.6kHz
 *         # there's a undocumented ways to achieve this
 *         if kbps < 60:             # <<<<<<<<<<<<<<
 *             #             Operating Clock Freq
 *             # SCL Freq = -----------------------  | M = 6 or 8; N = 1, 2, 3, , 127
 */
  }

  /* "ft4222/ft4222.pyx":463
 * 
 * 
 *     def i2cMaster_Init(self, kbps=100):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an I2C master with the requested I2C speed.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":492
 *             self.vendorCmdSet(0x52, n)
 * 
 *     def i2cMaster_Read(self, addr, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with START and STOP conditions.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_35i2cMaster_Read(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_34i2cMaster_Read[] = "Read data from the specified I2C slave device with START and STOP conditions.\n\n        Args:\n            addr (int): I2C slave address\n            bytesToRead (int): Number of bytes to read from slave\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_35i2cMaster_Read(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_addr = 0;
  PyObject *__pyx_v_bytesToRead = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_Read (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_addr,&__pyx_n_s_bytesToRead,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_addr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_bytesToRead)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_Read", 1, 2, 2, 1); __PYX_ERR(0, 492, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "i2cMaster_Read") < 0)) __PYX_ERR(0, 492, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_addr = values[0];
    __pyx_v_bytesToRead = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_Read", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 492, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_34i2cMaster_Read(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_addr, __pyx_v_bytesToRead);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_34i2cMaster_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_bytesToRead) {
  arrayobject *__pyx_v_buf = 0;
  uint16 __pyx_v_bytesRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  uint16 __pyx_t_5;
  uint16 __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_Read", 0);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":507
 *         """
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 507, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":509
 *             array[uint8] buf = array('B', [])
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 509, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 509, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":510
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)             # <<<<<<<<<<<<<<
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:
 */
  __pyx_t_5 = __Pyx_PyInt_As_uint16_t(__pyx_v_addr); if (unlikely((__pyx_t_5 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 510, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_6 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 510, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_Read(__pyx_v_self->_handle, __pyx_t_5, __pyx_v_buf->data.as_uchars, __pyx_t_6, (&__pyx_v_bytesRead));

  /* "ft4222/ft4222.pyx":511
 *         resize(buf, bytesToRead)
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytes(buf)
 */
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_bytesRead); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 511, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":512
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_7 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_7) {

    /* "ft4222/ft4222.pyx":513
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 513, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":512
 *         status = FT4222_I2CMaster_Read(self._handle, addr, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":514
 *         if status == FT4222_OK:
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_Write(self, addr, data):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 514, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":492
 *             self.vendorCmdSet(0x52, n)
 * 
 *     def i2cMaster_Read(self, addr, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with START and STOP conditions.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":516
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Write(self, addr, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with START and STOP conditions.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_37i2cMaster_Write(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_36i2cMaster_Write[] = "Write data to the specified I2C slave device with START and STOP conditions.\n\n        Args:\n            addr (int): I2C slave address\n            data (int, bytes, bytearray): Data to write to slave\n\n        Returns:\n            int: Bytes sent to slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_37i2cMaster_Write(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_addr = 0;
  PyObject *__pyx_v_data = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_Write (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_addr,&__pyx_n_s_data,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_addr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_Write", 1, 2, 2, 1); __PYX_ERR(0, 516, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "i2cMaster_Write") < 0)) __PYX_ERR(0, 516, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_addr = values[0];
    __pyx_v_data = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_Write", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 516, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_36i2cMaster_Write(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_addr, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_36i2cMaster_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_data) {
  uint16 __pyx_v_bytesSent;
  uint8 *__pyx_v_cdata;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  uint8 *__pyx_t_6;
  uint16 __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_Write", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":530
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":531
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 531, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_data);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 531, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":530
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":532
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_data); 
  __pyx_t_5 = (__pyx_t_1 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":533
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 533, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":532
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":536
 *         cdef:
 *             uint16 bytesSent
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 536, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_6;

  /* "ft4222/ft4222.pyx":537
 *             uint16 bytesSent
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytesSent
 */
  __pyx_t_7 = __Pyx_PyInt_As_uint16_t(__pyx_v_addr); if (unlikely((__pyx_t_7 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 537, __pyx_L1_error)
  __pyx_t_8 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 537, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_Write(__pyx_v_self->_handle, __pyx_t_7, __pyx_v_cdata, __pyx_t_8, (&__pyx_v_bytesSent));

  /* "ft4222/ft4222.pyx":538
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":539
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:
 *             return bytesSent             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyInt_From_uint16_t(__pyx_v_bytesSent); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":538
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_Write(self._handle, addr, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":540
 *         if status == FT4222_OK:
 *             return bytesSent
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_ReadEx(self, addr, flag, bytesToRead):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_4, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 540, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":516
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Write(self, addr, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with START and STOP conditions.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":542
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_ReadEx(self, addr, flag, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with the specified I2C condition.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_39i2cMaster_ReadEx(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_38i2cMaster_ReadEx[] = "Read data from the specified I2C slave device with the specified I2C condition.\n\n        Args:\n            addr (int): I2C slave address\n            flag (:obj:`ft4222.I2CMaster.Flag`): Flag to control start- and stopbit generation\n            bytesToRead (int): Number of bytes to read from slave\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_39i2cMaster_ReadEx(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_addr = 0;
  PyObject *__pyx_v_flag = 0;
  PyObject *__pyx_v_bytesToRead = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_ReadEx (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_addr,&__pyx_n_s_flag,&__pyx_n_s_bytesToRead,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_addr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_ReadEx", 1, 3, 3, 1); __PYX_ERR(0, 542, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_bytesToRead)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_ReadEx", 1, 3, 3, 2); __PYX_ERR(0, 542, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "i2cMaster_ReadEx") < 0)) __PYX_ERR(0, 542, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_addr = values[0];
    __pyx_v_flag = values[1];
    __pyx_v_bytesToRead = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_ReadEx", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 542, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_ReadEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_38i2cMaster_ReadEx(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_addr, __pyx_v_flag, __pyx_v_bytesToRead);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_38i2cMaster_ReadEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_flag, PyObject *__pyx_v_bytesToRead) {
  arrayobject *__pyx_v_buf = 0;
  uint16 __pyx_v_bytesRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  uint16 __pyx_t_5;
  uint8 __pyx_t_6;
  uint16 __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_ReadEx", 0);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":558
 *         """
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 558, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":560
 *             array[uint8] buf = array('B', [])
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 560, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 560, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":561
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)             # <<<<<<<<<<<<<<
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:
 */
  __pyx_t_5 = __Pyx_PyInt_As_uint16_t(__pyx_v_addr); if (unlikely((__pyx_t_5 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 561, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_uint8_t(__pyx_v_flag); if (unlikely((__pyx_t_6 == ((uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 561, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_7 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 561, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_ReadEx(__pyx_v_self->_handle, __pyx_t_5, __pyx_t_6, __pyx_v_buf->data.as_uchars, __pyx_t_7, (&__pyx_v_bytesRead));

  /* "ft4222/ft4222.pyx":562
 *         resize(buf, bytesToRead)
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytes(buf)
 */
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_bytesRead); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 562, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":563
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_8 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_8) {

    /* "ft4222/ft4222.pyx":564
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":563
 *         status = FT4222_I2CMaster_ReadEx(self._handle, addr, flag, buf.data.as_uchars, bytesToRead, &bytesRead)
 *         resize(buf, bytesRead)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":565
 *         if status == FT4222_OK:
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_WriteEx(self, addr, flag, data):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 565, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":542
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_ReadEx(self, addr, flag, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the specified I2C slave device with the specified I2C condition.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_ReadEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":567
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_WriteEx(self, addr, flag, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with the specified I2C condition.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_41i2cMaster_WriteEx(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_40i2cMaster_WriteEx[] = "Write data to the specified I2C slave device with the specified I2C condition.\n\n        Args:\n            addr (int): I2C slave address\n            flag (:obj:`ft4222.I2CMaster.Flag`): Flag to control start- and stopbit generation\n            data (int, bytes, bytearray): Data to write to slave\n\n        Returns:\n            int: Bytes sent to slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_41i2cMaster_WriteEx(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_addr = 0;
  PyObject *__pyx_v_flag = 0;
  PyObject *__pyx_v_data = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_WriteEx (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_addr,&__pyx_n_s_flag,&__pyx_n_s_data,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_addr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_WriteEx", 1, 3, 3, 1); __PYX_ERR(0, 567, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("i2cMaster_WriteEx", 1, 3, 3, 2); __PYX_ERR(0, 567, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "i2cMaster_WriteEx") < 0)) __PYX_ERR(0, 567, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_addr = values[0];
    __pyx_v_flag = values[1];
    __pyx_v_data = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("i2cMaster_WriteEx", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 567, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_WriteEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_40i2cMaster_WriteEx(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_addr, __pyx_v_flag, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_40i2cMaster_WriteEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_addr, PyObject *__pyx_v_flag, PyObject *__pyx_v_data) {
  uint16 __pyx_v_bytesSent;
  uint8 *__pyx_v_cdata;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  uint8 *__pyx_t_6;
  uint16 __pyx_t_7;
  uint8 __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_WriteEx", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":582
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":583
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 583, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_data);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 583, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":582
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":584
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_data); 
  __pyx_t_5 = (__pyx_t_1 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":585
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 585, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 585, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":584
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":588
 *         cdef:
 *             uint16 bytesSent
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 588, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_6;

  /* "ft4222/ft4222.pyx":589
 *             uint16 bytesSent
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytesSent
 */
  __pyx_t_7 = __Pyx_PyInt_As_uint16_t(__pyx_v_addr); if (unlikely((__pyx_t_7 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 589, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_uint8_t(__pyx_v_flag); if (unlikely((__pyx_t_8 == ((uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 589, __pyx_L1_error)
  __pyx_t_9 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 589, __pyx_L1_error)
  __pyx_v_status = FT4222_I2CMaster_WriteEx(__pyx_v_self->_handle, __pyx_t_7, __pyx_t_8, __pyx_v_cdata, __pyx_t_9, (&__pyx_v_bytesSent));

  /* "ft4222/ft4222.pyx":590
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":591
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:
 *             return bytesSent             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyInt_From_uint16_t(__pyx_v_bytesSent); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":590
 *             uint8* cdata = data
 *         status = FT4222_I2CMaster_WriteEx(self._handle, addr, flag, cdata, len(data), &bytesSent)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":592
 *         if status == FT4222_OK:
 *             return bytesSent
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_Reset(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_4, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 592, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":567
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_WriteEx(self, addr, flag, data):             # <<<<<<<<<<<<<<
 *         """Write data to the specified I2C slave device with the specified I2C condition.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_WriteEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":594
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the I2C master device.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_43i2cMaster_Reset(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_42i2cMaster_Reset[] = "Reset the I2C master device.\n\n        If the I2C bus encounters errors or works abnormally, this function will reset the I2C device.\n        It is not necessary to call I2CMaster_Init again after calling this reset function.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_43i2cMaster_Reset(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_Reset (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_42i2cMaster_Reset(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_42i2cMaster_Reset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_Reset", 0);

  /* "ft4222/ft4222.pyx":604
 * 
 *         """
 *         status = FT4222_I2CMaster_Reset(self._handle)             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT4222_I2CMaster_Reset(__pyx_v_self->_handle);

  /* "ft4222/ft4222.pyx":605
 *         """
 *         status = FT4222_I2CMaster_Reset(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_1 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":606
 *         status = FT4222_I2CMaster_Reset(self._handle)
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def i2cMaster_GetStatus(self):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 606, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":605
 *         """
 *         status = FT4222_I2CMaster_Reset(self._handle)
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":594
 *         raise FT4222DeviceError, status
 * 
 *     def i2cMaster_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the I2C master device.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_Reset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":608
 *             raise FT4222DeviceError, status
 * 
 *     def i2cMaster_GetStatus(self):             # <<<<<<<<<<<<<<
 *         """Read the status of the I2C master controller.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_45i2cMaster_GetStatus(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_44i2cMaster_GetStatus[] = "Read the status of the I2C master controller.\n\n        This can be used to poll a slave until its write-cycle is complete.\n\n        Returns:\n            :obj:`ft4222.I2CMaster.ControllerStatus`: Controller status\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_45i2cMaster_GetStatus(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("i2cMaster_GetStatus (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_44i2cMaster_GetStatus(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_44i2cMaster_GetStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  uint8 __pyx_v_cs;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("i2cMaster_GetStatus", 0);

  /* "ft4222/ft4222.pyx":621
 *         """
 *         cdef uint8 cs
 *         status = FT4222_I2CMaster_GetStatus(self._handle, &cs)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return ControllerStatus(cs)
 */
  __pyx_v_status = FT4222_I2CMaster_GetStatus(__pyx_v_self->_handle, (&__pyx_v_cs));

  /* "ft4222/ft4222.pyx":622
 *         cdef uint8 cs
 *         status = FT4222_I2CMaster_GetStatus(self._handle, &cs)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return ControllerStatus(cs)
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":623
 *         status = FT4222_I2CMaster_GetStatus(self._handle, &cs)
 *         if status == FT4222_OK:
 *             return ControllerStatus(cs)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_ControllerStatus); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 623, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_uint8_t(__pyx_v_cs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 623, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 623, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":622
 *         cdef uint8 cs
 *         status = FT4222_I2CMaster_GetStatus(self._handle, &cs)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return ControllerStatus(cs)
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":624
 *         if status == FT4222_OK:
 *             return ControllerStatus(cs)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 624, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":608
 *             raise FT4222DeviceError, status
 * 
 *     def i2cMaster_GetStatus(self):             # <<<<<<<<<<<<<<
 *         """Read the status of the I2C master controller.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.i2cMaster_GetStatus", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":627
 * 
 * 
 *     def spi_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_47spi_Reset(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_46spi_Reset[] = "Reset the SPI master or slave device\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_47spi_Reset(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spi_Reset (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_46spi_Reset(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_46spi_Reset(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spi_Reset", 0);

  /* "ft4222/ft4222.pyx":634
 * 
 *         """
 *         status = FT4222_SPI_Reset(self._handle);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT4222_SPI_Reset(__pyx_v_self->_handle);

  /* "ft4222/ft4222.pyx":635
 *         """
 *         status = FT4222_SPI_Reset(self._handle);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_1 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":636
 *         status = FT4222_SPI_Reset(self._handle);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spi_ResetTransaction(self, spiIdx):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 636, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":635
 *         """
 *         status = FT4222_SPI_Reset(self._handle);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":627
 * 
 * 
 *     def spi_Reset(self):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spi_Reset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":638
 *             raise FT4222DeviceError, status
 * 
 *     def spi_ResetTransaction(self, spiIdx):             # <<<<<<<<<<<<<<
 *         """Reset the SPI transaction
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_49spi_ResetTransaction(PyObject *__pyx_v_self, PyObject *__pyx_v_spiIdx); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_48spi_ResetTransaction[] = "Reset the SPI transaction\n\n        Args:\n            spiIdx (int): The index of the SPI transaction, which ranges from 0~3 depending on the mode of the chip.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_49spi_ResetTransaction(PyObject *__pyx_v_self, PyObject *__pyx_v_spiIdx) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spi_ResetTransaction (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_48spi_ResetTransaction(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_spiIdx));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_48spi_ResetTransaction(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_spiIdx) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint8 __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spi_ResetTransaction", 0);

  /* "ft4222/ft4222.pyx":648
 * 
 *         """
 *         status = FT4222_SPI_ResetTransaction(self._handle, spiIdx);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint8_t(__pyx_v_spiIdx); if (unlikely((__pyx_t_1 == ((uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 648, __pyx_L1_error)
  __pyx_v_status = FT4222_SPI_ResetTransaction(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":649
 *         """
 *         status = FT4222_SPI_ResetTransaction(self._handle, spiIdx);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":650
 *         status = FT4222_SPI_ResetTransaction(self._handle, spiIdx);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spi_SetDrivingStrength(self, clkStrength, ioStrength, ssoStrength):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 650, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":649
 *         """
 *         status = FT4222_SPI_ResetTransaction(self._handle, spiIdx);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":638
 *             raise FT4222DeviceError, status
 * 
 *     def spi_ResetTransaction(self, spiIdx):             # <<<<<<<<<<<<<<
 *         """Reset the SPI transaction
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spi_ResetTransaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":652
 *             raise FT4222DeviceError, status
 * 
 *     def spi_SetDrivingStrength(self, clkStrength, ioStrength, ssoStrength):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_51spi_SetDrivingStrength(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_50spi_SetDrivingStrength[] = "Reset the SPI master or slave device.\n\n        Args:\n            clkStrength (:obj:`ft4222.SPI.DrivingStrength`): Driving strength clock pin (master only)\n            ioStrength (:obj:`ft4222.SPI.DrivingStrength`): Driving strength io pin\n            ssoStrength (:obj:`ft4222.SPI.DrivingStrength`): Driving strength sso pin (master only)\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_51spi_SetDrivingStrength(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_clkStrength = 0;
  PyObject *__pyx_v_ioStrength = 0;
  PyObject *__pyx_v_ssoStrength = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spi_SetDrivingStrength (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_clkStrength,&__pyx_n_s_ioStrength,&__pyx_n_s_ssoStrength,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_clkStrength)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ioStrength)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spi_SetDrivingStrength", 1, 3, 3, 1); __PYX_ERR(0, 652, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ssoStrength)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spi_SetDrivingStrength", 1, 3, 3, 2); __PYX_ERR(0, 652, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "spi_SetDrivingStrength") < 0)) __PYX_ERR(0, 652, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_clkStrength = values[0];
    __pyx_v_ioStrength = values[1];
    __pyx_v_ssoStrength = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spi_SetDrivingStrength", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 652, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spi_SetDrivingStrength", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_50spi_SetDrivingStrength(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_clkStrength, __pyx_v_ioStrength, __pyx_v_ssoStrength);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_50spi_SetDrivingStrength(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_clkStrength, PyObject *__pyx_v_ioStrength, PyObject *__pyx_v_ssoStrength) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  SPI_DrivingStrength __pyx_t_1;
  SPI_DrivingStrength __pyx_t_2;
  SPI_DrivingStrength __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spi_SetDrivingStrength", 0);

  /* "ft4222/ft4222.pyx":664
 * 
 *         """
 *         status = FT4222_SPI_SetDrivingStrength(self._handle, clkStrength, ioStrength, ssoStrength);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((SPI_DrivingStrength)__Pyx_PyInt_As_SPI_DrivingStrength(__pyx_v_clkStrength)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 664, __pyx_L1_error)
  __pyx_t_2 = ((SPI_DrivingStrength)__Pyx_PyInt_As_SPI_DrivingStrength(__pyx_v_ioStrength)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 664, __pyx_L1_error)
  __pyx_t_3 = ((SPI_DrivingStrength)__Pyx_PyInt_As_SPI_DrivingStrength(__pyx_v_ssoStrength)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 664, __pyx_L1_error)
  __pyx_v_status = FT4222_SPI_SetDrivingStrength(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "ft4222/ft4222.pyx":665
 *         """
 *         status = FT4222_SPI_SetDrivingStrength(self._handle, clkStrength, ioStrength, ssoStrength);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_4 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_4)) {

    /* "ft4222/ft4222.pyx":666
 *         status = FT4222_SPI_SetDrivingStrength(self._handle, clkStrength, ioStrength, ssoStrength);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_Init(self, mode, clock, cpol, cpha, ssoMap):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_5, __pyx_t_6, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 666, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":665
 *         """
 *         status = FT4222_SPI_SetDrivingStrength(self._handle, clkStrength, ioStrength, ssoStrength);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":652
 *             raise FT4222DeviceError, status
 * 
 *     def spi_SetDrivingStrength(self, clkStrength, ioStrength, ssoStrength):             # <<<<<<<<<<<<<<
 *         """Reset the SPI master or slave device.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spi_SetDrivingStrength", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":668
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_Init(self, mode, clock, cpol, cpha, ssoMap):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI master under all modes.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_53spiMaster_Init(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_52spiMaster_Init[] = "Initialize as an SPI master under all modes.\n\n        Args:\n            mode (:obj:`ft4222.SPIMaster.Mode`): SPI transmission lines / mode\n            clock (:obj:`ft4222.SPIMaster.Clock`): Clock divider\n            cpol (:obj:`ft4222.SPI.Cpol`): Clock polarity\n            cpha (:obj:`ft4222.SPI.Cpha`): Clock phase\n            ssoMap (:obj:`ft4222.SPIMaster.SlaveSelect`): Slave selection output pins\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_53spiMaster_Init(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_mode = 0;
  PyObject *__pyx_v_clock = 0;
  PyObject *__pyx_v_cpol = 0;
  PyObject *__pyx_v_cpha = 0;
  PyObject *__pyx_v_ssoMap = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_Init (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_mode,&__pyx_n_s_clock,&__pyx_n_s_cpol,&__pyx_n_s_cpha,&__pyx_n_s_ssoMap,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_clock)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, 1); __PYX_ERR(0, 668, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cpol)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, 2); __PYX_ERR(0, 668, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, 3); __PYX_ERR(0, 668, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ssoMap)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, 4); __PYX_ERR(0, 668, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "spiMaster_Init") < 0)) __PYX_ERR(0, 668, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_mode = values[0];
    __pyx_v_clock = values[1];
    __pyx_v_cpol = values[2];
    __pyx_v_cpha = values[3];
    __pyx_v_ssoMap = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_Init", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 668, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_52spiMaster_Init(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_mode, __pyx_v_clock, __pyx_v_cpol, __pyx_v_cpha, __pyx_v_ssoMap);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_52spiMaster_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_mode, PyObject *__pyx_v_clock, PyObject *__pyx_v_cpol, PyObject *__pyx_v_cpha, PyObject *__pyx_v_ssoMap) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  FT4222_SPIMode __pyx_t_1;
  FT4222_SPIClock __pyx_t_2;
  FT4222_SPICPOL __pyx_t_3;
  FT4222_SPICPHA __pyx_t_4;
  uint8 __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_Init", 0);

  /* "ft4222/ft4222.pyx":682
 * 
 *         """
 *         status = FT4222_SPIMaster_Init(self._handle, mode, clock, cpol, cpha, ssoMap);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((FT4222_SPIMode)__Pyx_PyInt_As_FT4222_SPIMode(__pyx_v_mode)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 682, __pyx_L1_error)
  __pyx_t_2 = ((FT4222_SPIClock)__Pyx_PyInt_As_FT4222_SPIClock(__pyx_v_clock)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 682, __pyx_L1_error)
  __pyx_t_3 = ((FT4222_SPICPOL)__Pyx_PyInt_As_FT4222_SPICPOL(__pyx_v_cpol)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 682, __pyx_L1_error)
  __pyx_t_4 = ((FT4222_SPICPHA)__Pyx_PyInt_As_FT4222_SPICPHA(__pyx_v_cpha)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 682, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_uint8_t(__pyx_v_ssoMap); if (unlikely((__pyx_t_5 == ((uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 682, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_Init(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5);

  /* "ft4222/ft4222.pyx":683
 *         """
 *         status = FT4222_SPIMaster_Init(self._handle, mode, clock, cpol, cpha, ssoMap);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_6 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_6)) {

    /* "ft4222/ft4222.pyx":684
 *         status = FT4222_SPIMaster_Init(self._handle, mode, clock, cpol, cpha, ssoMap);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_SetLines(self, mode):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_7, __pyx_t_8, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 684, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":683
 *         """
 *         status = FT4222_SPIMaster_Init(self._handle, mode, clock, cpol, cpha, ssoMap);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":668
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_Init(self, mode, clock, cpol, cpha, ssoMap):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI master under all modes.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":686
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SetLines(self, mode):             # <<<<<<<<<<<<<<
 *         """Switch the FT4222H SPI master to single, dual, or quad mode.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_55spiMaster_SetLines(PyObject *__pyx_v_self, PyObject *__pyx_v_mode); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_54spiMaster_SetLines[] = "Switch the FT4222H SPI master to single, dual, or quad mode.\n\n        This overrides the mode passed to FT4222_SPIMaster_init. This might be needed if a\n        device accepts commands in single mode but data transfer is to use dual or quad mode.\n\n        Args:\n            mode (:obj:`ft4222.SPIMaster.Mode`): SPI transmission lines / mode\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_55spiMaster_SetLines(PyObject *__pyx_v_self, PyObject *__pyx_v_mode) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_SetLines (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_54spiMaster_SetLines(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_mode));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_54spiMaster_SetLines(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_mode) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  FT4222_SPIMode __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_SetLines", 0);

  /* "ft4222/ft4222.pyx":699
 * 
 *         """
 *         status = FT4222_SPIMaster_SetLines(self._handle, mode);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((FT4222_SPIMode)__Pyx_PyInt_As_FT4222_SPIMode(__pyx_v_mode)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 699, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_SetLines(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":700
 *         """
 *         status = FT4222_SPIMaster_SetLines(self._handle, mode);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":701
 *         status = FT4222_SPIMaster_SetLines(self._handle, mode);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_SingleRead(self, bytesToRead, isEndTransaction):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 701, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":700
 *         """
 *         status = FT4222_SPIMaster_SetLines(self._handle, mode);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":686
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SetLines(self, mode):             # <<<<<<<<<<<<<<
 *         """Switch the FT4222H SPI master to single, dual, or quad mode.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SetLines", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":703
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleRead(self, bytesToRead, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Read data from a SPI slave in single mode
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_57spiMaster_SingleRead(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_56spiMaster_SingleRead[] = "Read data from a SPI slave in single mode\n\n        Args:\n            bytesToRead (int): Number of bytes to read\n            isEndTransaction (bool): If True the slave select pin will be raised at the end\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_57spiMaster_SingleRead(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_bytesToRead = 0;
  PyObject *__pyx_v_isEndTransaction = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_SingleRead (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_bytesToRead,&__pyx_n_s_isEndTransaction,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_bytesToRead)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_isEndTransaction)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_SingleRead", 1, 2, 2, 1); __PYX_ERR(0, 703, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "spiMaster_SingleRead") < 0)) __PYX_ERR(0, 703, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_bytesToRead = values[0];
    __pyx_v_isEndTransaction = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_SingleRead", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 703, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleRead", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_56spiMaster_SingleRead(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_bytesToRead, __pyx_v_isEndTransaction);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_56spiMaster_SingleRead(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_bytesToRead, PyObject *__pyx_v_isEndTransaction) {
  arrayobject *__pyx_v_buf = 0;
  uint16 __pyx_v_bytesRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  uint16 __pyx_t_5;
  BOOL __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_SingleRead", 0);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":718
 *         """
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 718, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":720
 *             array[uint8] buf = array('B', [])
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)
 *         if status == FT4222_OK:
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 720, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 720, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":721
 *             uint16 bytesRead
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)
 */
  __pyx_t_5 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_5 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 721, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_BOOL(__pyx_v_isEndTransaction); if (unlikely((__pyx_t_6 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 721, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_SingleRead(__pyx_v_self->_handle, __pyx_v_buf->data.as_uchars, __pyx_t_5, (&__pyx_v_bytesRead), __pyx_t_6);

  /* "ft4222/ft4222.pyx":722
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 */
  __pyx_t_7 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_7) {

    /* "ft4222/ft4222.pyx":723
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
    __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_bytesRead); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 723, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":724
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 724, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":722
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_SingleRead(self._handle, buf.data.as_uchars, bytesToRead, &bytesRead, isEndTransaction)
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 */
  }

  /* "ft4222/ft4222.pyx":725
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_SingleWrite(self, data, isEndTransaction):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 725, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 725, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 725, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":703
 *             raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleRead(self, bytesToRead, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Read data from a SPI slave in single mode
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleRead", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":727
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write data to a SPI slave in single mode
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_59spiMaster_SingleWrite(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_58spiMaster_SingleWrite[] = "Write data to a SPI slave in single mode\n\n        Args:\n            data (bytes, bytearray, int): Data to write to slave\n            isEndTransaction (bool): If True the slave select pin will be raised at the end\n\n        Returns:\n            int: Bytes sent to slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_59spiMaster_SingleWrite(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_isEndTransaction = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_SingleWrite (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_isEndTransaction,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_isEndTransaction)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_SingleWrite", 1, 2, 2, 1); __PYX_ERR(0, 727, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "spiMaster_SingleWrite") < 0)) __PYX_ERR(0, 727, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_isEndTransaction = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_SingleWrite", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 727, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_58spiMaster_SingleWrite(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_data, __pyx_v_isEndTransaction);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_58spiMaster_SingleWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_isEndTransaction) {
  uint16 __pyx_v_bytesSent;
  uint8 *__pyx_v_cdata;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  uint8 *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  BOOL __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_SingleWrite", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":741
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":742
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 742, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_data);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 742, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":741
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":743
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_data); 
  __pyx_t_5 = (__pyx_t_1 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":744
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 744, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 744, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":743
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":747
 *         cdef:
 *             uint16 bytesSent
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);
 *         if status == FT4222_OK:
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 747, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_6;

  /* "ft4222/ft4222.pyx":748
 *             uint16 bytesSent
 *             uint8* cdata = data
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return bytesSent
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 748, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_BOOL(__pyx_v_isEndTransaction); if (unlikely((__pyx_t_8 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 748, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_SingleWrite(__pyx_v_self->_handle, __pyx_v_cdata, __pyx_t_7, (&__pyx_v_bytesSent), __pyx_t_8);

  /* "ft4222/ft4222.pyx":749
 *             uint8* cdata = data
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":750
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);
 *         if status == FT4222_OK:
 *             return bytesSent             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyInt_From_uint16_t(__pyx_v_bytesSent); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 750, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":749
 *             uint8* cdata = data
 *         status = FT4222_SPIMaster_SingleWrite(self._handle, cdata, len(data), &bytesSent, isEndTransaction);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return bytesSent
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":751
 *         if status == FT4222_OK:
 *             return bytesSent
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_SingleReadWrite(self, data, isEndTransaction):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 751, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 751, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_4, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 751, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":727
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write data to a SPI slave in single mode
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":753
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleReadWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in single mode
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_61spiMaster_SingleReadWrite(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_60spiMaster_SingleReadWrite[] = "Write and read data to and from a SPI slave in single mode\n\n        Args:\n            data (bytes, bytearray, int): Data to write to slave\n            isEndTransaction (bool): If True the slave select pin will be raised at the end\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_61spiMaster_SingleReadWrite(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_isEndTransaction = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_SingleReadWrite (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_isEndTransaction,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_isEndTransaction)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_SingleReadWrite", 1, 2, 2, 1); __PYX_ERR(0, 753, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "spiMaster_SingleReadWrite") < 0)) __PYX_ERR(0, 753, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_data = values[0];
    __pyx_v_isEndTransaction = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_SingleReadWrite", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 753, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleReadWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_60spiMaster_SingleReadWrite(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_data, __pyx_v_isEndTransaction);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_60spiMaster_SingleReadWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_isEndTransaction) {
  uint16 __pyx_v_sizeTransferred;
  uint8 *__pyx_v_cdata;
  arrayobject *__pyx_v_buf = 0;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  uint8 *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  BOOL __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_SingleReadWrite", 0);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":767
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":768
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 768, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_data);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 768, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":767
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":769
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_data); 
  __pyx_t_5 = (__pyx_t_1 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":770
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 sizeTransferred
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 770, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 770, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":769
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":773
 *         cdef:
 *             uint16 sizeTransferred
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 *             array[uint8] buf = array('B', [])
 *         resize(buf, len(data))
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 773, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_6;

  /* "ft4222/ft4222.pyx":774
 *             uint16 sizeTransferred
 *             uint8* cdata = data
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *         resize(buf, len(data))
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 */
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_4), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 774, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "ft4222/ft4222.pyx":775
 *             uint8* cdata = data
 *             array[uint8] buf = array('B', [])
 *         resize(buf, len(data))             # <<<<<<<<<<<<<<
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 *         if status == FT4222_OK:
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 775, __pyx_L1_error)
  __pyx_t_8 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_7); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 775, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":776
 *             array[uint8] buf = array('B', [])
 *         resize(buf, len(data))
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             resize(buf, sizeTransferred)
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 776, __pyx_L1_error)
  __pyx_t_9 = __Pyx_PyInt_As_BOOL(__pyx_v_isEndTransaction); if (unlikely((__pyx_t_9 == ((BOOL)-1)) && PyErr_Occurred())) __PYX_ERR(0, 776, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_SingleReadWrite(__pyx_v_self->_handle, __pyx_v_buf->data.as_uchars, __pyx_v_cdata, __pyx_t_7, (&__pyx_v_sizeTransferred), __pyx_t_9);

  /* "ft4222/ft4222.pyx":777
 *         resize(buf, len(data))
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, sizeTransferred)
 *             return bytes(buf)
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":778
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 *         if status == FT4222_OK:
 *             resize(buf, sizeTransferred)             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
    __pyx_t_8 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_sizeTransferred); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 778, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":779
 *         if status == FT4222_OK:
 *             resize(buf, sizeTransferred)
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 779, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":777
 *         resize(buf, len(data))
 *         status = FT4222_SPIMaster_SingleReadWrite(self._handle, buf.data.as_uchars, cdata, len(data), &sizeTransferred, isEndTransaction);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, sizeTransferred)
 *             return bytes(buf)
 */
  }

  /* "ft4222/ft4222.pyx":780
 *             resize(buf, sizeTransferred)
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_MultiReadWrite(self, singleWrite, multiWrite, bytesToRead):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_4, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 780, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":753
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_SingleReadWrite(self, data, isEndTransaction):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in single mode
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_SingleReadWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":782
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_MultiReadWrite(self, singleWrite, multiWrite, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in dual- or quad-mode (multi-mode).
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_63spiMaster_MultiReadWrite(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_62spiMaster_MultiReadWrite[] = "Write and read data to and from a SPI slave in dual- or quad-mode (multi-mode).\n\n        Args:\n            singleWrite (bytes, bytearray, int): Data to write to slave in signle-line mode (max. 15 bytes)\n            multiWrite (bytes, bytearray, int): Data to write to slave in multi-line mode (max. 65535 bytes)\n            bytesToRead (int):  Number of bytes to read on multi-line (max. 65535 bytes)\n\n        Returns:\n            bytes: Bytes read from slave in multi-line mode\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_63spiMaster_MultiReadWrite(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_singleWrite = 0;
  PyObject *__pyx_v_multiWrite = 0;
  PyObject *__pyx_v_bytesToRead = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_MultiReadWrite (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_singleWrite,&__pyx_n_s_multiWrite,&__pyx_n_s_bytesToRead,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_singleWrite)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_multiWrite)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_MultiReadWrite", 1, 3, 3, 1); __PYX_ERR(0, 782, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_bytesToRead)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiMaster_MultiReadWrite", 1, 3, 3, 2); __PYX_ERR(0, 782, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "spiMaster_MultiReadWrite") < 0)) __PYX_ERR(0, 782, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_singleWrite = values[0];
    __pyx_v_multiWrite = values[1];
    __pyx_v_bytesToRead = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiMaster_MultiReadWrite", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 782, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_MultiReadWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_62spiMaster_MultiReadWrite(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_singleWrite, __pyx_v_multiWrite, __pyx_v_bytesToRead);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_62spiMaster_MultiReadWrite(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_singleWrite, PyObject *__pyx_v_multiWrite, PyObject *__pyx_v_bytesToRead) {
  CYTHON_UNUSED PyObject *__pyx_v_data = NULL;
  PyObject *__pyx_v_write = NULL;
  uint8 *__pyx_v_cdata;
  arrayobject *__pyx_v_buf = 0;
  uint32 __pyx_v_bytesRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  uint8 *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  uint16 __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_MultiReadWrite", 0);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":797
 * 
 *         """
 *         if isinstance(singleWrite, int):             # <<<<<<<<<<<<<<
 *             data = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_singleWrite); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":798
 *         """
 *         if isinstance(singleWrite, int):
 *             data = bytes([singleWrite])             # <<<<<<<<<<<<<<
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 798, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_singleWrite);
    __Pyx_GIVEREF(__pyx_v_singleWrite);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_singleWrite);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 798, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_data = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":797
 * 
 *         """
 *         if isinstance(singleWrite, int):             # <<<<<<<<<<<<<<
 *             data = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":799
 *         if isinstance(singleWrite, int):
 *             data = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_singleWrite); 
  __pyx_t_5 = (__pyx_t_1 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = PyByteArray_Check(__pyx_v_singleWrite); 
  __pyx_t_1 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":800
 *             data = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         if isinstance(multiWrite, int):
 *             data = bytes([multiWrite])
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 800, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 800, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":799
 *         if isinstance(singleWrite, int):
 *             data = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":801
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):             # <<<<<<<<<<<<<<
 *             data = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_multiWrite); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":802
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):
 *             data = bytes([multiWrite])             # <<<<<<<<<<<<<<
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 802, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_multiWrite);
    __Pyx_GIVEREF(__pyx_v_multiWrite);
    PyList_SET_ITEM(__pyx_t_4, 0, __pyx_v_multiWrite);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 802, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_data, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "ft4222/ft4222.pyx":801
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         if isinstance(multiWrite, int):             # <<<<<<<<<<<<<<
 *             data = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 */
    goto __pyx_L6;
  }

  /* "ft4222/ft4222.pyx":803
 *         if isinstance(multiWrite, int):
 *             data = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         write = singleWrite + multiWrite
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_multiWrite); 
  __pyx_t_5 = (__pyx_t_1 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_5 = PyByteArray_Check(__pyx_v_multiWrite); 
  __pyx_t_1 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L7_bool_binop_done:;
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":804
 *             data = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         write = singleWrite + multiWrite
 *         cdef:
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 804, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":803
 *         if isinstance(multiWrite, int):
 *             data = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         write = singleWrite + multiWrite
 */
  }
  __pyx_L6:;

  /* "ft4222/ft4222.pyx":805
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")
 *         write = singleWrite + multiWrite             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint8* cdata = write
 */
  __pyx_t_3 = PyNumber_Add(__pyx_v_singleWrite, __pyx_v_multiWrite); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 805, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_write = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":807
 *         write = singleWrite + multiWrite
 *         cdef:
 *             uint8* cdata = write             # <<<<<<<<<<<<<<
 *             array[uint8] buf = array('B', [])
 *             uint32 bytesRead
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableUString(__pyx_v_write); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 807, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_6;

  /* "ft4222/ft4222.pyx":808
 *         cdef:
 *             uint8* cdata = write
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint32 bytesRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_3), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 808, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ft4222/ft4222.pyx":810
 *             array[uint8] buf = array('B', [])
 *             uint32 bytesRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);
 *         if status == FT4222_OK:
 */
  __pyx_t_7 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_7 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 810, __pyx_L1_error)
  __pyx_t_8 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_7); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 810, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":811
 *             uint32 bytesRead
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_singleWrite); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 811, __pyx_L1_error)
  __pyx_t_9 = PyObject_Length(__pyx_v_multiWrite); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 811, __pyx_L1_error)
  __pyx_t_10 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_10 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 811, __pyx_L1_error)
  __pyx_v_status = FT4222_SPIMaster_MultiReadWrite(__pyx_v_self->_handle, __pyx_v_buf->data.as_uchars, __pyx_v_cdata, __pyx_t_7, __pyx_t_9, __pyx_t_10, (&__pyx_v_bytesRead));

  /* "ft4222/ft4222.pyx":812
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":813
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
    __pyx_t_8 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_bytesRead); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 813, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":814
 *         if status == FT4222_OK:
 *             resize(buf, bytesRead)
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 814, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":812
 *         resize(buf, bytesToRead)
 *         status = FT4222_SPIMaster_MultiReadWrite(self._handle, buf.data.as_uchars, cdata, len(singleWrite), len(multiWrite), bytesToRead, &bytesRead);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 */
  }

  /* "ft4222/ft4222.pyx":815
 *             resize(buf, bytesRead)
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiMaster_EndTransaction(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 815, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 815, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(0, 815, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":782
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_MultiReadWrite(self, singleWrite, multiWrite, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Write and read data to and from a SPI slave in dual- or quad-mode (multi-mode).
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_MultiReadWrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_write);
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":817
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_EndTransaction(self):             # <<<<<<<<<<<<<<
 *         """End the current SPI transaction.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_65spiMaster_EndTransaction(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_64spiMaster_EndTransaction[] = "End the current SPI transaction.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_65spiMaster_EndTransaction(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiMaster_EndTransaction (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_64spiMaster_EndTransaction(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_64spiMaster_EndTransaction(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  DWORD __pyx_v_bytesSent;
  FT_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiMaster_EndTransaction", 0);

  /* "ft4222/ft4222.pyx":826
 *         cdef:
 *             DWORD bytesSent;
 *         status = FT_Write(self._handle, <unsigned char*>NULL, 0, &bytesSent);             # <<<<<<<<<<<<<<
 *         if status == FT_OK:
 *             return
 */
  __pyx_v_status = FT_Write(__pyx_v_self->_handle, ((unsigned char *)NULL), 0, (&__pyx_v_bytesSent));

  /* "ft4222/ft4222.pyx":827
 *             DWORD bytesSent;
 *         status = FT_Write(self._handle, <unsigned char*>NULL, 0, &bytesSent);
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((__pyx_v_status == FT_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":828
 *         status = FT_Write(self._handle, <unsigned char*>NULL, 0, &bytesSent);
 *         if status == FT_OK:
 *             return             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":827
 *             DWORD bytesSent;
 *         status = FT_Write(self._handle, <unsigned char*>NULL, 0, &bytesSent);
 *         if status == FT_OK:             # <<<<<<<<<<<<<<
 *             return
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":829
 *         if status == FT_OK:
 *             return
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_Init(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 829, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_ULONG(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 829, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 829, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":817
 *         raise FT4222DeviceError, status
 * 
 *     def spiMaster_EndTransaction(self):             # <<<<<<<<<<<<<<
 *         """End the current SPI transaction.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiMaster_EndTransaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":831
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Init(self):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an SPI slave. Default SPI_SlaveProtocol is SPI_SLAVE_WITH_PROTOCOL.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_67spiSlave_Init(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_66spiSlave_Init[] = "Initialize the FT4222H as an SPI slave. Default SPI_SlaveProtocol is SPI_SLAVE_WITH_PROTOCOL.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_67spiSlave_Init(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_Init (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_66spiSlave_Init(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_66spiSlave_Init(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_Init", 0);

  /* "ft4222/ft4222.pyx":838
 * 
 *         """
 *         status = FT4222_SPISlave_Init(self._handle);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_v_status = FT4222_SPISlave_Init(__pyx_v_self->_handle);

  /* "ft4222/ft4222.pyx":839
 *         """
 *         status = FT4222_SPISlave_Init(self._handle);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_1 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":840
 *         status = FT4222_SPISlave_Init(self._handle);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_InitEx(self, proto):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 840, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 840, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 840, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":839
 *         """
 *         status = FT4222_SPISlave_Init(self._handle);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":831
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Init(self):             # <<<<<<<<<<<<<<
 *         """Initialize the FT4222H as an SPI slave. Default SPI_SlaveProtocol is SPI_SLAVE_WITH_PROTOCOL.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_Init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":842
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_InitEx(self, proto):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI slave under all modes.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_69spiSlave_InitEx(PyObject *__pyx_v_self, PyObject *__pyx_v_proto); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_68spiSlave_InitEx[] = "Initialize as an SPI slave under all modes.\n\n        Args:\n            mode (:obj:`ft4222.SPISlave.Protocol`): SPI slave protocol\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_69spiSlave_InitEx(PyObject *__pyx_v_self, PyObject *__pyx_v_proto) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_InitEx (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_68spiSlave_InitEx(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_proto));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_68spiSlave_InitEx(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_proto) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  SPI_SlaveProtocol __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_InitEx", 0);

  /* "ft4222/ft4222.pyx":852
 * 
 *         """
 *         status = FT4222_SPISlave_InitEx(self._handle,proto);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((SPI_SlaveProtocol)__Pyx_PyInt_As_SPI_SlaveProtocol(__pyx_v_proto)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 852, __pyx_L1_error)
  __pyx_v_status = FT4222_SPISlave_InitEx(__pyx_v_self->_handle, __pyx_t_1);

  /* "ft4222/ft4222.pyx":853
 *         """
 *         status = FT4222_SPISlave_InitEx(self._handle,proto);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_2 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ft4222/ft4222.pyx":854
 *         status = FT4222_SPISlave_InitEx(self._handle,proto);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_Read(self, bytesToRead):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 854, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 854, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_3, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 854, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":853
 *         """
 *         status = FT4222_SPISlave_InitEx(self._handle,proto);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":842
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_InitEx(self, proto):             # <<<<<<<<<<<<<<
 *         """Initialize as an SPI slave under all modes.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_InitEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":856
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_Read(self, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the receive queue of the SPI slave device.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_71spiSlave_Read(PyObject *__pyx_v_self, PyObject *__pyx_v_bytesToRead); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_70spiSlave_Read[] = "Read data from the receive queue of the SPI slave device.\n\n        Args:\n            bytesToRead (int): Number of bytes to read\n\n        Returns:\n            bytes: Bytes read from slave\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_71spiSlave_Read(PyObject *__pyx_v_self, PyObject *__pyx_v_bytesToRead) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_Read (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_70spiSlave_Read(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_bytesToRead));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_70spiSlave_Read(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_bytesToRead) {
  arrayobject *__pyx_v_buf = 0;
  uint16 __pyx_v_sizeRead;
  FT4222_STATUS __pyx_v_status;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_buf;
  __Pyx_Buffer __pyx_pybuffer_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  uint16 __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_Read", 0);
  __pyx_pybuffer_buf.pybuffer.buf = NULL;
  __pyx_pybuffer_buf.refcount = 0;
  __pyx_pybuffernd_buf.data = NULL;
  __pyx_pybuffernd_buf.rcbuffer = &__pyx_pybuffer_buf;

  /* "ft4222/ft4222.pyx":870
 *         """
 *         cdef:
 *             array[uint8] buf = array('B', [])             # <<<<<<<<<<<<<<
 *             uint16 sizeRead
 *         resize(buf, bytesToRead)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 870, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 870, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_n_u_B);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_B);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_5array_array), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 870, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_buf.rcbuffer->pybuffer, (PyObject*)((arrayobject *)__pyx_t_1), &__Pyx_TypeInfo_nn_uint8, PyBUF_FORMAT| PyBUF_INDIRECT, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_buf = ((arrayobject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_buf.rcbuffer->pybuffer.buf = NULL;
      __PYX_ERR(0, 870, __pyx_L1_error)
    } else {__pyx_pybuffernd_buf.diminfo[0].strides = __pyx_pybuffernd_buf.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_buf.diminfo[0].shape = __pyx_pybuffernd_buf.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_buf.diminfo[0].suboffsets = __pyx_pybuffernd_buf.rcbuffer->pybuffer.suboffsets[0];
    }
  }
  __pyx_v_buf = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":872
 *             array[uint8] buf = array('B', [])
 *             uint16 sizeRead
 *         resize(buf, bytesToRead)             # <<<<<<<<<<<<<<
 * 
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);
 */
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 872, __pyx_L1_error)
  __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 872, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":874
 *         resize(buf, bytesToRead)
 * 
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             resize(buf, sizeRead)
 */
  __pyx_t_5 = __Pyx_PyInt_As_uint16_t(__pyx_v_bytesToRead); if (unlikely((__pyx_t_5 == ((uint16)-1)) && PyErr_Occurred())) __PYX_ERR(0, 874, __pyx_L1_error)
  __pyx_v_status = FT4222_SPISlave_Read(__pyx_v_self->_handle, __pyx_v_buf->data.as_uchars, __pyx_t_5, (&__pyx_v_sizeRead));

  /* "ft4222/ft4222.pyx":875
 * 
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, sizeRead)
 *             return bytes(buf)
 */
  __pyx_t_6 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_6) {

    /* "ft4222/ft4222.pyx":876
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);
 *         if status == FT4222_OK:
 *             resize(buf, sizeRead)             # <<<<<<<<<<<<<<
 *             return bytes(buf)
 *         raise FT4222DeviceError, status
 */
    __pyx_t_4 = resize(((arrayobject *)__pyx_v_buf), __pyx_v_sizeRead); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 876, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":877
 *         if status == FT4222_OK:
 *             resize(buf, sizeRead)
 *             return bytes(buf)             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), ((PyObject *)__pyx_v_buf)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":875
 * 
 *         status = FT4222_SPISlave_Read(self._handle, buf.data.as_uchars, bytesToRead, &sizeRead);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             resize(buf, sizeRead)
 *             return bytes(buf)
 */
  }

  /* "ft4222/ft4222.pyx":878
 *             resize(buf, sizeRead)
 *             return bytes(buf)
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_SetMode(self, cpol, cpha):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 878, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 878, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_Raise(__pyx_t_1, __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 878, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":856
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_Read(self, bytesToRead):             # <<<<<<<<<<<<<<
 *         """Read data from the receive queue of the SPI slave device.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_Read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_buf.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":880
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_SetMode(self, cpol, cpha):             # <<<<<<<<<<<<<<
 *         """Set SPI slave cpol and cpha. The Default value of cpol is (:obj:`ft4222.SPI.Cpol.CLK_IDLE_LOW`) , default value of cpha is (:obj:`ft4222.SPI.Cpol.CLK_LEADING`)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_73spiSlave_SetMode(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_72spiSlave_SetMode[] = "Set SPI slave cpol and cpha. The Default value of cpol is (:obj:`ft4222.SPI.Cpol.CLK_IDLE_LOW`) , default value of cpha is (:obj:`ft4222.SPI.Cpol.CLK_LEADING`)\n\n        Args:\n            cpol (:obj:`ft4222.SPI.Cpol`): Clock polarity\n            cpha (:obj:`ft4222.SPI.Cpha`): Clock phase\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_73spiSlave_SetMode(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_cpol = 0;
  PyObject *__pyx_v_cpha = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_SetMode (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_cpol,&__pyx_n_s_cpha,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cpol)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("spiSlave_SetMode", 1, 2, 2, 1); __PYX_ERR(0, 880, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "spiSlave_SetMode") < 0)) __PYX_ERR(0, 880, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_cpol = values[0];
    __pyx_v_cpha = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("spiSlave_SetMode", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 880, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_SetMode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_72spiSlave_SetMode(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), __pyx_v_cpol, __pyx_v_cpha);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_72spiSlave_SetMode(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_cpol, PyObject *__pyx_v_cpha) {
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  FT4222_SPICPOL __pyx_t_1;
  FT4222_SPICPHA __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_SetMode", 0);

  /* "ft4222/ft4222.pyx":891
 * 
 *         """
 *         status = FT4222_SPISlave_SetMode(self._handle, cpol, cpha);             # <<<<<<<<<<<<<<
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((FT4222_SPICPOL)__Pyx_PyInt_As_FT4222_SPICPOL(__pyx_v_cpol)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 891, __pyx_L1_error)
  __pyx_t_2 = ((FT4222_SPICPHA)__Pyx_PyInt_As_FT4222_SPICPHA(__pyx_v_cpha)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 891, __pyx_L1_error)
  __pyx_v_status = FT4222_SPISlave_SetMode(__pyx_v_self->_handle, __pyx_t_1, __pyx_t_2);

  /* "ft4222/ft4222.pyx":892
 *         """
 *         status = FT4222_SPISlave_SetMode(self._handle, cpol, cpha);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  __pyx_t_3 = ((__pyx_v_status != FT4222_OK) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "ft4222/ft4222.pyx":893
 *         status = FT4222_SPISlave_SetMode(self._handle, cpol, cpha);
 *         if status != FT4222_OK:
 *             raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_GetRxStatus(self):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_4, __pyx_t_5, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 893, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":892
 *         """
 *         status = FT4222_SPISlave_SetMode(self._handle, cpol, cpha);
 *         if status != FT4222_OK:             # <<<<<<<<<<<<<<
 *             raise FT4222DeviceError, status
 * 
 */
  }

  /* "ft4222/ft4222.pyx":880
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_SetMode(self, cpol, cpha):             # <<<<<<<<<<<<<<
 *         """Set SPI slave cpol and cpha. The Default value of cpol is (:obj:`ft4222.SPI.Cpol.CLK_IDLE_LOW`) , default value of cpha is (:obj:`ft4222.SPI.Cpol.CLK_LEADING`)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_SetMode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":895
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_GetRxStatus(self):             # <<<<<<<<<<<<<<
 *         """Get number of bytes in the receive queue.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_75spiSlave_GetRxStatus(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_74spiSlave_GetRxStatus[] = "Get number of bytes in the receive queue.\n\n        Returns:\n            pRxSize (uint16): Number of bytes in the receive queue\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_75spiSlave_GetRxStatus(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_GetRxStatus (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_74spiSlave_GetRxStatus(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_74spiSlave_GetRxStatus(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  uint16 __pyx_v_pRxSize;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_GetRxStatus", 0);

  /* "ft4222/ft4222.pyx":908
 *             uint16 pRxSize
 * 
 *         status = FT4222_SPISlave_GetRxStatus(self._handle, &pRxSize);             # <<<<<<<<<<<<<<
 * 
 *         if status == FT4222_OK:
 */
  __pyx_v_status = FT4222_SPISlave_GetRxStatus(__pyx_v_self->_handle, (&__pyx_v_pRxSize));

  /* "ft4222/ft4222.pyx":910
 *         status = FT4222_SPISlave_GetRxStatus(self._handle, &pRxSize);
 * 
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return pRxSize
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":911
 * 
 *         if status == FT4222_OK:
 *             return pRxSize             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_uint16_t(__pyx_v_pRxSize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 911, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":910
 *         status = FT4222_SPISlave_GetRxStatus(self._handle, &pRxSize);
 * 
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return pRxSize
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":912
 *         if status == FT4222_OK:
 *             return pRxSize
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 * 
 *     def spiSlave_Write(self, data):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_2, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 912, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":895
 *             raise FT4222DeviceError, status
 * 
 *     def spiSlave_GetRxStatus(self):             # <<<<<<<<<<<<<<
 *         """Get number of bytes in the receive queue.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_GetRxStatus", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ft4222/ft4222.pyx":914
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Write(self, data):             # <<<<<<<<<<<<<<
 *         """Write data to the transmit queue of the SPI slave device.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_77spiSlave_Write(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ft4222_6ft4222_6FT4222_76spiSlave_Write[] = "Write data to the transmit queue of the SPI slave device.\n\n        Args:\n            data (bytes, bytearray, int): Data to write to slave\n\n        Returns:\n            sizeTransferred (uint16): Number of bytes written to the device.\n\n        Raises:\n            FT4222DeviceError: on error\n\n        ";
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_77spiSlave_Write(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("spiSlave_Write (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_76spiSlave_Write(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v_data));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_76spiSlave_Write(struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, PyObject *__pyx_v_data) {
  uint16 __pyx_v_sizeTransferred;
  uint8 *__pyx_v_cdata;
  FT4222_STATUS __pyx_v_status;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  uint8 *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("spiSlave_Write", 0);
  __Pyx_INCREF(__pyx_v_data);

  /* "ft4222/ft4222.pyx":927
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_data); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ft4222/ft4222.pyx":928
 *         """
 *         if isinstance(data, int):
 *             data = bytes([data])             # <<<<<<<<<<<<<<
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 */
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 928, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_data);
    __Pyx_GIVEREF(__pyx_v_data);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_data);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 928, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_data, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ft4222/ft4222.pyx":927
 * 
 *         """
 *         if isinstance(data, int):             # <<<<<<<<<<<<<<
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 */
    goto __pyx_L3;
  }

  /* "ft4222/ft4222.pyx":929
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_data); 
  __pyx_t_5 = (__pyx_t_1 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = PyByteArray_Check(__pyx_v_data); 
  __pyx_t_1 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L4_bool_binop_done:;
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ft4222/ft4222.pyx":930
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 sizeTransferred
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 930, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 930, __pyx_L1_error)

    /* "ft4222/ft4222.pyx":929
 *         if isinstance(data, int):
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):             # <<<<<<<<<<<<<<
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")
 *         cdef:
 */
  }
  __pyx_L3:;

  /* "ft4222/ft4222.pyx":933
 *         cdef:
 *             uint16 sizeTransferred
 *             uint8* cdata = data             # <<<<<<<<<<<<<<
 * 
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);
 */
  __pyx_t_6 = __Pyx_PyObject_AsWritableUString(__pyx_v_data); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 933, __pyx_L1_error)
  __pyx_v_cdata = __pyx_t_6;

  /* "ft4222/ft4222.pyx":935
 *             uint8* cdata = data
 * 
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);             # <<<<<<<<<<<<<<
 *         if status == FT4222_OK:
 *             return sizeTransferred
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 935, __pyx_L1_error)
  __pyx_v_status = FT4222_SPISlave_Write(__pyx_v_self->_handle, __pyx_v_cdata, __pyx_t_7, (&__pyx_v_sizeTransferred));

  /* "ft4222/ft4222.pyx":936
 * 
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return sizeTransferred
 *         raise FT4222DeviceError, status
 */
  __pyx_t_1 = ((__pyx_v_status == FT4222_OK) != 0);
  if (__pyx_t_1) {

    /* "ft4222/ft4222.pyx":937
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);
 *         if status == FT4222_OK:
 *             return sizeTransferred             # <<<<<<<<<<<<<<
 *         raise FT4222DeviceError, status
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyInt_From_uint16_t(__pyx_v_sizeTransferred); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 937, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "ft4222/ft4222.pyx":936
 * 
 *         status = FT4222_SPISlave_Write(self._handle, cdata, len(data), &sizeTransferred);
 *         if status == FT4222_OK:             # <<<<<<<<<<<<<<
 *             return sizeTransferred
 *         raise FT4222DeviceError, status
 */
  }

  /* "ft4222/ft4222.pyx":938
 *         if status == FT4222_OK:
 *             return sizeTransferred
 *         raise FT4222DeviceError, status             # <<<<<<<<<<<<<<
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_FT4222DeviceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 938, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyInt_From_FT4222_STATUS(__pyx_v_status); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 938, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_Raise(__pyx_t_4, __pyx_t_3, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 938, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":914
 *         raise FT4222DeviceError, status
 * 
 *     def spiSlave_Write(self, data):             # <<<<<<<<<<<<<<
 *         """Write data to the transmit queue of the SPI slave device.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.spiSlave_Write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_79__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_79__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_78__reduce_cython__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_78__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_81__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6ft4222_6ft4222_6FT4222_81__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ft4222_6ft4222_6FT4222_80__setstate_cython__(((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ft4222_6ft4222_6FT4222_80__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6ft4222_6ft4222_FT4222 *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ft4222.ft4222.FT4222.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/array.pxd":93
 *             __data_union data
 * 
 *         def __getbuffer__(self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_7cpython_5array_5array_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_7cpython_5array_5array_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7cpython_5array_5array___getbuffer__(((arrayobject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7cpython_5array_5array___getbuffer__(arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info, CYTHON_UNUSED int __pyx_v_flags) {
  PyObject *__pyx_v_item_count = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char *__pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  char __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  if (__pyx_v_info == NULL) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "cpython/array.pxd":98
 *             # In particular strided access is always provided regardless
 *             # of flags
 *             item_count = Py_SIZE(self)             # <<<<<<<<<<<<<<
 * 
 *             info.suboffsets = NULL
 */
  __pyx_t_1 = PyInt_FromSsize_t(Py_SIZE(((PyObject *)__pyx_v_self))); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_item_count = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cpython/array.pxd":100
 *             item_count = Py_SIZE(self)
 * 
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.buf = self.data.as_chars
 *             info.readonly = 0
 */
  __pyx_v_info->suboffsets = NULL;

  /* "cpython/array.pxd":101
 * 
 *             info.suboffsets = NULL
 *             info.buf = self.data.as_chars             # <<<<<<<<<<<<<<
 *             info.readonly = 0
 *             info.ndim = 1
 */
  __pyx_t_2 = __pyx_v_self->data.as_chars;
  __pyx_v_info->buf = __pyx_t_2;

  /* "cpython/array.pxd":102
 *             info.suboffsets = NULL
 *             info.buf = self.data.as_chars
 *             info.readonly = 0             # <<<<<<<<<<<<<<
 *             info.ndim = 1
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)
 */
  __pyx_v_info->readonly = 0;

  /* "cpython/array.pxd":103
 *             info.buf = self.data.as_chars
 *             info.readonly = 0
 *             info.ndim = 1             # <<<<<<<<<<<<<<
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)
 *             info.len = info.itemsize * item_count
 */
  __pyx_v_info->ndim = 1;

  /* "cpython/array.pxd":104
 *             info.readonly = 0
 *             info.ndim = 1
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)             # <<<<<<<<<<<<<<
 *             info.len = info.itemsize * item_count
 * 
 */
  __pyx_t_3 = __pyx_v_self->ob_descr->itemsize;
  __pyx_v_info->itemsize = __pyx_t_3;

  /* "cpython/array.pxd":105
 *             info.ndim = 1
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)
 *             info.len = info.itemsize * item_count             # <<<<<<<<<<<<<<
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 */
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_info->itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyNumber_Multiply(__pyx_t_1, __pyx_v_item_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_4); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_info->len = __pyx_t_5;

  /* "cpython/array.pxd":107
 *             info.len = info.itemsize * item_count
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)             # <<<<<<<<<<<<<<
 *             if not info.shape:
 *                 raise MemoryError()
 */
  __pyx_v_info->shape = ((Py_ssize_t *)PyObject_Malloc(((sizeof(Py_ssize_t)) + 2)));

  /* "cpython/array.pxd":108
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 *             if not info.shape:             # <<<<<<<<<<<<<<
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing
 */
  __pyx_t_6 = ((!(__pyx_v_info->shape != 0)) != 0);
  if (unlikely(__pyx_t_6)) {

    /* "cpython/array.pxd":109
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 *             if not info.shape:
 *                 raise MemoryError()             # <<<<<<<<<<<<<<
 *             info.shape[0] = item_count      # constant regardless of resizing
 *             info.strides = &info.itemsize
 */
    PyErr_NoMemory(); __PYX_ERR(2, 109, __pyx_L1_error)

    /* "cpython/array.pxd":108
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 *             if not info.shape:             # <<<<<<<<<<<<<<
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing
 */
  }

  /* "cpython/array.pxd":110
 *             if not info.shape:
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing             # <<<<<<<<<<<<<<
 *             info.strides = &info.itemsize
 * 
 */
  __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_v_item_count); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 110, __pyx_L1_error)
  (__pyx_v_info->shape[0]) = __pyx_t_5;

  /* "cpython/array.pxd":111
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing
 *             info.strides = &info.itemsize             # <<<<<<<<<<<<<<
 * 
 *             info.format = <char*> (info.shape + 1)
 */
  __pyx_v_info->strides = (&__pyx_v_info->itemsize);

  /* "cpython/array.pxd":113
 *             info.strides = &info.itemsize
 * 
 *             info.format = <char*> (info.shape + 1)             # <<<<<<<<<<<<<<
 *             info.format[0] = self.ob_descr.typecode
 *             info.format[1] = 0
 */
  __pyx_v_info->format = ((char *)(__pyx_v_info->shape + 1));

  /* "cpython/array.pxd":114
 * 
 *             info.format = <char*> (info.shape + 1)
 *             info.format[0] = self.ob_descr.typecode             # <<<<<<<<<<<<<<
 *             info.format[1] = 0
 *             info.obj = self
 */
  __pyx_t_7 = __pyx_v_self->ob_descr->typecode;
  (__pyx_v_info->format[0]) = __pyx_t_7;

  /* "cpython/array.pxd":115
 *             info.format = <char*> (info.shape + 1)
 *             info.format[0] = self.ob_descr.typecode
 *             info.format[1] = 0             # <<<<<<<<<<<<<<
 *             info.obj = self
 * 
 */
  (__pyx_v_info->format[1]) = 0;

  /* "cpython/array.pxd":116
 *             info.format[0] = self.ob_descr.typecode
 *             info.format[1] = 0
 *             info.obj = self             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(self, Py_buffer* info):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "cpython/array.pxd":93
 *             __data_union data
 * 
 *         def __getbuffer__(self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cpython.array.array.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_item_count);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/array.pxd":118
 *             info.obj = self
 * 
 *         def __releasebuffer__(self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             PyObject_Free(info.shape)
 * 
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_7cpython_5array_5array_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_7cpython_5array_5array_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_7cpython_5array_5array_2__releasebuffer__(((arrayobject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7cpython_5array_5array_2__releasebuffer__(CYTHON_UNUSED arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "cpython/array.pxd":119
 * 
 *         def __releasebuffer__(self, Py_buffer* info):
 *             PyObject_Free(info.shape)             # <<<<<<<<<<<<<<
 * 
 *     array newarrayobject(PyTypeObject* type, Py_ssize_t size, arraydescr *descr)
 */
  PyObject_Free(__pyx_v_info->shape);

  /* "cpython/array.pxd":118
 *             info.obj = self
 * 
 *         def __releasebuffer__(self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             PyObject_Free(info.shape)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "cpython/array.pxd":130
 * 
 * 
 * cdef inline array clone(array template, Py_ssize_t length, bint zero):             # <<<<<<<<<<<<<<
 *     """ fast creation of a new array, given a template array.
 *     type will be same as template.
 */

static CYTHON_INLINE arrayobject *__pyx_f_7cpython_5array_clone(arrayobject *__pyx_v_template, Py_ssize_t __pyx_v_length, int __pyx_v_zero) {
  arrayobject *__pyx_v_op = 0;
  arrayobject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);

  /* "cpython/array.pxd":134
 *     type will be same as template.
 *     if zero is true, new array will be initialized with zeroes."""
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)             # <<<<<<<<<<<<<<
 *     if zero and op is not None:
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 */
  __pyx_t_1 = ((PyObject *)newarrayobject(Py_TYPE(((PyObject *)__pyx_v_template)), __pyx_v_length, __pyx_v_template->ob_descr)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_op = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cpython/array.pxd":135
 *     if zero is true, new array will be initialized with zeroes."""
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)
 *     if zero and op is not None:             # <<<<<<<<<<<<<<
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 *     return op
 */
  __pyx_t_3 = (__pyx_v_zero != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (((PyObject *)__pyx_v_op) != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "cpython/array.pxd":136
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)
 *     if zero and op is not None:
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)             # <<<<<<<<<<<<<<
 *     return op
 * 
 */
    (void)(memset(__pyx_v_op->data.as_chars, 0, (__pyx_v_length * __pyx_v_op->ob_descr->itemsize)));

    /* "cpython/array.pxd":135
 *     if zero is true, new array will be initialized with zeroes."""
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)
 *     if zero and op is not None:             # <<<<<<<<<<<<<<
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 *     return op
 */
  }

  /* "cpython/array.pxd":137
 *     if zero and op is not None:
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 *     return op             # <<<<<<<<<<<<<<
 * 
 * cdef inline array copy(array self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_op));
  __pyx_r = __pyx_v_op;
  goto __pyx_L0;

  /* "cpython/array.pxd":130
 * 
 * 
 * cdef inline array clone(array template, Py_ssize_t length, bint zero):             # <<<<<<<<<<<<<<
 *     """ fast creation of a new array, given a template array.
 *     type will be same as template.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.array.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_op);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/array.pxd":139
 *     return op
 * 
 * cdef inline array copy(array self):             # <<<<<<<<<<<<<<
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 */

static CYTHON_INLINE arrayobject *__pyx_f_7cpython_5array_copy(arrayobject *__pyx_v_self) {
  arrayobject *__pyx_v_op = 0;
  arrayobject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "cpython/array.pxd":141
 * cdef inline array copy(array self):
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)             # <<<<<<<<<<<<<<
 *     memcpy(op.data.as_chars, self.data.as_chars, Py_SIZE(op) * op.ob_descr.itemsize)
 *     return op
 */
  __pyx_t_1 = ((PyObject *)newarrayobject(Py_TYPE(((PyObject *)__pyx_v_self)), Py_SIZE(((PyObject *)__pyx_v_self)), __pyx_v_self->ob_descr)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_op = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cpython/array.pxd":142
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 *     memcpy(op.data.as_chars, self.data.as_chars, Py_SIZE(op) * op.ob_descr.itemsize)             # <<<<<<<<<<<<<<
 *     return op
 * 
 */
  (void)(memcpy(__pyx_v_op->data.as_chars, __pyx_v_self->data.as_chars, (Py_SIZE(((PyObject *)__pyx_v_op)) * __pyx_v_op->ob_descr->itemsize)));

  /* "cpython/array.pxd":143
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 *     memcpy(op.data.as_chars, self.data.as_chars, Py_SIZE(op) * op.ob_descr.itemsize)
 *     return op             # <<<<<<<<<<<<<<
 * 
 * cdef inline int extend_buffer(array self, char* stuff, Py_ssize_t n) except -1:
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_op));
  __pyx_r = __pyx_v_op;
  goto __pyx_L0;

  /* "cpython/array.pxd":139
 *     return op
 * 
 * cdef inline array copy(array self):             # <<<<<<<<<<<<<<
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.array.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_op);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/array.pxd":145
 *     return op
 * 
 * cdef inline int extend_buffer(array self, char* stuff, Py_ssize_t n) except -1:             # <<<<<<<<<<<<<<
 *     """ efficient appending of new stuff of same type
 *     (e.g. of same array type)
 */

static CYTHON_INLINE int __pyx_f_7cpython_5array_extend_buffer(arrayobject *__pyx_v_self, char *__pyx_v_stuff, Py_ssize_t __pyx_v_n) {
  Py_ssize_t __pyx_v_itemsize;
  Py_ssize_t __pyx_v_origsize;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("extend_buffer", 0);

  /* "cpython/array.pxd":149
 *     (e.g. of same array type)
 *     n: number of elements (not number of bytes!) """
 *     cdef Py_ssize_t itemsize = self.ob_descr.itemsize             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t origsize = Py_SIZE(self)
 *     resize_smart(self, origsize + n)
 */
  __pyx_t_1 = __pyx_v_self->ob_descr->itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "cpython/array.pxd":150
 *     n: number of elements (not number of bytes!) """
 *     cdef Py_ssize_t itemsize = self.ob_descr.itemsize
 *     cdef Py_ssize_t origsize = Py_SIZE(self)             # <<<<<<<<<<<<<<
 *     resize_smart(self, origsize + n)
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)
 */
  __pyx_v_origsize = Py_SIZE(((PyObject *)__pyx_v_self));

  /* "cpython/array.pxd":151
 *     cdef Py_ssize_t itemsize = self.ob_descr.itemsize
 *     cdef Py_ssize_t origsize = Py_SIZE(self)
 *     resize_smart(self, origsize + n)             # <<<<<<<<<<<<<<
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)
 *     return 0
 */
  __pyx_t_1 = resize_smart(__pyx_v_self, (__pyx_v_origsize + __pyx_v_n)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 151, __pyx_L1_error)

  /* "cpython/array.pxd":152
 *     cdef Py_ssize_t origsize = Py_SIZE(self)
 *     resize_smart(self, origsize + n)
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (void)(memcpy((__pyx_v_self->data.as_chars + (__pyx_v_origsize * __pyx_v_itemsize)), __pyx_v_stuff, (__pyx_v_n * __pyx_v_itemsize)));

  /* "cpython/array.pxd":153
 *     resize_smart(self, origsize + n)
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline int extend(array self, array other) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cpython/array.pxd":145
 *     return op
 * 
 * cdef inline int extend_buffer(array self, char* stuff, Py_ssize_t n) except -1:             # <<<<<<<<<<<<<<
 *     """ efficient appending of new stuff of same type
 *     (e.g. of same array type)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cpython.array.extend_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/array.pxd":155
 *     return 0
 * 
 * cdef inline int extend(array self, array other) except -1:             # <<<<<<<<<<<<<<
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 */

static CYTHON_INLINE int __pyx_f_7cpython_5array_extend(arrayobject *__pyx_v_self, arrayobject *__pyx_v_other) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("extend", 0);

  /* "cpython/array.pxd":157
 * cdef inline int extend(array self, array other) except -1:
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:             # <<<<<<<<<<<<<<
 *         PyErr_BadArgument()
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 */
  __pyx_t_1 = ((__pyx_v_self->ob_descr->typecode != __pyx_v_other->ob_descr->typecode) != 0);
  if (__pyx_t_1) {

    /* "cpython/array.pxd":158
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 *         PyErr_BadArgument()             # <<<<<<<<<<<<<<
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 */
    __pyx_t_2 = PyErr_BadArgument(); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(2, 158, __pyx_L1_error)

    /* "cpython/array.pxd":157
 * cdef inline int extend(array self, array other) except -1:
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:             # <<<<<<<<<<<<<<
 *         PyErr_BadArgument()
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 */
  }

  /* "cpython/array.pxd":159
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 *         PyErr_BadArgument()
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))             # <<<<<<<<<<<<<<
 * 
 * cdef inline void zero(array self):
 */
  __pyx_t_2 = __pyx_f_7cpython_5array_extend_buffer(__pyx_v_self, __pyx_v_other->data.as_chars, Py_SIZE(((PyObject *)__pyx_v_other))); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 159, __pyx_L1_error)
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "cpython/array.pxd":155
 *     return 0
 * 
 * cdef inline int extend(array self, array other) except -1:             # <<<<<<<<<<<<<<
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cpython.array.extend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/array.pxd":161
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 * cdef inline void zero(array self):             # <<<<<<<<<<<<<<
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)
 */

static CYTHON_INLINE void __pyx_f_7cpython_5array_zero(arrayobject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("zero", 0);

  /* "cpython/array.pxd":163
 * cdef inline void zero(array self):
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)             # <<<<<<<<<<<<<<
 */
  (void)(memset(__pyx_v_self->data.as_chars, 0, (Py_SIZE(((PyObject *)__pyx_v_self)) * __pyx_v_self->ob_descr->itemsize)));

  /* "cpython/array.pxd":161
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 * cdef inline void zero(array self):             # <<<<<<<<<<<<<<
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}
static struct __pyx_vtabstruct_6ft4222_6ft4222__closer __pyx_vtable_6ft4222_6ft4222__closer;

static PyObject *__pyx_tp_new_6ft4222_6ft4222__closer(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6ft4222_6ft4222__closer *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ft4222_6ft4222__closer *)o);
  p->__pyx_vtab = __pyx_vtabptr_6ft4222_6ft4222__closer;
  return o;
}

static void __pyx_tp_dealloc_6ft4222_6ft4222__closer(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_6ft4222_6ft4222__closer[] = {
  {"close", (PyCFunction)__pyx_pw_6ft4222_6ft4222_7_closer_1close, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ft4222_6ft4222_7_closer_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ft4222_6ft4222_7_closer_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ft4222_6ft4222__closer = {
  PyVarObject_HEAD_INIT(0, 0)
  "ft4222.ft4222._closer", /*tp_name*/
  sizeof(struct __pyx_obj_6ft4222_6ft4222__closer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ft4222_6ft4222__closer, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "Wrap a hid_device *ptr and a provide a way to call hid_close() on it.\n    Used internally for weakref.finalize, which only accepts Python objects.\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ft4222_6ft4222__closer, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ft4222_6ft4222__closer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_6ft4222_6ft4222_FT4222 __pyx_vtable_6ft4222_6ft4222_FT4222;

static PyObject *__pyx_tp_new_6ft4222_6ft4222_FT4222(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6ft4222_6ft4222_FT4222 *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ft4222_6ft4222_FT4222 *)o);
  p->__pyx_vtab = __pyx_vtabptr_6ft4222_6ft4222_FT4222;
  p->_close = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ft4222_6ft4222_FT4222(PyObject *o) {
  struct __pyx_obj_6ft4222_6ft4222_FT4222 *p = (struct __pyx_obj_6ft4222_6ft4222_FT4222 *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  if (p->__weakref__) PyObject_ClearWeakRefs(o);
  Py_CLEAR(p->_close);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6ft4222_6ft4222_FT4222(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ft4222_6ft4222_FT4222 *p = (struct __pyx_obj_6ft4222_6ft4222_FT4222 *)o;
  if (p->_close) {
    e = (*v)(p->_close, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ft4222_6ft4222_FT4222(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ft4222_6ft4222_FT4222 *p = (struct __pyx_obj_6ft4222_6ft4222_FT4222 *)o;
  tmp = ((PyObject*)p->_close);
  p->_close = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ft4222_6ft4222_6FT4222_chipVersion(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ft4222_6ft4222_6FT4222_11chipVersion_1__get__(o);
}

static PyObject *__pyx_getprop_6ft4222_6ft4222_6FT4222_libVersion(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ft4222_6ft4222_6FT4222_10libVersion_1__get__(o);
}

static PyObject *__pyx_getprop_6ft4222_6ft4222_6FT4222_chipRevision(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ft4222_6ft4222_6FT4222_12chipRevision_1__get__(o);
}

static PyMethodDef __pyx_methods_6ft4222_6ft4222_FT4222[] = {
  {"close", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_3close, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_6FT4222_2close},
  {"setTimeouts", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_7setTimeouts, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_6setTimeouts},
  {"setClock", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_9setClock, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_8setClock},
  {"getClock", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_11getClock, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_6FT4222_10getClock},
  {"setSuspendOut", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_13setSuspendOut, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_12setSuspendOut},
  {"setWakeUpInterrupt", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_15setWakeUpInterrupt, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_14setWakeUpInterrupt},
  {"vendorCmdGet", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_17vendorCmdGet, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_16vendorCmdGet},
  {"vendorCmdSet", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_19vendorCmdSet, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_18vendorCmdSet},
  {"gpio_Init", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_21gpio_Init, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_20gpio_Init},
  {"gpio_Read", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_23gpio_Read, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_22gpio_Read},
  {"gpio_Write", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_25gpio_Write, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_24gpio_Write},
  {"gpio_SetInputTrigger", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_27gpio_SetInputTrigger, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_26gpio_SetInputTrigger},
  {"gpio_GetTriggerStatus", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_29gpio_GetTriggerStatus, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_28gpio_GetTriggerStatus},
  {"gpio_ReadTriggerQueue", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_31gpio_ReadTriggerQueue, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_30gpio_ReadTriggerQueue},
  {"i2cMaster_Init", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_33i2cMaster_Init, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_32i2cMaster_Init},
  {"i2cMaster_Read", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_35i2cMaster_Read, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_34i2cMaster_Read},
  {"i2cMaster_Write", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_37i2cMaster_Write, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_36i2cMaster_Write},
  {"i2cMaster_ReadEx", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_39i2cMaster_ReadEx, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_38i2cMaster_ReadEx},
  {"i2cMaster_WriteEx", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_41i2cMaster_WriteEx, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_40i2cMaster_WriteEx},
  {"i2cMaster_Reset", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_43i2cMaster_Reset, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_6FT4222_42i2cMaster_Reset},
  {"i2cMaster_GetStatus", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_45i2cMaster_GetStatus, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_6FT4222_44i2cMaster_GetStatus},
  {"spi_Reset", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_47spi_Reset, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_6FT4222_46spi_Reset},
  {"spi_ResetTransaction", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_49spi_ResetTransaction, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_48spi_ResetTransaction},
  {"spi_SetDrivingStrength", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_51spi_SetDrivingStrength, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_50spi_SetDrivingStrength},
  {"spiMaster_Init", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_53spiMaster_Init, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_52spiMaster_Init},
  {"spiMaster_SetLines", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_55spiMaster_SetLines, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_54spiMaster_SetLines},
  {"spiMaster_SingleRead", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_57spiMaster_SingleRead, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_56spiMaster_SingleRead},
  {"spiMaster_SingleWrite", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_59spiMaster_SingleWrite, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_58spiMaster_SingleWrite},
  {"spiMaster_SingleReadWrite", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_61spiMaster_SingleReadWrite, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_60spiMaster_SingleReadWrite},
  {"spiMaster_MultiReadWrite", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_63spiMaster_MultiReadWrite, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_62spiMaster_MultiReadWrite},
  {"spiMaster_EndTransaction", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_65spiMaster_EndTransaction, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_6FT4222_64spiMaster_EndTransaction},
  {"spiSlave_Init", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_67spiSlave_Init, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_6FT4222_66spiSlave_Init},
  {"spiSlave_InitEx", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_69spiSlave_InitEx, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_68spiSlave_InitEx},
  {"spiSlave_Read", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_71spiSlave_Read, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_70spiSlave_Read},
  {"spiSlave_SetMode", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ft4222_6ft4222_6FT4222_73spiSlave_SetMode, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ft4222_6ft4222_6FT4222_72spiSlave_SetMode},
  {"spiSlave_GetRxStatus", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_75spiSlave_GetRxStatus, METH_NOARGS, __pyx_doc_6ft4222_6ft4222_6FT4222_74spiSlave_GetRxStatus},
  {"spiSlave_Write", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_77spiSlave_Write, METH_O, __pyx_doc_6ft4222_6ft4222_6FT4222_76spiSlave_Write},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_79__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ft4222_6ft4222_6FT4222_81__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ft4222_6ft4222_FT4222[] = {
  {(char *)"chipVersion", __pyx_getprop_6ft4222_6ft4222_6FT4222_chipVersion, 0, (char *)"Chip version as number", 0},
  {(char *)"libVersion", __pyx_getprop_6ft4222_6ft4222_6FT4222_libVersion, 0, (char *)"Library version as number", 0},
  {(char *)"chipRevision", __pyx_getprop_6ft4222_6ft4222_6FT4222_chipRevision, 0, (char *)"The revision of the chip in human readable format", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ft4222_6ft4222_FT4222 = {
  PyVarObject_HEAD_INIT(0, 0)
  "ft4222.ft4222.FT4222", /*tp_name*/
  sizeof(struct __pyx_obj_6ft4222_6ft4222_FT4222), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ft4222_6ft4222_FT4222, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_6ft4222_6ft4222_6FT4222_5__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6ft4222_6ft4222_FT4222, /*tp_traverse*/
  __pyx_tp_clear_6ft4222_6ft4222_FT4222, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ft4222_6ft4222_FT4222, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ft4222_6ft4222_FT4222, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ft4222_6ft4222_6FT4222_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ft4222_6ft4222_FT4222, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_ft4222(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_ft4222},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "ft4222",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_u_B, __pyx_k_B, sizeof(__pyx_k_B), 0, 1, 0, 1},
  {&__pyx_n_s_CLK_24, __pyx_k_CLK_24, sizeof(__pyx_k_CLK_24), 0, 0, 1, 1},
  {&__pyx_n_s_CLK_48, __pyx_k_CLK_48, sizeof(__pyx_k_CLK_48), 0, 0, 1, 1},
  {&__pyx_n_s_CLK_60, __pyx_k_CLK_60, sizeof(__pyx_k_CLK_60), 0, 0, 1, 1},
  {&__pyx_n_s_CLK_80, __pyx_k_CLK_80, sizeof(__pyx_k_CLK_80), 0, 0, 1, 1},
  {&__pyx_n_u_CLK_NOT_SUPPORTED, __pyx_k_CLK_NOT_SUPPORTED, sizeof(__pyx_k_CLK_NOT_SUPPORTED), 0, 1, 0, 1},
  {&__pyx_kp_s_Chip_system_clock_Attributes_CLK, __pyx_k_Chip_system_clock_Attributes_CLK, sizeof(__pyx_k_Chip_system_clock_Attributes_CLK), 0, 0, 1, 0},
  {&__pyx_n_s_ControllerStatus, __pyx_k_ControllerStatus, sizeof(__pyx_k_ControllerStatus), 0, 0, 1, 1},
  {&__pyx_n_u_DEVICE_LIST_NOT_READY, __pyx_k_DEVICE_LIST_NOT_READY, sizeof(__pyx_k_DEVICE_LIST_NOT_READY), 0, 1, 0, 1},
  {&__pyx_n_u_DEVICE_NOT_FOUND, __pyx_k_DEVICE_NOT_FOUND, sizeof(__pyx_k_DEVICE_NOT_FOUND), 0, 1, 0, 1},
  {&__pyx_n_u_DEVICE_NOT_OPENED, __pyx_k_DEVICE_NOT_OPENED, sizeof(__pyx_k_DEVICE_NOT_OPENED), 0, 1, 0, 1},
  {&__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE, __pyx_k_DEVICE_NOT_OPENED_FOR_ERASE, sizeof(__pyx_k_DEVICE_NOT_OPENED_FOR_ERASE), 0, 1, 0, 1},
  {&__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE, __pyx_k_DEVICE_NOT_OPENED_FOR_WRITE, sizeof(__pyx_k_DEVICE_NOT_OPENED_FOR_WRITE), 0, 1, 0, 1},
  {&__pyx_n_u_DEVICE_NOT_SUPPORTED, __pyx_k_DEVICE_NOT_SUPPORTED, sizeof(__pyx_k_DEVICE_NOT_SUPPORTED), 0, 1, 0, 1},
  {&__pyx_n_s_Dir, __pyx_k_Dir, sizeof(__pyx_k_Dir), 0, 0, 1, 1},
  {&__pyx_n_u_EEPROM_ERASE_FAILED, __pyx_k_EEPROM_ERASE_FAILED, sizeof(__pyx_k_EEPROM_ERASE_FAILED), 0, 1, 0, 1},
  {&__pyx_n_u_EEPROM_NOT_PRESENT, __pyx_k_EEPROM_NOT_PRESENT, sizeof(__pyx_k_EEPROM_NOT_PRESENT), 0, 1, 0, 1},
  {&__pyx_n_u_EEPROM_NOT_PROGRAMMED, __pyx_k_EEPROM_NOT_PROGRAMMED, sizeof(__pyx_k_EEPROM_NOT_PROGRAMMED), 0, 1, 0, 1},
  {&__pyx_n_u_EEPROM_READ_FAILED, __pyx_k_EEPROM_READ_FAILED, sizeof(__pyx_k_EEPROM_READ_FAILED), 0, 1, 0, 1},
  {&__pyx_n_u_EEPROM_WRITE_FAILED, __pyx_k_EEPROM_WRITE_FAILED, sizeof(__pyx_k_EEPROM_WRITE_FAILED), 0, 1, 0, 1},
  {&__pyx_n_u_EVENT_NOT_SUPPORTED, __pyx_k_EVENT_NOT_SUPPORTED, sizeof(__pyx_k_EVENT_NOT_SUPPORTED), 0, 1, 0, 1},
  {&__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE, __pyx_k_EXCEEDED_MAX_TRANSFER_SIZE, sizeof(__pyx_k_EXCEEDED_MAX_TRANSFER_SIZE), 0, 1, 0, 1},
  {&__pyx_kp_s_Exception_class_for_status_messa, __pyx_k_Exception_class_for_status_messa, sizeof(__pyx_k_Exception_class_for_status_messa), 0, 0, 1, 0},
  {&__pyx_n_u_FAILED_TO_READ_DEVICE, __pyx_k_FAILED_TO_READ_DEVICE, sizeof(__pyx_k_FAILED_TO_READ_DEVICE), 0, 1, 0, 1},
  {&__pyx_n_u_FAILED_TO_WRITE_DEVICE0, __pyx_k_FAILED_TO_WRITE_DEVICE0, sizeof(__pyx_k_FAILED_TO_WRITE_DEVICE0), 0, 1, 0, 1},
  {&__pyx_n_s_FT2XXDeviceError, __pyx_k_FT2XXDeviceError, sizeof(__pyx_k_FT2XXDeviceError), 0, 0, 1, 1},
  {&__pyx_n_s_FT2XXDeviceError___init, __pyx_k_FT2XXDeviceError___init, sizeof(__pyx_k_FT2XXDeviceError___init), 0, 0, 1, 1},
  {&__pyx_n_s_FT2XXDeviceError___str, __pyx_k_FT2XXDeviceError___str, sizeof(__pyx_k_FT2XXDeviceError___str), 0, 0, 1, 1},
  {&__pyx_n_s_FT4222, __pyx_k_FT4222, sizeof(__pyx_k_FT4222), 0, 0, 1, 1},
  {&__pyx_n_s_FT4222DeviceError, __pyx_k_FT4222DeviceError, sizeof(__pyx_k_FT4222DeviceError), 0, 0, 1, 1},
  {&__pyx_n_s_FT4222DeviceError___init, __pyx_k_FT4222DeviceError___init, sizeof(__pyx_k_FT4222DeviceError___init), 0, 0, 1, 1},
  {&__pyx_n_s_FT4222DeviceError___str, __pyx_k_FT4222DeviceError___str, sizeof(__pyx_k_FT4222DeviceError___str), 0, 0, 1, 1},
  {&__pyx_kp_u_FT4222_chipVersion_0x_x_s_libVer, __pyx_k_FT4222_chipVersion_0x_x_s_libVer, sizeof(__pyx_k_FT4222_chipVersion_0x_x_s_libVer), 0, 1, 0, 0},
  {&__pyx_n_u_FUN_NOT_SUPPORT, __pyx_k_FUN_NOT_SUPPORT, sizeof(__pyx_k_FUN_NOT_SUPPORT), 0, 1, 0, 1},
  {&__pyx_n_s_GPIO, __pyx_k_GPIO, sizeof(__pyx_k_GPIO), 0, 0, 1, 1},
  {&__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM, __pyx_k_GPIO_EXCEEDED_MAX_PORTNUM, sizeof(__pyx_k_GPIO_EXCEEDED_MAX_PORTNUM), 0, 1, 0, 1},
  {&__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED, __pyx_k_GPIO_INPUT_NOT_SUPPORTED, sizeof(__pyx_k_GPIO_INPUT_NOT_SUPPORTED), 0, 1, 0, 1},
  {&__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE, __pyx_k_GPIO_NOT_SUPPORTED_IN_THIS_MODE, sizeof(__pyx_k_GPIO_NOT_SUPPORTED_IN_THIS_MODE), 0, 1, 0, 1},
  {&__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT, __pyx_k_GPIO_OPENDRAIN_INVALID_IN_OUTPUT, sizeof(__pyx_k_GPIO_OPENDRAIN_INVALID_IN_OUTPUT), 0, 1, 0, 1},
  {&__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO, __pyx_k_GPIO_PULLDOWN_INVALID_IN_INPUTMO, sizeof(__pyx_k_GPIO_PULLDOWN_INVALID_IN_INPUTMO), 0, 1, 0, 1},
  {&__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE, __pyx_k_GPIO_PULLUP_INVALID_IN_INPUTMODE, sizeof(__pyx_k_GPIO_PULLUP_INVALID_IN_INPUTMODE), 0, 1, 0, 1},
  {&__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED, __pyx_k_GPIO_WRITE_NOT_SUPPORTED, sizeof(__pyx_k_GPIO_WRITE_NOT_SUPPORTED), 0, 1, 0, 1},
  {&__pyx_n_s_I2CMaster, __pyx_k_I2CMaster, sizeof(__pyx_k_I2CMaster), 0, 0, 1, 1},
  {&__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE, __pyx_k_I2C_NOT_SUPPORTED_IN_THIS_MODE, sizeof(__pyx_k_I2C_NOT_SUPPORTED_IN_THIS_MODE), 0, 1, 0, 1},
  {&__pyx_n_s_INPUT, __pyx_k_INPUT, sizeof(__pyx_k_INPUT), 0, 0, 1, 1},
  {&__pyx_n_u_INSUFFICIENT_RESOURCES, __pyx_k_INSUFFICIENT_RESOURCES, sizeof(__pyx_k_INSUFFICIENT_RESOURCES), 0, 1, 0, 1},
  {&__pyx_n_u_INTERRUPT_NOT_SUPPORTED, __pyx_k_INTERRUPT_NOT_SUPPORTED, sizeof(__pyx_k_INTERRUPT_NOT_SUPPORTED), 0, 1, 0, 1},
  {&__pyx_n_u_INVAILD_FUNCTION, __pyx_k_INVAILD_FUNCTION, sizeof(__pyx_k_INVAILD_FUNCTION), 0, 1, 0, 1},
  {&__pyx_n_u_INVALID_ARGS, __pyx_k_INVALID_ARGS, sizeof(__pyx_k_INVALID_ARGS), 0, 1, 0, 1},
  {&__pyx_n_u_INVALID_BAUD_RATE, __pyx_k_INVALID_BAUD_RATE, sizeof(__pyx_k_INVALID_BAUD_RATE), 0, 1, 0, 1},
  {&__pyx_n_u_INVALID_HANDLE, __pyx_k_INVALID_HANDLE, sizeof(__pyx_k_INVALID_HANDLE), 0, 1, 0, 1},
  {&__pyx_n_u_INVALID_PARAMETER, __pyx_k_INVALID_PARAMETER, sizeof(__pyx_k_INVALID_PARAMETER), 0, 1, 0, 1},
  {&__pyx_n_u_INVALID_POINTER, __pyx_k_INVALID_POINTER, sizeof(__pyx_k_INVALID_POINTER), 0, 1, 0, 1},
  {&__pyx_n_u_IO_ERROR, __pyx_k_IO_ERROR, sizeof(__pyx_k_IO_ERROR), 0, 1, 0, 1},
  {&__pyx_n_u_IS_NOT_I2C_MODE, __pyx_k_IS_NOT_I2C_MODE, sizeof(__pyx_k_IS_NOT_I2C_MODE), 0, 1, 0, 1},
  {&__pyx_n_u_IS_NOT_SPI_MODE, __pyx_k_IS_NOT_SPI_MODE, sizeof(__pyx_k_IS_NOT_SPI_MODE), 0, 1, 0, 1},
  {&__pyx_n_u_IS_NOT_SPI_MULTI_MODE, __pyx_k_IS_NOT_SPI_MULTI_MODE, sizeof(__pyx_k_IS_NOT_SPI_MULTI_MODE), 0, 1, 0, 1},
  {&__pyx_n_u_IS_NOT_SPI_SINGLE_MODE, __pyx_k_IS_NOT_SPI_SINGLE_MODE, sizeof(__pyx_k_IS_NOT_SPI_SINGLE_MODE), 0, 1, 0, 1},
  {&__pyx_n_s_IntEnum, __pyx_k_IntEnum, sizeof(__pyx_k_IntEnum), 0, 0, 1, 1},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_n_u_NOT_SUPPORTED, __pyx_k_NOT_SUPPORTED, sizeof(__pyx_k_NOT_SUPPORTED), 0, 1, 0, 1},
  {&__pyx_n_u_OK, __pyx_k_OK, sizeof(__pyx_k_OK), 0, 1, 0, 1},
  {&__pyx_n_u_OTHER_ERROR, __pyx_k_OTHER_ERROR, sizeof(__pyx_k_OTHER_ERROR), 0, 1, 0, 1},
  {&__pyx_kp_u_Rev_A, __pyx_k_Rev_A, sizeof(__pyx_k_Rev_A), 0, 1, 0, 0},
  {&__pyx_kp_u_Rev_B, __pyx_k_Rev_B, sizeof(__pyx_k_Rev_B), 0, 1, 0, 0},
  {&__pyx_kp_u_Rev_C, __pyx_k_Rev_C, sizeof(__pyx_k_Rev_C), 0, 1, 0, 0},
  {&__pyx_kp_u_Rev_unknown, __pyx_k_Rev_unknown, sizeof(__pyx_k_Rev_unknown), 0, 1, 0, 0},
  {&__pyx_n_s_SysClock, __pyx_k_SysClock, sizeof(__pyx_k_SysClock), 0, 0, 1, 1},
  {&__pyx_n_s_Trigger, __pyx_k_Trigger, sizeof(__pyx_k_Trigger), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_u_VENDER_CMD_NOT_SUPPORTED, __pyx_k_VENDER_CMD_NOT_SUPPORTED, sizeof(__pyx_k_VENDER_CMD_NOT_SUPPORTED), 0, 1, 0, 1},
  {&__pyx_n_u_WRONG_I2C_ADDR, __pyx_k_WRONG_I2C_ADDR, sizeof(__pyx_k_WRONG_I2C_ADDR), 0, 1, 0, 1},
  {&__pyx_n_s_addr, __pyx_k_addr, sizeof(__pyx_k_addr), 0, 0, 1, 1},
  {&__pyx_n_s_bytesToRead, __pyx_k_bytesToRead, sizeof(__pyx_k_bytesToRead), 0, 0, 1, 1},
  {&__pyx_n_s_cdesc, __pyx_k_cdesc, sizeof(__pyx_k_cdesc), 0, 0, 1, 1},
  {&__pyx_n_s_chipRevision, __pyx_k_chipRevision, sizeof(__pyx_k_chipRevision), 0, 0, 1, 1},
  {&__pyx_n_s_chip_rev_map, __pyx_k_chip_rev_map, sizeof(__pyx_k_chip_rev_map), 0, 0, 1, 1},
  {&__pyx_n_s_chip_rev_min_lib, __pyx_k_chip_rev_min_lib, sizeof(__pyx_k_chip_rev_min_lib), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_clkStrength, __pyx_k_clkStrength, sizeof(__pyx_k_clkStrength), 0, 0, 1, 1},
  {&__pyx_n_s_clock, __pyx_k_clock, sizeof(__pyx_k_clock), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_closer, __pyx_k_closer, sizeof(__pyx_k_closer), 0, 0, 1, 1},
  {&__pyx_n_s_cpha, __pyx_k_cpha, sizeof(__pyx_k_cpha), 0, 0, 1, 1},
  {&__pyx_n_s_cpol, __pyx_k_cpol, sizeof(__pyx_k_cpol), 0, 0, 1, 1},
  {&__pyx_n_s_createDeviceInfoList, __pyx_k_createDeviceInfoList, sizeof(__pyx_k_createDeviceInfoList), 0, 0, 1, 1},
  {&__pyx_n_s_cserial, __pyx_k_cserial, sizeof(__pyx_k_cserial), 0, 0, 1, 1},
  {&__pyx_n_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_desc, __pyx_k_desc, sizeof(__pyx_k_desc), 0, 0, 1, 1},
  {&__pyx_n_u_description, __pyx_k_description, sizeof(__pyx_k_description), 0, 1, 0, 1},
  {&__pyx_n_s_devnum, __pyx_k_devnum, sizeof(__pyx_k_devnum), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_enum, __pyx_k_enum, sizeof(__pyx_k_enum), 0, 0, 1, 1},
  {&__pyx_n_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 1},
  {&__pyx_n_s_finalize, __pyx_k_finalize, sizeof(__pyx_k_finalize), 0, 0, 1, 1},
  {&__pyx_n_s_flag, __pyx_k_flag, sizeof(__pyx_k_flag), 0, 0, 1, 1},
  {&__pyx_n_u_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 1, 0, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_ft4222_ft4222, __pyx_k_ft4222_ft4222, sizeof(__pyx_k_ft4222_ft4222), 0, 0, 1, 1},
  {&__pyx_kp_s_ft4222_ft4222_pyx, __pyx_k_ft4222_ft4222_pyx, sizeof(__pyx_k_ft4222_ft4222_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_ftd2xx_msgs, __pyx_k_ftd2xx_msgs, sizeof(__pyx_k_ftd2xx_msgs), 0, 0, 1, 1},
  {&__pyx_n_s_ftd4222_msgs, __pyx_k_ftd4222_msgs, sizeof(__pyx_k_ftd4222_msgs), 0, 0, 1, 1},
  {&__pyx_n_s_getDeviceInfoDetail, __pyx_k_getDeviceInfoDetail, sizeof(__pyx_k_getDeviceInfoDetail), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_gpio0, __pyx_k_gpio0, sizeof(__pyx_k_gpio0), 0, 0, 1, 1},
  {&__pyx_n_s_gpio1, __pyx_k_gpio1, sizeof(__pyx_k_gpio1), 0, 0, 1, 1},
  {&__pyx_n_s_gpio2, __pyx_k_gpio2, sizeof(__pyx_k_gpio2), 0, 0, 1, 1},
  {&__pyx_n_s_gpio3, __pyx_k_gpio3, sizeof(__pyx_k_gpio3), 0, 0, 1, 1},
  {&__pyx_n_s_gpio_GetTriggerStatus, __pyx_k_gpio_GetTriggerStatus, sizeof(__pyx_k_gpio_GetTriggerStatus), 0, 0, 1, 1},
  {&__pyx_n_s_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 0, 1, 1},
  {&__pyx_n_s_handle, __pyx_k_handle, sizeof(__pyx_k_handle), 0, 0, 1, 1},
  {&__pyx_n_u_handle, __pyx_k_handle, sizeof(__pyx_k_handle), 0, 1, 0, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_u_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_u_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 1, 0, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_ioStrength, __pyx_k_ioStrength, sizeof(__pyx_k_ioStrength), 0, 0, 1, 1},
  {&__pyx_n_s_isEndTransaction, __pyx_k_isEndTransaction, sizeof(__pyx_k_isEndTransaction), 0, 0, 1, 1},
  {&__pyx_n_s_kbps, __pyx_k_kbps, sizeof(__pyx_k_kbps), 0, 0, 1, 1},
  {&__pyx_n_s_l, __pyx_k_l, sizeof(__pyx_k_l), 0, 0, 1, 1},
  {&__pyx_n_s_locId, __pyx_k_locId, sizeof(__pyx_k_locId), 0, 0, 1, 1},
  {&__pyx_n_u_location, __pyx_k_location, sizeof(__pyx_k_location), 0, 1, 0, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_message, __pyx_k_message, sizeof(__pyx_k_message), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_msgnum, __pyx_k_msgnum, sizeof(__pyx_k_msgnum), 0, 0, 1, 1},
  {&__pyx_n_s_multiWrite, __pyx_k_multiWrite, sizeof(__pyx_k_multiWrite), 0, 0, 1, 1},
  {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_nb, __pyx_k_nb, sizeof(__pyx_k_nb), 0, 0, 1, 1},
  {&__pyx_n_s_openByDescription, __pyx_k_openByDescription, sizeof(__pyx_k_openByDescription), 0, 0, 1, 1},
  {&__pyx_n_s_openByLocation, __pyx_k_openByLocation, sizeof(__pyx_k_openByLocation), 0, 0, 1, 1},
  {&__pyx_n_s_openBySerial, __pyx_k_openBySerial, sizeof(__pyx_k_openBySerial), 0, 0, 1, 1},
  {&__pyx_n_s_portNum, __pyx_k_portNum, sizeof(__pyx_k_portNum), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_readSize, __pyx_k_readSize, sizeof(__pyx_k_readSize), 0, 0, 1, 1},
  {&__pyx_n_s_read_timeout, __pyx_k_read_timeout, sizeof(__pyx_k_read_timeout), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_req, __pyx_k_req, sizeof(__pyx_k_req), 0, 0, 1, 1},
  {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_kp_s_self__handle_cannot_be_converted, __pyx_k_self__handle_cannot_be_converted, sizeof(__pyx_k_self__handle_cannot_be_converted), 0, 0, 1, 0},
  {&__pyx_n_s_serial, __pyx_k_serial, sizeof(__pyx_k_serial), 0, 0, 1, 1},
  {&__pyx_n_u_serial, __pyx_k_serial, sizeof(__pyx_k_serial), 0, 1, 0, 1},
  {&__pyx_n_s_setClock, __pyx_k_setClock, sizeof(__pyx_k_setClock), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_singleWrite, __pyx_k_singleWrite, sizeof(__pyx_k_singleWrite), 0, 0, 1, 1},
  {&__pyx_n_s_ssoMap, __pyx_k_ssoMap, sizeof(__pyx_k_ssoMap), 0, 0, 1, 1},
  {&__pyx_n_s_ssoStrength, __pyx_k_ssoStrength, sizeof(__pyx_k_ssoStrength), 0, 0, 1, 1},
  {&__pyx_n_s_status, __pyx_k_status, sizeof(__pyx_k_status), 0, 0, 1, 1},
  {&__pyx_n_s_str, __pyx_k_str, sizeof(__pyx_k_str), 0, 0, 1, 1},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_t, __pyx_k_t, sizeof(__pyx_k_t), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_kp_u_the_data_argument_must_be_of_typ, __pyx_k_the_data_argument_must_be_of_typ, sizeof(__pyx_k_the_data_argument_must_be_of_typ), 0, 1, 0, 0},
  {&__pyx_kp_u_the_multiWrite_argument_must_be, __pyx_k_the_multiWrite_argument_must_be, sizeof(__pyx_k_the_multiWrite_argument_must_be), 0, 1, 0, 0},
  {&__pyx_kp_u_the_singleWrite_argument_must_be, __pyx_k_the_singleWrite_argument_must_be, sizeof(__pyx_k_the_singleWrite_argument_must_be), 0, 1, 0, 0},
  {&__pyx_n_s_trigger, __pyx_k_trigger, sizeof(__pyx_k_trigger), 0, 0, 1, 1},
  {&__pyx_n_u_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 1, 0, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_kp_u_utf_8, __pyx_k_utf_8, sizeof(__pyx_k_utf_8), 0, 1, 0, 0},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_vendorCmdSet, __pyx_k_vendorCmdSet, sizeof(__pyx_k_vendorCmdSet), 0, 0, 1, 1},
  {&__pyx_n_s_weakref, __pyx_k_weakref, sizeof(__pyx_k_weakref), 0, 0, 1, 1},
  {&__pyx_n_s_weakref_finalize, __pyx_k_weakref_finalize, sizeof(__pyx_k_weakref_finalize), 0, 0, 1, 1},
  {&__pyx_n_s_write_timeout, __pyx_k_write_timeout, sizeof(__pyx_k_write_timeout), 0, 0, 1, 1},
  {&__pyx_n_s_xrange, __pyx_k_xrange, sizeof(__pyx_k_xrange), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 72, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 231, __pyx_L1_error)
  #if PY_MAJOR_VERSION >= 3
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_xrange) __PYX_ERR(0, 352, __pyx_L1_error)
  #else
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_xrange); if (!__pyx_builtin_xrange) __PYX_ERR(0, 352, __pyx_L1_error)
  #endif
  __pyx_builtin_round = __Pyx_GetBuiltinName(__pyx_n_s_round); if (!__pyx_builtin_round) __PYX_ERR(0, 488, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(2, 109, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_self__handle_cannot_be_converted); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":4
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_self__handle_cannot_be_converted); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "ft4222/ft4222.pyx":326
 *             data = bytes([data])
 *         elif not isinstance(data, (bytes, bytearray)):
 *             raise TypeError("the data argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         cdef:
 *             uint16 bytesSent
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_the_data_argument_must_be_of_typ); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "ft4222/ft4222.pyx":800
 *             data = bytes([singleWrite])
 *         elif not isinstance(singleWrite, (bytes, bytearray)):
 *             raise TypeError("the singleWrite argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         if isinstance(multiWrite, int):
 *             data = bytes([multiWrite])
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_u_the_singleWrite_argument_must_be); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "ft4222/ft4222.pyx":804
 *             data = bytes([multiWrite])
 *         elif not isinstance(multiWrite, (bytes, bytearray)):
 *             raise TypeError("the multiWrite argument must be of type 'int', 'bytes' or 'bytearray'")             # <<<<<<<<<<<<<<
 *         write = singleWrite + multiWrite
 *         cdef:
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_u_the_multiWrite_argument_must_be); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_self__handle_cannot_be_converted); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "(tree fragment)":4
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._handle cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_self__handle_cannot_be_converted); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "ft4222/ft4222.pyx":60
 * class FT2XXDeviceError(Exception):
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         self.message = __ftd2xx_msgs[msgnum]
 * 
 */
  __pyx_tuple__12 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_msgnum); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);
  __pyx_codeobj__13 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__12, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_init, 60, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__13)) __PYX_ERR(0, 60, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":63
 *         self.message = __ftd2xx_msgs[msgnum]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);
  __pyx_codeobj__15 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__14, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_str, 63, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__15)) __PYX_ERR(0, 63, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":68
 * class FT4222DeviceError(FT2XXDeviceError):
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = __ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 */
  __pyx_tuple__16 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_msgnum); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);
  __pyx_codeobj__17 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__16, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_init, 68, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__17)) __PYX_ERR(0, 68, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":74
 *             super(FT4222DeviceError, self).__init__(msgnum)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);
  __pyx_codeobj__19 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__18, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_str, 74, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__19)) __PYX_ERR(0, 74, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":92
 *     CLK_80 = 3
 * 
 * def createDeviceInfoList():             # <<<<<<<<<<<<<<
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 */
  __pyx_tuple__20 = PyTuple_Pack(2, __pyx_n_s_nb, __pyx_n_s_status); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);
  __pyx_codeobj__21 = (PyObject*)__Pyx_PyCode_New(0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__20, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_createDeviceInfoList, 92, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__21)) __PYX_ERR(0, 92, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":100
 *     raise FT2XXDeviceError, status
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):             # <<<<<<<<<<<<<<
 *     """Get an entry from the internal device info list. Set update to
 *     False to avoid a slow call to createDeviceInfoList."""
 */
  __pyx_tuple__22 = PyTuple_Pack(10, __pyx_n_s_devnum, __pyx_n_s_update, __pyx_n_s_f, __pyx_n_s_t, __pyx_n_s_i, __pyx_n_s_l, __pyx_n_s_h, __pyx_n_s_n, __pyx_n_s_d, __pyx_n_s_status); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);
  __pyx_codeobj__23 = (PyObject*)__Pyx_PyCode_New(2, 0, 10, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__22, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_getDeviceInfoDetail, 100, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__23)) __PYX_ERR(0, 100, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":117
 *     raise FT2XXDeviceError, status
 * 
 * def openBySerial(serial):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by serial number"""
 *     cdef FT_HANDLE handle
 */
  __pyx_tuple__24 = PyTuple_Pack(4, __pyx_n_s_serial, __pyx_n_s_handle, __pyx_n_s_cserial, __pyx_n_s_status); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_codeobj__25 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__24, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_openBySerial, 117, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__25)) __PYX_ERR(0, 117, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":126
 *     raise FT2XXDeviceError, status
 * 
 * def openByDescription(desc):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by description
 * 
 */
  __pyx_tuple__26 = PyTuple_Pack(4, __pyx_n_s_desc, __pyx_n_s_handle, __pyx_n_s_cdesc, __pyx_n_s_status); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);
  __pyx_codeobj__27 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__26, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_openByDescription, 126, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__27)) __PYX_ERR(0, 126, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":149
 *     raise FT2XXDeviceError, status
 * 
 * def openByLocation(locId):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by location
 * 
 */
  __pyx_tuple__28 = PyTuple_Pack(3, __pyx_n_s_locId, __pyx_n_s_handle, __pyx_n_s_status); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_codeobj__29 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ft4222_ft4222_pyx, __pyx_n_s_openByLocation, 149, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__29)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_24000000_0 = PyFloat_FromDouble(24000000.0); if (unlikely(!__pyx_float_24000000_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_60 = PyInt_FromLong(60); if (unlikely(!__pyx_int_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_82 = PyInt_FromLong(82); if (unlikely(!__pyx_int_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_100 = PyInt_FromLong(100); if (unlikely(!__pyx_int_100)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_8000 = PyInt_FromLong(8000); if (unlikely(!__pyx_int_8000)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16908288 = PyInt_FromLong(16908288L); if (unlikely(!__pyx_int_16908288)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16973824 = PyInt_FromLong(16973824L); if (unlikely(!__pyx_int_16973824)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1109524736 = PyInt_FromLong(1109524736L); if (unlikely(!__pyx_int_1109524736)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1109524992 = PyInt_FromLong(1109524992L); if (unlikely(!__pyx_int_1109524992)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1109525248 = PyInt_FromLong(1109525248L); if (unlikely(!__pyx_int_1109525248)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_6ft4222_6ft4222__closer = &__pyx_vtable_6ft4222_6ft4222__closer;
  __pyx_vtable_6ft4222_6ft4222__closer.wrap = (PyObject *(*)(FT_HANDLE))__pyx_f_6ft4222_6ft4222_7_closer_wrap;
  if (PyType_Ready(&__pyx_type_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 169, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ft4222_6ft4222__closer.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ft4222_6ft4222__closer.tp_dictoffset && __pyx_type_6ft4222_6ft4222__closer.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ft4222_6ft4222__closer.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ft4222_6ft4222__closer.tp_dict, __pyx_vtabptr_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 169, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_closer, (PyObject *)&__pyx_type_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 169, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ft4222_6ft4222__closer) < 0) __PYX_ERR(0, 169, __pyx_L1_error)
  __pyx_ptype_6ft4222_6ft4222__closer = &__pyx_type_6ft4222_6ft4222__closer;
  __pyx_vtabptr_6ft4222_6ft4222_FT4222 = &__pyx_vtable_6ft4222_6ft4222_FT4222;
  __pyx_vtable_6ft4222_6ft4222_FT4222._get_version = (PyObject *(*)(struct __pyx_obj_6ft4222_6ft4222_FT4222 *))__pyx_f_6ft4222_6ft4222_6FT4222__get_version;
  if (PyType_Ready(&__pyx_type_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 187, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ft4222_6ft4222_FT4222.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ft4222_6ft4222_FT4222.tp_dictoffset && __pyx_type_6ft4222_6ft4222_FT4222.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ft4222_6ft4222_FT4222.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ft4222_6ft4222_FT4222.tp_dict, __pyx_vtabptr_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 187, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_FT4222, (PyObject *)&__pyx_type_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 187, __pyx_L1_error)
  if (__pyx_type_6ft4222_6ft4222_FT4222.tp_weaklistoffset == 0) __pyx_type_6ft4222_6ft4222_FT4222.tp_weaklistoffset = offsetof(struct __pyx_obj_6ft4222_6ft4222_FT4222, __weakref__);
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ft4222_6ft4222_FT4222) < 0) __PYX_ERR(0, 187, __pyx_L1_error)
  __pyx_ptype_6ft4222_6ft4222_FT4222 = &__pyx_type_6ft4222_6ft4222_FT4222;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("array"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_5array_array = __Pyx_ImportType(__pyx_t_1, "array", "array", sizeof(arrayobject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_5array_array) __PYX_ERR(2, 58, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initft4222(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initft4222(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_ft4222(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_ft4222(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_ft4222(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'ft4222' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_ft4222(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("ft4222", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ft4222__ft4222) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ft4222.ft4222")) {
      if (unlikely(PyDict_SetItemString(modules, "ft4222.ft4222", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ft4222/ft4222.pyx":15
 * from cpython.array cimport array, resize
 * from libc.stdio cimport printf
 * from enum import IntEnum             # <<<<<<<<<<<<<<
 * from .GPIO import Dir, Trigger
 * from .I2CMaster import ControllerStatus
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_IntEnum);
  __Pyx_GIVEREF(__pyx_n_s_IntEnum);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_IntEnum);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_enum, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_IntEnum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_IntEnum, __pyx_t_1) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":16
 * from libc.stdio cimport printf
 * from enum import IntEnum
 * from .GPIO import Dir, Trigger             # <<<<<<<<<<<<<<
 * from .I2CMaster import ControllerStatus
 * from weakref import finalize as weakref_finalize
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_Dir);
  __Pyx_GIVEREF(__pyx_n_s_Dir);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_Dir);
  __Pyx_INCREF(__pyx_n_s_Trigger);
  __Pyx_GIVEREF(__pyx_n_s_Trigger);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_Trigger);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_GPIO, __pyx_t_2, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_Dir); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Dir, __pyx_t_2) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_Trigger); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Trigger, __pyx_t_2) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":17
 * from enum import IntEnum
 * from .GPIO import Dir, Trigger
 * from .I2CMaster import ControllerStatus             # <<<<<<<<<<<<<<
 * from weakref import finalize as weakref_finalize
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ControllerStatus);
  __Pyx_GIVEREF(__pyx_n_s_ControllerStatus);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_ControllerStatus);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_I2CMaster, __pyx_t_1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ControllerStatus); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ControllerStatus, __pyx_t_1) < 0) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":18
 * from .GPIO import Dir, Trigger
 * from .I2CMaster import ControllerStatus
 * from weakref import finalize as weakref_finalize             # <<<<<<<<<<<<<<
 * 
 * IF UNAME_SYSNAME == "Windows":
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_finalize);
  __Pyx_GIVEREF(__pyx_n_s_finalize);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_finalize);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_weakref, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_finalize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_weakref_finalize, __pyx_t_2) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":28
 * 
 * 
 * __ftd2xx_msgs = ['OK', 'INVALID_HANDLE', 'DEVICE_NOT_FOUND', 'DEVICE_NOT_OPENED',             # <<<<<<<<<<<<<<
 *                  'IO_ERROR', 'INSUFFICIENT_RESOURCES', 'INVALID_PARAMETER',
 *                  'INVALID_BAUD_RATE', 'DEVICE_NOT_OPENED_FOR_ERASE',
 */
  __pyx_t_1 = PyList_New(20); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_u_OK);
  __Pyx_GIVEREF(__pyx_n_u_OK);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_u_OK);
  __Pyx_INCREF(__pyx_n_u_INVALID_HANDLE);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_HANDLE);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_u_INVALID_HANDLE);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_FOUND);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_FOUND);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_u_DEVICE_NOT_FOUND);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_OPENED);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_OPENED);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_u_DEVICE_NOT_OPENED);
  __Pyx_INCREF(__pyx_n_u_IO_ERROR);
  __Pyx_GIVEREF(__pyx_n_u_IO_ERROR);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_u_IO_ERROR);
  __Pyx_INCREF(__pyx_n_u_INSUFFICIENT_RESOURCES);
  __Pyx_GIVEREF(__pyx_n_u_INSUFFICIENT_RESOURCES);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_u_INSUFFICIENT_RESOURCES);
  __Pyx_INCREF(__pyx_n_u_INVALID_PARAMETER);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_PARAMETER);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_n_u_INVALID_PARAMETER);
  __Pyx_INCREF(__pyx_n_u_INVALID_BAUD_RATE);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_BAUD_RATE);
  PyList_SET_ITEM(__pyx_t_1, 7, __pyx_n_u_INVALID_BAUD_RATE);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE);
  PyList_SET_ITEM(__pyx_t_1, 8, __pyx_n_u_DEVICE_NOT_OPENED_FOR_ERASE);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE);
  PyList_SET_ITEM(__pyx_t_1, 9, __pyx_n_u_DEVICE_NOT_OPENED_FOR_WRITE);
  __Pyx_INCREF(__pyx_n_u_FAILED_TO_WRITE_DEVICE0);
  __Pyx_GIVEREF(__pyx_n_u_FAILED_TO_WRITE_DEVICE0);
  PyList_SET_ITEM(__pyx_t_1, 10, __pyx_n_u_FAILED_TO_WRITE_DEVICE0);
  __Pyx_INCREF(__pyx_n_u_EEPROM_READ_FAILED);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_READ_FAILED);
  PyList_SET_ITEM(__pyx_t_1, 11, __pyx_n_u_EEPROM_READ_FAILED);
  __Pyx_INCREF(__pyx_n_u_EEPROM_WRITE_FAILED);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_WRITE_FAILED);
  PyList_SET_ITEM(__pyx_t_1, 12, __pyx_n_u_EEPROM_WRITE_FAILED);
  __Pyx_INCREF(__pyx_n_u_EEPROM_ERASE_FAILED);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_ERASE_FAILED);
  PyList_SET_ITEM(__pyx_t_1, 13, __pyx_n_u_EEPROM_ERASE_FAILED);
  __Pyx_INCREF(__pyx_n_u_EEPROM_NOT_PRESENT);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_NOT_PRESENT);
  PyList_SET_ITEM(__pyx_t_1, 14, __pyx_n_u_EEPROM_NOT_PRESENT);
  __Pyx_INCREF(__pyx_n_u_EEPROM_NOT_PROGRAMMED);
  __Pyx_GIVEREF(__pyx_n_u_EEPROM_NOT_PROGRAMMED);
  PyList_SET_ITEM(__pyx_t_1, 15, __pyx_n_u_EEPROM_NOT_PROGRAMMED);
  __Pyx_INCREF(__pyx_n_u_INVALID_ARGS);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_ARGS);
  PyList_SET_ITEM(__pyx_t_1, 16, __pyx_n_u_INVALID_ARGS);
  __Pyx_INCREF(__pyx_n_u_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_NOT_SUPPORTED);
  PyList_SET_ITEM(__pyx_t_1, 17, __pyx_n_u_NOT_SUPPORTED);
  __Pyx_INCREF(__pyx_n_u_OTHER_ERROR);
  __Pyx_GIVEREF(__pyx_n_u_OTHER_ERROR);
  PyList_SET_ITEM(__pyx_t_1, 18, __pyx_n_u_OTHER_ERROR);
  __Pyx_INCREF(__pyx_n_u_DEVICE_LIST_NOT_READY);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_LIST_NOT_READY);
  PyList_SET_ITEM(__pyx_t_1, 19, __pyx_n_u_DEVICE_LIST_NOT_READY);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ftd2xx_msgs, __pyx_t_1) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":36
 *                  'NOT_SUPPORTED', 'OTHER_ERROR', 'DEVICE_LIST_NOT_READY']
 * 
 * __ftd4222_msgs = ['DEVICE_NOT_SUPPORTED', 'CLK_NOT_SUPPORTED','VENDER_CMD_NOT_SUPPORTED',             # <<<<<<<<<<<<<<
 *                   'IS_NOT_SPI_MODE', 'IS_NOT_I2C_MODE', 'IS_NOT_SPI_SINGLE_MODE',
 *                   'IS_NOT_SPI_MULTI_MODE', 'WRONG_I2C_ADDR', 'INVAILD_FUNCTION',
 */
  __pyx_t_1 = PyList_New(23); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_u_DEVICE_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_DEVICE_NOT_SUPPORTED);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_u_DEVICE_NOT_SUPPORTED);
  __Pyx_INCREF(__pyx_n_u_CLK_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_CLK_NOT_SUPPORTED);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_u_CLK_NOT_SUPPORTED);
  __Pyx_INCREF(__pyx_n_u_VENDER_CMD_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_VENDER_CMD_NOT_SUPPORTED);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_u_VENDER_CMD_NOT_SUPPORTED);
  __Pyx_INCREF(__pyx_n_u_IS_NOT_SPI_MODE);
  __Pyx_GIVEREF(__pyx_n_u_IS_NOT_SPI_MODE);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_u_IS_NOT_SPI_MODE);
  __Pyx_INCREF(__pyx_n_u_IS_NOT_I2C_MODE);
  __Pyx_GIVEREF(__pyx_n_u_IS_NOT_I2C_MODE);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_u_IS_NOT_I2C_MODE);
  __Pyx_INCREF(__pyx_n_u_IS_NOT_SPI_SINGLE_MODE);
  __Pyx_GIVEREF(__pyx_n_u_IS_NOT_SPI_SINGLE_MODE);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_u_IS_NOT_SPI_SINGLE_MODE);
  __Pyx_INCREF(__pyx_n_u_IS_NOT_SPI_MULTI_MODE);
  __Pyx_GIVEREF(__pyx_n_u_IS_NOT_SPI_MULTI_MODE);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_n_u_IS_NOT_SPI_MULTI_MODE);
  __Pyx_INCREF(__pyx_n_u_WRONG_I2C_ADDR);
  __Pyx_GIVEREF(__pyx_n_u_WRONG_I2C_ADDR);
  PyList_SET_ITEM(__pyx_t_1, 7, __pyx_n_u_WRONG_I2C_ADDR);
  __Pyx_INCREF(__pyx_n_u_INVAILD_FUNCTION);
  __Pyx_GIVEREF(__pyx_n_u_INVAILD_FUNCTION);
  PyList_SET_ITEM(__pyx_t_1, 8, __pyx_n_u_INVAILD_FUNCTION);
  __Pyx_INCREF(__pyx_n_u_INVALID_POINTER);
  __Pyx_GIVEREF(__pyx_n_u_INVALID_POINTER);
  PyList_SET_ITEM(__pyx_t_1, 9, __pyx_n_u_INVALID_POINTER);
  __Pyx_INCREF(__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE);
  __Pyx_GIVEREF(__pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE);
  PyList_SET_ITEM(__pyx_t_1, 10, __pyx_n_u_EXCEEDED_MAX_TRANSFER_SIZE);
  __Pyx_INCREF(__pyx_n_u_FAILED_TO_READ_DEVICE);
  __Pyx_GIVEREF(__pyx_n_u_FAILED_TO_READ_DEVICE);
  PyList_SET_ITEM(__pyx_t_1, 11, __pyx_n_u_FAILED_TO_READ_DEVICE);
  __Pyx_INCREF(__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE);
  __Pyx_GIVEREF(__pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE);
  PyList_SET_ITEM(__pyx_t_1, 12, __pyx_n_u_I2C_NOT_SUPPORTED_IN_THIS_MODE);
  __Pyx_INCREF(__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE);
  PyList_SET_ITEM(__pyx_t_1, 13, __pyx_n_u_GPIO_NOT_SUPPORTED_IN_THIS_MODE);
  __Pyx_INCREF(__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM);
  PyList_SET_ITEM(__pyx_t_1, 14, __pyx_n_u_GPIO_EXCEEDED_MAX_PORTNUM);
  __Pyx_INCREF(__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_WRITE_NOT_SUPPORTED);
  PyList_SET_ITEM(__pyx_t_1, 15, __pyx_n_u_GPIO_WRITE_NOT_SUPPORTED);
  __Pyx_INCREF(__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE);
  PyList_SET_ITEM(__pyx_t_1, 16, __pyx_n_u_GPIO_PULLUP_INVALID_IN_INPUTMODE);
  __Pyx_INCREF(__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO);
  PyList_SET_ITEM(__pyx_t_1, 17, __pyx_n_u_GPIO_PULLDOWN_INVALID_IN_INPUTMO);
  __Pyx_INCREF(__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT);
  PyList_SET_ITEM(__pyx_t_1, 18, __pyx_n_u_GPIO_OPENDRAIN_INVALID_IN_OUTPUT);
  __Pyx_INCREF(__pyx_n_u_INTERRUPT_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_INTERRUPT_NOT_SUPPORTED);
  PyList_SET_ITEM(__pyx_t_1, 19, __pyx_n_u_INTERRUPT_NOT_SUPPORTED);
  __Pyx_INCREF(__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_GPIO_INPUT_NOT_SUPPORTED);
  PyList_SET_ITEM(__pyx_t_1, 20, __pyx_n_u_GPIO_INPUT_NOT_SUPPORTED);
  __Pyx_INCREF(__pyx_n_u_EVENT_NOT_SUPPORTED);
  __Pyx_GIVEREF(__pyx_n_u_EVENT_NOT_SUPPORTED);
  PyList_SET_ITEM(__pyx_t_1, 21, __pyx_n_u_EVENT_NOT_SUPPORTED);
  __Pyx_INCREF(__pyx_n_u_FUN_NOT_SUPPORT);
  __Pyx_GIVEREF(__pyx_n_u_FUN_NOT_SUPPORT);
  PyList_SET_ITEM(__pyx_t_1, 22, __pyx_n_u_FUN_NOT_SUPPORT);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ftd4222_msgs, __pyx_t_1) < 0) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":52
 * # 0x01020000; Revision C chips require version 1.3 or later of LibFT4222, indicated
 * # by dllVersion being greater than 0x01030000.
 * __chip_rev_map = { 0x42220100: "Rev. A", 0x42220200: "Rev. B", 0x42220300: "Rev. C" }             # <<<<<<<<<<<<<<
 * __chip_rev_min_lib = { 0x42220100: 0, 0x42220200: 0x01020000, 0x42220300: 0x01030000 }
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_1109524736, __pyx_kp_u_Rev_A) < 0) __PYX_ERR(0, 52, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_1109524992, __pyx_kp_u_Rev_B) < 0) __PYX_ERR(0, 52, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_1109525248, __pyx_kp_u_Rev_C) < 0) __PYX_ERR(0, 52, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chip_rev_map, __pyx_t_1) < 0) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":53
 * # by dllVersion being greater than 0x01030000.
 * __chip_rev_map = { 0x42220100: "Rev. A", 0x42220200: "Rev. B", 0x42220300: "Rev. C" }
 * __chip_rev_min_lib = { 0x42220100: 0, 0x42220200: 0x01020000, 0x42220300: 0x01030000 }             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_1109524736, __pyx_int_0) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_1109524992, __pyx_int_16908288) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_int_1109525248, __pyx_int_16973824) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chip_rev_min_lib, __pyx_t_1) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":58
 * DEF MAX_DESCRIPTION_SIZE = 256
 * 
 * class FT2XXDeviceError(Exception):             # <<<<<<<<<<<<<<
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_FT2XXDeviceError, __pyx_n_s_FT2XXDeviceError, (PyObject *) NULL, __pyx_n_s_ft4222_ft4222, __pyx_kp_s_Exception_class_for_status_messa); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ft4222/ft4222.pyx":60
 * class FT2XXDeviceError(Exception):
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         self.message = __ftd2xx_msgs[msgnum]
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_16FT2XXDeviceError_1__init__, 0, __pyx_n_s_FT2XXDeviceError___init, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__13)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_init, __pyx_t_4) < 0) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ft4222/ft4222.pyx":63
 *         self.message = __ftd2xx_msgs[msgnum]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_16FT2XXDeviceError_3__str__, 0, __pyx_n_s_FT2XXDeviceError___str, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__15)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_str, __pyx_t_4) < 0) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ft4222/ft4222.pyx":58
 * DEF MAX_DESCRIPTION_SIZE = 256
 * 
 * class FT2XXDeviceError(Exception):             # <<<<<<<<<<<<<<
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_FT2XXDeviceError, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FT2XXDeviceError, __pyx_t_4) < 0) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":66
 *         return self.message
 * 
 * class FT4222DeviceError(FT2XXDeviceError):             # <<<<<<<<<<<<<<
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FT2XXDeviceError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_t_2, __pyx_n_s_FT4222DeviceError, __pyx_n_s_FT4222DeviceError, (PyObject *) NULL, __pyx_n_s_ft4222_ft4222, __pyx_kp_s_Exception_class_for_status_messa); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ft4222/ft4222.pyx":68
 * class FT4222DeviceError(FT2XXDeviceError):
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):             # <<<<<<<<<<<<<<
 *         if msgnum >= FT4222_DEVICE_NOT_SUPPORTED:
 *             self.message = __ftd4222_msgs[msgnum - FT4222_DEVICE_NOT_SUPPORTED]
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_17FT4222DeviceError_1__init__, 0, __pyx_n_s_FT4222DeviceError___init, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__17)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_init, __pyx_t_4) < 0) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ft4222/ft4222.pyx":74
 *             super(FT4222DeviceError, self).__init__(msgnum)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_6ft4222_6ft4222_17FT4222DeviceError_3__str__, 0, __pyx_n_s_FT4222DeviceError___str, NULL, __pyx_n_s_ft4222_ft4222, __pyx_d, ((PyObject *)__pyx_codeobj__19)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_str, __pyx_t_4) < 0) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "ft4222/ft4222.pyx":66
 *         return self.message
 * 
 * class FT4222DeviceError(FT2XXDeviceError):             # <<<<<<<<<<<<<<
 *     """Exception class for status messages"""
 *     def __init__(self, msgnum):
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_FT4222DeviceError, __pyx_t_2, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FT4222DeviceError, __pyx_t_4) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ft4222/ft4222.pyx":77
 *         return self.message
 * 
 * class SysClock(IntEnum):             # <<<<<<<<<<<<<<
 *     """Chip system clock
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_IntEnum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_SysClock, __pyx_n_s_SysClock, (PyObject *) NULL, __pyx_n_s_ft4222_ft4222, __pyx_kp_s_Chip_system_clock_Attributes_CLK); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ft4222/ft4222.pyx":87
 * 
 *     """
 *     CLK_60 = 0             # <<<<<<<<<<<<<<
 *     CLK_24 = 1
 *     CLK_48 = 2
 */
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CLK_60, __pyx_int_0) < 0) __PYX_ERR(0, 87, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":88
 *     """
 *     CLK_60 = 0
 *     CLK_24 = 1             # <<<<<<<<<<<<<<
 *     CLK_48 = 2
 *     CLK_80 = 3
 */
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CLK_24, __pyx_int_1) < 0) __PYX_ERR(0, 88, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":89
 *     CLK_60 = 0
 *     CLK_24 = 1
 *     CLK_48 = 2             # <<<<<<<<<<<<<<
 *     CLK_80 = 3
 * 
 */
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CLK_48, __pyx_int_2) < 0) __PYX_ERR(0, 89, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":90
 *     CLK_24 = 1
 *     CLK_48 = 2
 *     CLK_80 = 3             # <<<<<<<<<<<<<<
 * 
 * def createDeviceInfoList():
 */
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_CLK_80, __pyx_int_3) < 0) __PYX_ERR(0, 90, __pyx_L1_error)

  /* "ft4222/ft4222.pyx":77
 *         return self.message
 * 
 * class SysClock(IntEnum):             # <<<<<<<<<<<<<<
 *     """Chip system clock
 * 
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SysClock, __pyx_t_1, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SysClock, __pyx_t_4) < 0) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":92
 *     CLK_80 = 3
 * 
 * def createDeviceInfoList():             # <<<<<<<<<<<<<<
 *     """Create the internal device info list and return number of entries"""
 *     cdef DWORD nb
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_6ft4222_6ft4222_1createDeviceInfoList, NULL, __pyx_n_s_ft4222_ft4222); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_createDeviceInfoList, __pyx_t_1) < 0) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":100
 *     raise FT2XXDeviceError, status
 * 
 * def getDeviceInfoDetail(devnum=0, update=True):             # <<<<<<<<<<<<<<
 *     """Get an entry from the internal device info list. Set update to
 *     False to avoid a slow call to createDeviceInfoList."""
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_6ft4222_6ft4222_3getDeviceInfoDetail, NULL, __pyx_n_s_ft4222_ft4222); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_getDeviceInfoDetail, __pyx_t_1) < 0) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":117
 *     raise FT2XXDeviceError, status
 * 
 * def openBySerial(serial):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by serial number"""
 *     cdef FT_HANDLE handle
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_6ft4222_6ft4222_5openBySerial, NULL, __pyx_n_s_ft4222_ft4222); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_openBySerial, __pyx_t_1) < 0) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":126
 *     raise FT2XXDeviceError, status
 * 
 * def openByDescription(desc):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by description
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_6ft4222_6ft4222_7openByDescription, NULL, __pyx_n_s_ft4222_ft4222); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_openByDescription, __pyx_t_1) < 0) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":149
 *     raise FT2XXDeviceError, status
 * 
 * def openByLocation(locId):             # <<<<<<<<<<<<<<
 *     """Open a handle to a usb device by location
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_6ft4222_6ft4222_9openByLocation, NULL, __pyx_n_s_ft4222_ft4222); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_openByLocation, __pyx_t_1) < 0) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":335
 * 
 * 
 *     def gpio_Init(self, *args, gpio0=Dir.INPUT, gpio1=Dir.INPUT, gpio2=Dir.INPUT, gpio3=Dir.INPUT):             # <<<<<<<<<<<<<<
 *         """Initialize the GPIO interface.
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Dir); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_k__4 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_Dir); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_k__5 = __pyx_t_1;
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Dir); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_k__6 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_Dir); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_INPUT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_k__7 = __pyx_t_1;
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ft4222/ft4222.pyx":1
 * #  _____ _____ _____             # <<<<<<<<<<<<<<
 * # |_    |   __| __  |
 * # |_| | |__   |    -|
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cpython/array.pxd":161
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 * cdef inline void zero(array self):             # <<<<<<<<<<<<<<
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init ft4222.ft4222", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ft4222.ft4222");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* IsLittleEndian */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void)
{
  union {
    uint32_t u32;
    uint8_t u8[4];
  } S;
  S.u32 = 0x01020304;
  return S.u8[0] == 4;
}

/* BufferFormatCheck */
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t <= '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case '?': return "'bool'";
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case '?': case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number, ndim;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    ndim = ctx->head->field->type->ndim;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
        CYTHON_FALLTHROUGH;
      case '?': case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if ((ctx->enc_type == *ts) && (got_Z == ctx->is_complex) &&
            (ctx->enc_packmode == ctx->new_packmode) && (!ctx->is_valid_array)) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
        CYTHON_FALLTHROUGH;
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}

/* BufferGetAndValidate */
  static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (unlikely(info->buf == NULL)) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}
static void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static int __Pyx__GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  buf->buf = NULL;
  if (unlikely(__Pyx_GetBuffer(obj, buf, flags) == -1)) {
    __Pyx_ZeroBuffer(buf);
    return -1;
  }
  if (unlikely(buf->ndim != nd)) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if (unlikely((size_t)buf->itemsize != dtype->size)) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_SafeReleaseBuffer(buf);
  return -1;
}

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* PyFloatBinop */
  #if !CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(operand) if (unlikely(zerodivision_check && ((operand) == 0))) {\
    PyErr_SetString(PyExc_ZeroDivisionError, "float division by zero");\
    return NULL;\
}
static PyObject* __Pyx_PyFloat_TrueDivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double a = floatval;
    double b, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op2))) {
        b = PyFloat_AS_DOUBLE(op2);
        __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        b = (double) PyInt_AS_LONG(op2);
        __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
    } else
    #endif
    if (likely(PyLong_CheckExact(op2))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op2)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op2);
        switch (size) {
            case  0: __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(0) break;
            case -1: b = -(double) digits[0]; break;
            case  1: b = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    b = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            b = PyLong_AsDouble(op2);
            if (unlikely(b == -1.0 && PyErr_Occurred())) return NULL;
            __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
        }
    } else {
        return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
    }
        __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* PyObject_GenericGetAttrNoDict */
    #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
    #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
    static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
    static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
    static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
    #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
    static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CalculateMetaclass */
    static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchCommonType */
    static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
    #include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
    static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* Py3ClassCreate */
    static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
    #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (__Pyx_TypeCheck(obj, __pyx_ptype_7cpython_5array_array)) return __pyx_pw_7cpython_5array_5array_1__getbuffer__(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
    if ((0)) {}
        else if (__Pyx_TypeCheck(obj, __pyx_ptype_7cpython_5array_array)) __pyx_pw_7cpython_5array_5array_3__releasebuffer__(obj, view);
    view->obj = NULL;
    Py_DECREF(obj);
}
#endif


    /* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT4222_STATUS(FT4222_STATUS value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_STATUS neg_one = (FT4222_STATUS) -1, const_zero = (FT4222_STATUS) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(FT4222_STATUS) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(FT4222_STATUS) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT4222_STATUS) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(FT4222_STATUS) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT4222_STATUS) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(FT4222_STATUS),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_DWORD(DWORD value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const DWORD neg_one = (DWORD) -1, const_zero = (DWORD) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(DWORD) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(DWORD) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(DWORD) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(DWORD) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(DWORD) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(DWORD),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ULONG(ULONG value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const ULONG neg_one = (ULONG) -1, const_zero = (ULONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(ULONG) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(ULONG) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ULONG) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(ULONG) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ULONG) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(ULONG),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE DWORD __Pyx_PyInt_As_DWORD(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const DWORD neg_one = (DWORD) -1, const_zero = (DWORD) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(DWORD) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(DWORD, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (DWORD) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (DWORD) 0;
                case  1: __PYX_VERIFY_RETURN_INT(DWORD, digit, digits[0])
                case 2:
                    if (8 * sizeof(DWORD) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(DWORD) >= 2 * PyLong_SHIFT) {
                            return (DWORD) (((((DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(DWORD) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(DWORD) >= 3 * PyLong_SHIFT) {
                            return (DWORD) (((((((DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(DWORD) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(DWORD) >= 4 * PyLong_SHIFT) {
                            return (DWORD) (((((((((DWORD)digits[3]) << PyLong_SHIFT) | (DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (DWORD) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(DWORD) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(DWORD, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(DWORD) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(DWORD, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (DWORD) 0;
                case -1: __PYX_VERIFY_RETURN_INT(DWORD, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(DWORD,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(DWORD) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(DWORD, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(DWORD) - 1 > 2 * PyLong_SHIFT) {
                            return (DWORD) (((DWORD)-1)*(((((DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(DWORD) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(DWORD) - 1 > 2 * PyLong_SHIFT) {
                            return (DWORD) ((((((DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(DWORD) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(DWORD, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(DWORD) - 1 > 3 * PyLong_SHIFT) {
                            return (DWORD) (((DWORD)-1)*(((((((DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(DWORD) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(DWORD) - 1 > 3 * PyLong_SHIFT) {
                            return (DWORD) ((((((((DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(DWORD) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(DWORD, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(DWORD) - 1 > 4 * PyLong_SHIFT) {
                            return (DWORD) (((DWORD)-1)*(((((((((DWORD)digits[3]) << PyLong_SHIFT) | (DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(DWORD) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(DWORD, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(DWORD) - 1 > 4 * PyLong_SHIFT) {
                            return (DWORD) ((((((((((DWORD)digits[3]) << PyLong_SHIFT) | (DWORD)digits[2]) << PyLong_SHIFT) | (DWORD)digits[1]) << PyLong_SHIFT) | (DWORD)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(DWORD) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(DWORD, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(DWORD) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(DWORD, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            DWORD val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (DWORD) -1;
        }
    } else {
        DWORD val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (DWORD) -1;
        val = __Pyx_PyInt_As_DWORD(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to DWORD");
    return (DWORD) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to DWORD");
    return (DWORD) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE ULONG __Pyx_PyInt_As_ULONG(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const ULONG neg_one = (ULONG) -1, const_zero = (ULONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(ULONG) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(ULONG, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (ULONG) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (ULONG) 0;
                case  1: __PYX_VERIFY_RETURN_INT(ULONG, digit, digits[0])
                case 2:
                    if (8 * sizeof(ULONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ULONG) >= 2 * PyLong_SHIFT) {
                            return (ULONG) (((((ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(ULONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ULONG) >= 3 * PyLong_SHIFT) {
                            return (ULONG) (((((((ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(ULONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ULONG) >= 4 * PyLong_SHIFT) {
                            return (ULONG) (((((((((ULONG)digits[3]) << PyLong_SHIFT) | (ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (ULONG) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(ULONG) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(ULONG, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(ULONG) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(ULONG, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (ULONG) 0;
                case -1: __PYX_VERIFY_RETURN_INT(ULONG, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(ULONG,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(ULONG) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ULONG, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ULONG) - 1 > 2 * PyLong_SHIFT) {
                            return (ULONG) (((ULONG)-1)*(((((ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(ULONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ULONG) - 1 > 2 * PyLong_SHIFT) {
                            return (ULONG) ((((((ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(ULONG) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ULONG, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ULONG) - 1 > 3 * PyLong_SHIFT) {
                            return (ULONG) (((ULONG)-1)*(((((((ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(ULONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ULONG) - 1 > 3 * PyLong_SHIFT) {
                            return (ULONG) ((((((((ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(ULONG) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ULONG, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ULONG) - 1 > 4 * PyLong_SHIFT) {
                            return (ULONG) (((ULONG)-1)*(((((((((ULONG)digits[3]) << PyLong_SHIFT) | (ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(ULONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ULONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ULONG) - 1 > 4 * PyLong_SHIFT) {
                            return (ULONG) ((((((((((ULONG)digits[3]) << PyLong_SHIFT) | (ULONG)digits[2]) << PyLong_SHIFT) | (ULONG)digits[1]) << PyLong_SHIFT) | (ULONG)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(ULONG) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(ULONG, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(ULONG) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(ULONG, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            ULONG val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (ULONG) -1;
        }
    } else {
        ULONG val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (ULONG) -1;
        val = __Pyx_PyInt_As_ULONG(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to ULONG");
    return (ULONG) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to ULONG");
    return (ULONG) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE FT4222_ClockRate __Pyx_PyInt_As_FT4222_ClockRate(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_ClockRate neg_one = (FT4222_ClockRate) -1, const_zero = (FT4222_ClockRate) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(FT4222_ClockRate) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_ClockRate) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_ClockRate) 0;
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, digit, digits[0])
                case 2:
                    if (8 * sizeof(FT4222_ClockRate) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_ClockRate) >= 2 * PyLong_SHIFT) {
                            return (FT4222_ClockRate) (((((FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_ClockRate) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_ClockRate) >= 3 * PyLong_SHIFT) {
                            return (FT4222_ClockRate) (((((((FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_ClockRate) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_ClockRate) >= 4 * PyLong_SHIFT) {
                            return (FT4222_ClockRate) (((((((((FT4222_ClockRate)digits[3]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_ClockRate) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(FT4222_ClockRate) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_ClockRate, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_ClockRate) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_ClockRate, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_ClockRate) 0;
                case -1: __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_ClockRate,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(FT4222_ClockRate) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_ClockRate) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_ClockRate) (((FT4222_ClockRate)-1)*(((((FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(FT4222_ClockRate) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_ClockRate) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_ClockRate) ((((((FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(FT4222_ClockRate) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_ClockRate) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_ClockRate) (((FT4222_ClockRate)-1)*(((((((FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_ClockRate) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_ClockRate) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_ClockRate) ((((((((FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(FT4222_ClockRate) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_ClockRate) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_ClockRate) (((FT4222_ClockRate)-1)*(((((((((FT4222_ClockRate)digits[3]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_ClockRate) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_ClockRate, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_ClockRate) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_ClockRate) ((((((((((FT4222_ClockRate)digits[3]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[2]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[1]) << PyLong_SHIFT) | (FT4222_ClockRate)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(FT4222_ClockRate) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_ClockRate, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_ClockRate) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_ClockRate, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            FT4222_ClockRate val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (FT4222_ClockRate) -1;
        }
    } else {
        FT4222_ClockRate val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_ClockRate) -1;
        val = __Pyx_PyInt_As_FT4222_ClockRate(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_ClockRate");
    return (FT4222_ClockRate) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_ClockRate");
    return (FT4222_ClockRate) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT4222_ClockRate(FT4222_ClockRate value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_ClockRate neg_one = (FT4222_ClockRate) -1, const_zero = (FT4222_ClockRate) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(FT4222_ClockRate) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(FT4222_ClockRate) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT4222_ClockRate) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(FT4222_ClockRate) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT4222_ClockRate) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(FT4222_ClockRate),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE BOOL __Pyx_PyInt_As_BOOL(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const BOOL neg_one = (BOOL) -1, const_zero = (BOOL) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(BOOL) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(BOOL, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (BOOL) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (BOOL) 0;
                case  1: __PYX_VERIFY_RETURN_INT(BOOL, digit, digits[0])
                case 2:
                    if (8 * sizeof(BOOL) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(BOOL) >= 2 * PyLong_SHIFT) {
                            return (BOOL) (((((BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(BOOL) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(BOOL) >= 3 * PyLong_SHIFT) {
                            return (BOOL) (((((((BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(BOOL) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(BOOL) >= 4 * PyLong_SHIFT) {
                            return (BOOL) (((((((((BOOL)digits[3]) << PyLong_SHIFT) | (BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (BOOL) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(BOOL) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(BOOL, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(BOOL) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(BOOL, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (BOOL) 0;
                case -1: __PYX_VERIFY_RETURN_INT(BOOL, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(BOOL,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(BOOL) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(BOOL, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(BOOL) - 1 > 2 * PyLong_SHIFT) {
                            return (BOOL) (((BOOL)-1)*(((((BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(BOOL) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(BOOL) - 1 > 2 * PyLong_SHIFT) {
                            return (BOOL) ((((((BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(BOOL) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(BOOL, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(BOOL) - 1 > 3 * PyLong_SHIFT) {
                            return (BOOL) (((BOOL)-1)*(((((((BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(BOOL) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(BOOL) - 1 > 3 * PyLong_SHIFT) {
                            return (BOOL) ((((((((BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(BOOL) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(BOOL, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(BOOL) - 1 > 4 * PyLong_SHIFT) {
                            return (BOOL) (((BOOL)-1)*(((((((((BOOL)digits[3]) << PyLong_SHIFT) | (BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(BOOL) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(BOOL, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(BOOL) - 1 > 4 * PyLong_SHIFT) {
                            return (BOOL) ((((((((((BOOL)digits[3]) << PyLong_SHIFT) | (BOOL)digits[2]) << PyLong_SHIFT) | (BOOL)digits[1]) << PyLong_SHIFT) | (BOOL)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(BOOL) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(BOOL, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(BOOL) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(BOOL, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            BOOL val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (BOOL) -1;
        }
    } else {
        BOOL val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (BOOL) -1;
        val = __Pyx_PyInt_As_BOOL(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to BOOL");
    return (BOOL) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to BOOL");
    return (BOOL) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE UCHAR __Pyx_PyInt_As_UCHAR(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const UCHAR neg_one = (UCHAR) -1, const_zero = (UCHAR) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(UCHAR) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(UCHAR, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (UCHAR) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (UCHAR) 0;
                case  1: __PYX_VERIFY_RETURN_INT(UCHAR, digit, digits[0])
                case 2:
                    if (8 * sizeof(UCHAR) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(UCHAR) >= 2 * PyLong_SHIFT) {
                            return (UCHAR) (((((UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(UCHAR) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(UCHAR) >= 3 * PyLong_SHIFT) {
                            return (UCHAR) (((((((UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(UCHAR) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(UCHAR) >= 4 * PyLong_SHIFT) {
                            return (UCHAR) (((((((((UCHAR)digits[3]) << PyLong_SHIFT) | (UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (UCHAR) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(UCHAR) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(UCHAR, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(UCHAR) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(UCHAR, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (UCHAR) 0;
                case -1: __PYX_VERIFY_RETURN_INT(UCHAR, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(UCHAR,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(UCHAR) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(UCHAR, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(UCHAR) - 1 > 2 * PyLong_SHIFT) {
                            return (UCHAR) (((UCHAR)-1)*(((((UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(UCHAR) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(UCHAR) - 1 > 2 * PyLong_SHIFT) {
                            return (UCHAR) ((((((UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(UCHAR) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(UCHAR, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(UCHAR) - 1 > 3 * PyLong_SHIFT) {
                            return (UCHAR) (((UCHAR)-1)*(((((((UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(UCHAR) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(UCHAR) - 1 > 3 * PyLong_SHIFT) {
                            return (UCHAR) ((((((((UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(UCHAR) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(UCHAR, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(UCHAR) - 1 > 4 * PyLong_SHIFT) {
                            return (UCHAR) (((UCHAR)-1)*(((((((((UCHAR)digits[3]) << PyLong_SHIFT) | (UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(UCHAR) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(UCHAR, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(UCHAR) - 1 > 4 * PyLong_SHIFT) {
                            return (UCHAR) ((((((((((UCHAR)digits[3]) << PyLong_SHIFT) | (UCHAR)digits[2]) << PyLong_SHIFT) | (UCHAR)digits[1]) << PyLong_SHIFT) | (UCHAR)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(UCHAR) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(UCHAR, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(UCHAR) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(UCHAR, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            UCHAR val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (UCHAR) -1;
        }
    } else {
        UCHAR val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (UCHAR) -1;
        val = __Pyx_PyInt_As_UCHAR(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to UCHAR");
    return (UCHAR) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to UCHAR");
    return (UCHAR) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE USHORT __Pyx_PyInt_As_USHORT(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const USHORT neg_one = (USHORT) -1, const_zero = (USHORT) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(USHORT) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(USHORT, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (USHORT) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (USHORT) 0;
                case  1: __PYX_VERIFY_RETURN_INT(USHORT, digit, digits[0])
                case 2:
                    if (8 * sizeof(USHORT) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(USHORT) >= 2 * PyLong_SHIFT) {
                            return (USHORT) (((((USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(USHORT) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(USHORT) >= 3 * PyLong_SHIFT) {
                            return (USHORT) (((((((USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(USHORT) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(USHORT) >= 4 * PyLong_SHIFT) {
                            return (USHORT) (((((((((USHORT)digits[3]) << PyLong_SHIFT) | (USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (USHORT) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(USHORT) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(USHORT, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(USHORT) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(USHORT, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (USHORT) 0;
                case -1: __PYX_VERIFY_RETURN_INT(USHORT, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(USHORT,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(USHORT) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(USHORT, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(USHORT) - 1 > 2 * PyLong_SHIFT) {
                            return (USHORT) (((USHORT)-1)*(((((USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(USHORT) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(USHORT) - 1 > 2 * PyLong_SHIFT) {
                            return (USHORT) ((((((USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(USHORT) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(USHORT, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(USHORT) - 1 > 3 * PyLong_SHIFT) {
                            return (USHORT) (((USHORT)-1)*(((((((USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(USHORT) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(USHORT) - 1 > 3 * PyLong_SHIFT) {
                            return (USHORT) ((((((((USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(USHORT) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(USHORT, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(USHORT) - 1 > 4 * PyLong_SHIFT) {
                            return (USHORT) (((USHORT)-1)*(((((((((USHORT)digits[3]) << PyLong_SHIFT) | (USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(USHORT) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(USHORT, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(USHORT) - 1 > 4 * PyLong_SHIFT) {
                            return (USHORT) ((((((((((USHORT)digits[3]) << PyLong_SHIFT) | (USHORT)digits[2]) << PyLong_SHIFT) | (USHORT)digits[1]) << PyLong_SHIFT) | (USHORT)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(USHORT) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(USHORT, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(USHORT) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(USHORT, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            USHORT val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (USHORT) -1;
        }
    } else {
        USHORT val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (USHORT) -1;
        val = __Pyx_PyInt_As_USHORT(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to USHORT");
    return (USHORT) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to USHORT");
    return (USHORT) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE GPIO_Dir __Pyx_PyInt_As_GPIO_Dir(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const GPIO_Dir neg_one = (GPIO_Dir) -1, const_zero = (GPIO_Dir) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GPIO_Dir) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GPIO_Dir, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GPIO_Dir) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GPIO_Dir) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GPIO_Dir, digit, digits[0])
                case 2:
                    if (8 * sizeof(GPIO_Dir) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Dir) >= 2 * PyLong_SHIFT) {
                            return (GPIO_Dir) (((((GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GPIO_Dir) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Dir) >= 3 * PyLong_SHIFT) {
                            return (GPIO_Dir) (((((((GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GPIO_Dir) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Dir) >= 4 * PyLong_SHIFT) {
                            return (GPIO_Dir) (((((((((GPIO_Dir)digits[3]) << PyLong_SHIFT) | (GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GPIO_Dir) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GPIO_Dir) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Dir, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GPIO_Dir) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Dir, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GPIO_Dir) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GPIO_Dir, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GPIO_Dir,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GPIO_Dir) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Dir, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Dir) - 1 > 2 * PyLong_SHIFT) {
                            return (GPIO_Dir) (((GPIO_Dir)-1)*(((((GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GPIO_Dir) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Dir) - 1 > 2 * PyLong_SHIFT) {
                            return (GPIO_Dir) ((((((GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GPIO_Dir) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Dir, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Dir) - 1 > 3 * PyLong_SHIFT) {
                            return (GPIO_Dir) (((GPIO_Dir)-1)*(((((((GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GPIO_Dir) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Dir) - 1 > 3 * PyLong_SHIFT) {
                            return (GPIO_Dir) ((((((((GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GPIO_Dir) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Dir, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Dir) - 1 > 4 * PyLong_SHIFT) {
                            return (GPIO_Dir) (((GPIO_Dir)-1)*(((((((((GPIO_Dir)digits[3]) << PyLong_SHIFT) | (GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GPIO_Dir) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Dir, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Dir) - 1 > 4 * PyLong_SHIFT) {
                            return (GPIO_Dir) ((((((((((GPIO_Dir)digits[3]) << PyLong_SHIFT) | (GPIO_Dir)digits[2]) << PyLong_SHIFT) | (GPIO_Dir)digits[1]) << PyLong_SHIFT) | (GPIO_Dir)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GPIO_Dir) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Dir, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GPIO_Dir) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Dir, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GPIO_Dir val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GPIO_Dir) -1;
        }
    } else {
        GPIO_Dir val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GPIO_Dir) -1;
        val = __Pyx_PyInt_As_GPIO_Dir(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GPIO_Dir");
    return (GPIO_Dir) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GPIO_Dir");
    return (GPIO_Dir) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE GPIO_Port __Pyx_PyInt_As_GPIO_Port(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const GPIO_Port neg_one = (GPIO_Port) -1, const_zero = (GPIO_Port) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GPIO_Port) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GPIO_Port, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GPIO_Port) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GPIO_Port) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GPIO_Port, digit, digits[0])
                case 2:
                    if (8 * sizeof(GPIO_Port) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Port) >= 2 * PyLong_SHIFT) {
                            return (GPIO_Port) (((((GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GPIO_Port) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Port) >= 3 * PyLong_SHIFT) {
                            return (GPIO_Port) (((((((GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GPIO_Port) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Port) >= 4 * PyLong_SHIFT) {
                            return (GPIO_Port) (((((((((GPIO_Port)digits[3]) << PyLong_SHIFT) | (GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GPIO_Port) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GPIO_Port) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Port, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GPIO_Port) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Port, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GPIO_Port) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GPIO_Port, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GPIO_Port,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GPIO_Port) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Port, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Port) - 1 > 2 * PyLong_SHIFT) {
                            return (GPIO_Port) (((GPIO_Port)-1)*(((((GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GPIO_Port) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Port) - 1 > 2 * PyLong_SHIFT) {
                            return (GPIO_Port) ((((((GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GPIO_Port) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Port, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Port) - 1 > 3 * PyLong_SHIFT) {
                            return (GPIO_Port) (((GPIO_Port)-1)*(((((((GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GPIO_Port) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Port) - 1 > 3 * PyLong_SHIFT) {
                            return (GPIO_Port) ((((((((GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GPIO_Port) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Port, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Port) - 1 > 4 * PyLong_SHIFT) {
                            return (GPIO_Port) (((GPIO_Port)-1)*(((((((((GPIO_Port)digits[3]) << PyLong_SHIFT) | (GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GPIO_Port) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Port, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Port) - 1 > 4 * PyLong_SHIFT) {
                            return (GPIO_Port) ((((((((((GPIO_Port)digits[3]) << PyLong_SHIFT) | (GPIO_Port)digits[2]) << PyLong_SHIFT) | (GPIO_Port)digits[1]) << PyLong_SHIFT) | (GPIO_Port)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GPIO_Port) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Port, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GPIO_Port) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Port, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GPIO_Port val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GPIO_Port) -1;
        }
    } else {
        GPIO_Port val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GPIO_Port) -1;
        val = __Pyx_PyInt_As_GPIO_Port(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GPIO_Port");
    return (GPIO_Port) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GPIO_Port");
    return (GPIO_Port) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_BOOL(BOOL value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const BOOL neg_one = (BOOL) -1, const_zero = (BOOL) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(BOOL) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(BOOL) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(BOOL) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(BOOL) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(BOOL) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(BOOL),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE GPIO_Trigger __Pyx_PyInt_As_GPIO_Trigger(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const GPIO_Trigger neg_one = (GPIO_Trigger) -1, const_zero = (GPIO_Trigger) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GPIO_Trigger) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GPIO_Trigger) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GPIO_Trigger) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GPIO_Trigger, digit, digits[0])
                case 2:
                    if (8 * sizeof(GPIO_Trigger) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Trigger) >= 2 * PyLong_SHIFT) {
                            return (GPIO_Trigger) (((((GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GPIO_Trigger) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Trigger) >= 3 * PyLong_SHIFT) {
                            return (GPIO_Trigger) (((((((GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GPIO_Trigger) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Trigger) >= 4 * PyLong_SHIFT) {
                            return (GPIO_Trigger) (((((((((GPIO_Trigger)digits[3]) << PyLong_SHIFT) | (GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GPIO_Trigger) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GPIO_Trigger) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Trigger, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GPIO_Trigger) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Trigger, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GPIO_Trigger) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GPIO_Trigger, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GPIO_Trigger,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GPIO_Trigger) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Trigger) - 1 > 2 * PyLong_SHIFT) {
                            return (GPIO_Trigger) (((GPIO_Trigger)-1)*(((((GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GPIO_Trigger) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Trigger) - 1 > 2 * PyLong_SHIFT) {
                            return (GPIO_Trigger) ((((((GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GPIO_Trigger) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Trigger) - 1 > 3 * PyLong_SHIFT) {
                            return (GPIO_Trigger) (((GPIO_Trigger)-1)*(((((((GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GPIO_Trigger) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Trigger) - 1 > 3 * PyLong_SHIFT) {
                            return (GPIO_Trigger) ((((((((GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GPIO_Trigger) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Trigger) - 1 > 4 * PyLong_SHIFT) {
                            return (GPIO_Trigger) (((GPIO_Trigger)-1)*(((((((((GPIO_Trigger)digits[3]) << PyLong_SHIFT) | (GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GPIO_Trigger) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GPIO_Trigger, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GPIO_Trigger) - 1 > 4 * PyLong_SHIFT) {
                            return (GPIO_Trigger) ((((((((((GPIO_Trigger)digits[3]) << PyLong_SHIFT) | (GPIO_Trigger)digits[2]) << PyLong_SHIFT) | (GPIO_Trigger)digits[1]) << PyLong_SHIFT) | (GPIO_Trigger)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GPIO_Trigger) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Trigger, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GPIO_Trigger) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GPIO_Trigger, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GPIO_Trigger val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GPIO_Trigger) -1;
        }
    } else {
        GPIO_Trigger val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GPIO_Trigger) -1;
        val = __Pyx_PyInt_As_GPIO_Trigger(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GPIO_Trigger");
    return (GPIO_Trigger) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GPIO_Trigger");
    return (GPIO_Trigger) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint16_t neg_one = (uint16_t) -1, const_zero = (uint16_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint16_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint16_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint16_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint16_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE uint16_t __Pyx_PyInt_As_uint16_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint16_t neg_one = (uint16_t) -1, const_zero = (uint16_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint16_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint16_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint16_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint16_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint16_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint16_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) >= 2 * PyLong_SHIFT) {
                            return (uint16_t) (((((uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint16_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) >= 3 * PyLong_SHIFT) {
                            return (uint16_t) (((((((uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint16_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) >= 4 * PyLong_SHIFT) {
                            return (uint16_t) (((((((((uint16_t)digits[3]) << PyLong_SHIFT) | (uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint16_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint16_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint16_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint16_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint16_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint16_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint16_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint16_t) (((uint16_t)-1)*(((((uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint16_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint16_t) ((((((uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint16_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint16_t) (((uint16_t)-1)*(((((((uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint16_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint16_t) ((((((((uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint16_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint16_t) (((uint16_t)-1)*(((((((((uint16_t)digits[3]) << PyLong_SHIFT) | (uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint16_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint16_t) ((((((((((uint16_t)digits[3]) << PyLong_SHIFT) | (uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint16_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint16_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint16_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint16_t) -1;
        }
    } else {
        uint16_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint16_t) -1;
        val = __Pyx_PyInt_As_uint16_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint16_t");
    return (uint16_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint16_t");
    return (uint16_t) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GPIO_Trigger(GPIO_Trigger value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const GPIO_Trigger neg_one = (GPIO_Trigger) -1, const_zero = (GPIO_Trigger) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GPIO_Trigger) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GPIO_Trigger) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GPIO_Trigger) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GPIO_Trigger) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GPIO_Trigger) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(GPIO_Trigger),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint32_t) -1;
        }
    } else {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyInt_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint8_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint8_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint8_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint8_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint8_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 2 * PyLong_SHIFT) {
                            return (uint8_t) (((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 3 * PyLong_SHIFT) {
                            return (uint8_t) (((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 4 * PyLong_SHIFT) {
                            return (uint8_t) (((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint8_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint8_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint8_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint8_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint8_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint8_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8_t) ((((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8_t) ((((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8_t) ((((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint8_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint8_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint8_t) -1;
        }
    } else {
        uint8_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint8_t) -1;
        val = __Pyx_PyInt_As_uint8_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint8_t");
    return (uint8_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint8_t");
    return (uint8_t) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint8_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint8_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint8_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint8_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE SPI_DrivingStrength __Pyx_PyInt_As_SPI_DrivingStrength(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const SPI_DrivingStrength neg_one = (SPI_DrivingStrength) -1, const_zero = (SPI_DrivingStrength) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(SPI_DrivingStrength) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (SPI_DrivingStrength) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (SPI_DrivingStrength) 0;
                case  1: __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, digit, digits[0])
                case 2:
                    if (8 * sizeof(SPI_DrivingStrength) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_DrivingStrength) >= 2 * PyLong_SHIFT) {
                            return (SPI_DrivingStrength) (((((SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(SPI_DrivingStrength) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_DrivingStrength) >= 3 * PyLong_SHIFT) {
                            return (SPI_DrivingStrength) (((((((SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(SPI_DrivingStrength) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_DrivingStrength) >= 4 * PyLong_SHIFT) {
                            return (SPI_DrivingStrength) (((((((((SPI_DrivingStrength)digits[3]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (SPI_DrivingStrength) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(SPI_DrivingStrength) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_DrivingStrength, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(SPI_DrivingStrength) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_DrivingStrength, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (SPI_DrivingStrength) 0;
                case -1: __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(SPI_DrivingStrength) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_DrivingStrength) - 1 > 2 * PyLong_SHIFT) {
                            return (SPI_DrivingStrength) (((SPI_DrivingStrength)-1)*(((((SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(SPI_DrivingStrength) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_DrivingStrength) - 1 > 2 * PyLong_SHIFT) {
                            return (SPI_DrivingStrength) ((((((SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(SPI_DrivingStrength) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_DrivingStrength) - 1 > 3 * PyLong_SHIFT) {
                            return (SPI_DrivingStrength) (((SPI_DrivingStrength)-1)*(((((((SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(SPI_DrivingStrength) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_DrivingStrength) - 1 > 3 * PyLong_SHIFT) {
                            return (SPI_DrivingStrength) ((((((((SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(SPI_DrivingStrength) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_DrivingStrength) - 1 > 4 * PyLong_SHIFT) {
                            return (SPI_DrivingStrength) (((SPI_DrivingStrength)-1)*(((((((((SPI_DrivingStrength)digits[3]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(SPI_DrivingStrength) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_DrivingStrength, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_DrivingStrength) - 1 > 4 * PyLong_SHIFT) {
                            return (SPI_DrivingStrength) ((((((((((SPI_DrivingStrength)digits[3]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[2]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[1]) << PyLong_SHIFT) | (SPI_DrivingStrength)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(SPI_DrivingStrength) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_DrivingStrength, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(SPI_DrivingStrength) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_DrivingStrength, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            SPI_DrivingStrength val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (SPI_DrivingStrength) -1;
        }
    } else {
        SPI_DrivingStrength val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (SPI_DrivingStrength) -1;
        val = __Pyx_PyInt_As_SPI_DrivingStrength(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to SPI_DrivingStrength");
    return (SPI_DrivingStrength) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to SPI_DrivingStrength");
    return (SPI_DrivingStrength) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE FT4222_SPIMode __Pyx_PyInt_As_FT4222_SPIMode(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_SPIMode neg_one = (FT4222_SPIMode) -1, const_zero = (FT4222_SPIMode) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(FT4222_SPIMode) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_SPIMode) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_SPIMode) 0;
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, digit, digits[0])
                case 2:
                    if (8 * sizeof(FT4222_SPIMode) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIMode) >= 2 * PyLong_SHIFT) {
                            return (FT4222_SPIMode) (((((FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_SPIMode) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIMode) >= 3 * PyLong_SHIFT) {
                            return (FT4222_SPIMode) (((((((FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_SPIMode) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIMode) >= 4 * PyLong_SHIFT) {
                            return (FT4222_SPIMode) (((((((((FT4222_SPIMode)digits[3]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_SPIMode) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(FT4222_SPIMode) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIMode, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_SPIMode) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIMode, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_SPIMode) 0;
                case -1: __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_SPIMode,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(FT4222_SPIMode) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIMode) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_SPIMode) (((FT4222_SPIMode)-1)*(((((FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(FT4222_SPIMode) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIMode) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_SPIMode) ((((((FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(FT4222_SPIMode) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIMode) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_SPIMode) (((FT4222_SPIMode)-1)*(((((((FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_SPIMode) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIMode) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_SPIMode) ((((((((FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(FT4222_SPIMode) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIMode) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_SPIMode) (((FT4222_SPIMode)-1)*(((((((((FT4222_SPIMode)digits[3]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_SPIMode) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIMode, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIMode) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_SPIMode) ((((((((((FT4222_SPIMode)digits[3]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[2]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[1]) << PyLong_SHIFT) | (FT4222_SPIMode)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(FT4222_SPIMode) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIMode, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_SPIMode) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIMode, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            FT4222_SPIMode val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (FT4222_SPIMode) -1;
        }
    } else {
        FT4222_SPIMode val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_SPIMode) -1;
        val = __Pyx_PyInt_As_FT4222_SPIMode(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_SPIMode");
    return (FT4222_SPIMode) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_SPIMode");
    return (FT4222_SPIMode) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE FT4222_SPIClock __Pyx_PyInt_As_FT4222_SPIClock(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_SPIClock neg_one = (FT4222_SPIClock) -1, const_zero = (FT4222_SPIClock) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(FT4222_SPIClock) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_SPIClock) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_SPIClock) 0;
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, digit, digits[0])
                case 2:
                    if (8 * sizeof(FT4222_SPIClock) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIClock) >= 2 * PyLong_SHIFT) {
                            return (FT4222_SPIClock) (((((FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_SPIClock) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIClock) >= 3 * PyLong_SHIFT) {
                            return (FT4222_SPIClock) (((((((FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_SPIClock) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIClock) >= 4 * PyLong_SHIFT) {
                            return (FT4222_SPIClock) (((((((((FT4222_SPIClock)digits[3]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_SPIClock) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(FT4222_SPIClock) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIClock, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_SPIClock) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIClock, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_SPIClock) 0;
                case -1: __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_SPIClock,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(FT4222_SPIClock) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIClock) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_SPIClock) (((FT4222_SPIClock)-1)*(((((FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(FT4222_SPIClock) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIClock) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_SPIClock) ((((((FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(FT4222_SPIClock) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIClock) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_SPIClock) (((FT4222_SPIClock)-1)*(((((((FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_SPIClock) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIClock) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_SPIClock) ((((((((FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(FT4222_SPIClock) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIClock) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_SPIClock) (((FT4222_SPIClock)-1)*(((((((((FT4222_SPIClock)digits[3]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_SPIClock) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPIClock, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPIClock) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_SPIClock) ((((((((((FT4222_SPIClock)digits[3]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[2]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[1]) << PyLong_SHIFT) | (FT4222_SPIClock)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(FT4222_SPIClock) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIClock, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_SPIClock) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPIClock, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            FT4222_SPIClock val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (FT4222_SPIClock) -1;
        }
    } else {
        FT4222_SPIClock val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_SPIClock) -1;
        val = __Pyx_PyInt_As_FT4222_SPIClock(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_SPIClock");
    return (FT4222_SPIClock) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_SPIClock");
    return (FT4222_SPIClock) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE FT4222_SPICPOL __Pyx_PyInt_As_FT4222_SPICPOL(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_SPICPOL neg_one = (FT4222_SPICPOL) -1, const_zero = (FT4222_SPICPOL) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(FT4222_SPICPOL) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_SPICPOL) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_SPICPOL) 0;
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, digit, digits[0])
                case 2:
                    if (8 * sizeof(FT4222_SPICPOL) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPOL) >= 2 * PyLong_SHIFT) {
                            return (FT4222_SPICPOL) (((((FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_SPICPOL) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPOL) >= 3 * PyLong_SHIFT) {
                            return (FT4222_SPICPOL) (((((((FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_SPICPOL) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPOL) >= 4 * PyLong_SHIFT) {
                            return (FT4222_SPICPOL) (((((((((FT4222_SPICPOL)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_SPICPOL) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(FT4222_SPICPOL) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPOL, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_SPICPOL) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPOL, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_SPICPOL) 0;
                case -1: __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(FT4222_SPICPOL) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPOL) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_SPICPOL) (((FT4222_SPICPOL)-1)*(((((FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(FT4222_SPICPOL) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPOL) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_SPICPOL) ((((((FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(FT4222_SPICPOL) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPOL) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_SPICPOL) (((FT4222_SPICPOL)-1)*(((((((FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_SPICPOL) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPOL) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_SPICPOL) ((((((((FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(FT4222_SPICPOL) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPOL) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_SPICPOL) (((FT4222_SPICPOL)-1)*(((((((((FT4222_SPICPOL)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_SPICPOL) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPOL, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPOL) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_SPICPOL) ((((((((((FT4222_SPICPOL)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPOL)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(FT4222_SPICPOL) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPOL, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_SPICPOL) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPOL, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            FT4222_SPICPOL val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (FT4222_SPICPOL) -1;
        }
    } else {
        FT4222_SPICPOL val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_SPICPOL) -1;
        val = __Pyx_PyInt_As_FT4222_SPICPOL(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_SPICPOL");
    return (FT4222_SPICPOL) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_SPICPOL");
    return (FT4222_SPICPOL) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE FT4222_SPICPHA __Pyx_PyInt_As_FT4222_SPICPHA(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const FT4222_SPICPHA neg_one = (FT4222_SPICPHA) -1, const_zero = (FT4222_SPICPHA) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(FT4222_SPICPHA) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT4222_SPICPHA) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_SPICPHA) 0;
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, digit, digits[0])
                case 2:
                    if (8 * sizeof(FT4222_SPICPHA) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPHA) >= 2 * PyLong_SHIFT) {
                            return (FT4222_SPICPHA) (((((FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_SPICPHA) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPHA) >= 3 * PyLong_SHIFT) {
                            return (FT4222_SPICPHA) (((((((FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_SPICPHA) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPHA) >= 4 * PyLong_SHIFT) {
                            return (FT4222_SPICPHA) (((((((((FT4222_SPICPHA)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT4222_SPICPHA) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(FT4222_SPICPHA) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPHA, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_SPICPHA) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPHA, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT4222_SPICPHA) 0;
                case -1: __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(FT4222_SPICPHA) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPHA) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_SPICPHA) (((FT4222_SPICPHA)-1)*(((((FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(FT4222_SPICPHA) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPHA) - 1 > 2 * PyLong_SHIFT) {
                            return (FT4222_SPICPHA) ((((((FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(FT4222_SPICPHA) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPHA) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_SPICPHA) (((FT4222_SPICPHA)-1)*(((((((FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT4222_SPICPHA) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPHA) - 1 > 3 * PyLong_SHIFT) {
                            return (FT4222_SPICPHA) ((((((((FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(FT4222_SPICPHA) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPHA) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_SPICPHA) (((FT4222_SPICPHA)-1)*(((((((((FT4222_SPICPHA)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT4222_SPICPHA) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT4222_SPICPHA, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT4222_SPICPHA) - 1 > 4 * PyLong_SHIFT) {
                            return (FT4222_SPICPHA) ((((((((((FT4222_SPICPHA)digits[3]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[2]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[1]) << PyLong_SHIFT) | (FT4222_SPICPHA)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(FT4222_SPICPHA) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPHA, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT4222_SPICPHA) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT4222_SPICPHA, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            FT4222_SPICPHA val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (FT4222_SPICPHA) -1;
        }
    } else {
        FT4222_SPICPHA val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT4222_SPICPHA) -1;
        val = __Pyx_PyInt_As_FT4222_SPICPHA(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT4222_SPICPHA");
    return (FT4222_SPICPHA) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT4222_SPICPHA");
    return (FT4222_SPICPHA) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE SPI_SlaveProtocol __Pyx_PyInt_As_SPI_SlaveProtocol(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const SPI_SlaveProtocol neg_one = (SPI_SlaveProtocol) -1, const_zero = (SPI_SlaveProtocol) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(SPI_SlaveProtocol) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (SPI_SlaveProtocol) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (SPI_SlaveProtocol) 0;
                case  1: __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, digit, digits[0])
                case 2:
                    if (8 * sizeof(SPI_SlaveProtocol) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_SlaveProtocol) >= 2 * PyLong_SHIFT) {
                            return (SPI_SlaveProtocol) (((((SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(SPI_SlaveProtocol) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_SlaveProtocol) >= 3 * PyLong_SHIFT) {
                            return (SPI_SlaveProtocol) (((((((SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(SPI_SlaveProtocol) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_SlaveProtocol) >= 4 * PyLong_SHIFT) {
                            return (SPI_SlaveProtocol) (((((((((SPI_SlaveProtocol)digits[3]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (SPI_SlaveProtocol) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(SPI_SlaveProtocol) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_SlaveProtocol, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(SPI_SlaveProtocol) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_SlaveProtocol, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (SPI_SlaveProtocol) 0;
                case -1: __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(SPI_SlaveProtocol) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_SlaveProtocol) - 1 > 2 * PyLong_SHIFT) {
                            return (SPI_SlaveProtocol) (((SPI_SlaveProtocol)-1)*(((((SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(SPI_SlaveProtocol) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_SlaveProtocol) - 1 > 2 * PyLong_SHIFT) {
                            return (SPI_SlaveProtocol) ((((((SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(SPI_SlaveProtocol) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_SlaveProtocol) - 1 > 3 * PyLong_SHIFT) {
                            return (SPI_SlaveProtocol) (((SPI_SlaveProtocol)-1)*(((((((SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(SPI_SlaveProtocol) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_SlaveProtocol) - 1 > 3 * PyLong_SHIFT) {
                            return (SPI_SlaveProtocol) ((((((((SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(SPI_SlaveProtocol) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_SlaveProtocol) - 1 > 4 * PyLong_SHIFT) {
                            return (SPI_SlaveProtocol) (((SPI_SlaveProtocol)-1)*(((((((((SPI_SlaveProtocol)digits[3]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(SPI_SlaveProtocol) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(SPI_SlaveProtocol, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(SPI_SlaveProtocol) - 1 > 4 * PyLong_SHIFT) {
                            return (SPI_SlaveProtocol) ((((((((((SPI_SlaveProtocol)digits[3]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[2]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[1]) << PyLong_SHIFT) | (SPI_SlaveProtocol)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(SPI_SlaveProtocol) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_SlaveProtocol, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(SPI_SlaveProtocol) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(SPI_SlaveProtocol, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            SPI_SlaveProtocol val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (SPI_SlaveProtocol) -1;
        }
    } else {
        SPI_SlaveProtocol val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (SPI_SlaveProtocol) -1;
        val = __Pyx_PyInt_As_SPI_SlaveProtocol(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to SPI_SlaveProtocol");
    return (SPI_SlaveProtocol) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to SPI_SlaveProtocol");
    return (SPI_SlaveProtocol) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
